package com.ibm.etools.beaninfo.adapters;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.io.FileNotFoundException;
import java.lang.ref.WeakReference;
import java.text.MessageFormat;
import java.util.*;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.ibm.etools.beaninfo.*;
import com.ibm.etools.emf.diff.MofDMTool;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ecore.impl.ESuperAdapter;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.java.*;
import com.ibm.etools.java.adapters.ReflectionAdaptor;
import com.ibm.etools.java.beaninfo.BeaninfoSuperAdapter;
import com.ibm.etools.java.beaninfo.IIntrospectionAdapter;
import com.ibm.etools.java.impl.JavaClassImpl;
import com.ibm.etools.logging.util.Level;
import com.ibm.etools.proxy.*;

/**
 * Beaninfo adapter for doing introspection on a Java Model class.
 */

public class BeaninfoClassAdapter extends AdapterImpl implements IIntrospectionAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	/**
	 * @version 	1.0
	 * @author
	 */

	protected boolean fHasIntrospected;
	protected boolean fIsIntrospecting;
	
	protected boolean fHasIntrospectedProperties;
	protected boolean fIsIntrospectingProperties;
	
	protected boolean fIsDoingAllAttributes;
	protected boolean fIsDoingAllReferences;	
	
	protected boolean fHasIntrospectedBehaviors;
	protected boolean fIsIntrospectingBehaviors;
	
	protected boolean fIsDoingAllBehaviors;
	
	protected boolean fHasIntrospectedEvents;
	protected boolean fIsIntrospectingEvents;	
	
	protected boolean fIsDoingAllEvents;	
	
	protected boolean fRetrievedExtensionDocument;
	
	protected IBeanProxy fBeaninfo;
	protected BeaninfoAdapterFactory fFactory;
	
	protected static EStructuralFeature 
		sEAttributesSF,
		sEReferencesSF,
		sEBehaviorsSF,
		sEventsSF,
		sESuperSF;
	
	private WeakReference fStaleFactory;	// When reference not null, then this factory is a stale factory and 
						// a new one is needed when the factory returned == this one.
						// It is a WeakRef so that if the factory goes away on its own
						// that we don't hold onto it.
	
	// A temporary hashset of the local properties. Used when creating a new
	// property to use the old one. It is cleared out at the end of attribute introspection.
	// It is only built once during attribute introspection so that we have the snapshot
	// of before the introspection (i.e. the ones that were specifically added by users
	// and not through introspection). The ones that we are adding do not need to be checked
	// against each other because there will not be conflicts.
	//
	// Also at the end, we will go through the properties and see if it exists in the fPropertiesMap and
	// it is not the value Boolean.FALSE in the map, and the entry is implicitly created. This means this
	// was an implicit entry from the previous introspection and was not re-created in this introspection.
	private HashMap fPropertiesMap;
	private List fAttributesRealList;	// Temp pointer to the real list we are building. It is the true list in java class.
	
	// A temporary hashset of the local behaviors. Used when creating a new
	// behavior to use the old one. It is cleared out at the end of behavior introspection.
	// It is only built once during behavior introspection so that we have the snapshot
	// of before the introspection (i.e. the ones that were specifically added by users
	// and not through introspection). The ones that we are adding do not need to be checked
	// against each other because there will not be conflicts.
	//
	// Also at the end, we will go through the behaviors and see if it exists in the fBehaviorsMap and
	// it is not the value Boolean.FALSE in the map, and the entry is implicitly created. This means this
	// was an implicit entry from the previous introspection and was not re-created in this introspection.
	private HashMap fBehaviorsMap;
	private List fBehaviorsRealList;	// Temp pointer to the real list we are building. It is the true list in java class.
	private HashSet fNewBehaviors;	// A set of behaviors names as we create them so that we don't create a duplicate.

	// A temporary hashset of the local events. Used when creating a new
	// event to use the old one. It is cleared out at the end of event introspection.
	// It is only built once during event introspection so that we have the snapshot
	// of before the introspection (i.e. the ones that were specifically added by users
	// and not through introspection). The ones that we are adding do not need to be checked
	// against each other because there will not be conflicts.
	//
	// Also at the end, we will go through the events and see if it exists in the fEventsMap and
	// it is not the value Boolean.FALSE in the map, and the entry is implicitly created. This means this
	// was an implicit entry from the previous introspection and was not re-created in this introspection.
	private HashMap fEventsMap;
	private List fEventsRealList;	// Temp pointer to the real list we are building. It is the true list in java class.
	
	public BeaninfoClassAdapter(BeaninfoAdapterFactory factory) {
		super();
		fFactory = factory;
		
		if (sEAttributesSF == null) {
			// initialize the features we need to access in JavaClass.
			EcorePackage pkg = (EcorePackage) RefRegister.getPackage(EcorePackage.packageURI);
			sEAttributesSF = pkg.getEDataStructure_EAttributes();
			sEReferencesSF = pkg.getEStructure_EReferences();			
			sEBehaviorsSF = pkg.getEType_EBehaviors();
			sESuperSF = pkg.getEGeneralizableElement_ESuper();
			
			JavaRefPackage jpkg = (JavaRefPackage) RefRegister.getPackage(JavaRefPackage.packageURI);
			sEventsSF = jpkg.getJavaClass_Events();
		}
	}
	
	protected final ProxyFactoryRegistry getRegistry() {
		ProxyFactoryRegistry factory = fFactory.getRegistry();
		if (fStaleFactory != null && factory == fStaleFactory.get()) {
			// We need to recycle the factory. The returned factory is the same factory when it went stale.
			factory = fFactory.recycleRegistry();
		}
		fStaleFactory = null;	// Whether we recycled or not, it is no longer stale.
		return factory;
	}
	
	/**
	 * Return whether this adapter has been marked as stale. Needed
	 * by the users so that they can recycle if necessary.
	 */
	public boolean isStale() {
		return fStaleFactory != null;
	}
		
	
	protected BeaninfoAdapterFactory getAdapterFactory() {
		return fFactory;
	}
	
	public boolean isAdapterForType(Object key) {
		return IIntrospectionAdapter.ADAPTER_KEY.equals(key);
	}

	/**
	 * This map is keyed by name. It is a snapshot of the properties at the
	 * time the introspection/reflection of properties was started. It is used
	 * for quick lookup.
	 *
	 * Once a property is used, the entry is replaced with a Boolean.FALSE. This
	 * is so we know which have already been used and at the end, which need
	 * to be deleted since they weren't used (i.e. the ones that aren't FALSE).
	 */
	protected HashMap getPropertiesMap() {
		if (fPropertiesMap == null) {
			List localAttrs = (List) getJavaClass().primRefValue(sEAttributesSF);
			List localRefs = (List) getJavaClass().primRefValue(sEReferencesSF);
			fPropertiesMap = new HashMap(localAttrs.size() + localRefs.size());
			Iterator itr = localAttrs.iterator();
			while (itr.hasNext()) {
				EAttribute attr = (EAttribute) itr.next();
				fPropertiesMap.put(attr.getName(), attr);
			}
			itr = localRefs.iterator();
			while (itr.hasNext()) {
				EReference ref = (EReference) itr.next();
				fPropertiesMap.put(ref.getName(), ref);
			}			
		}
		return fPropertiesMap;		
	}
	
	/**
	 * Get it once so that we don't need to keep getting it over and over.
	 */
	protected List getAttributesList() {
		if (fAttributesRealList == null)
			fAttributesRealList = (List) getJavaClass().primRefValue(sEAttributesSF);
		return fAttributesRealList;
	}
	
	/**
	 * The map is keyed by name. If a Method is passed in, then the
	 * id of the method is used (this is in reflection), if an IBeanProxy
	 * is passed in, then an id is created and looked up (this is in introspection).
	 * The map is used for a quick lookup of behaviors at the time introspection
	 * of behaviors started.
	 *
	 * Once a behavior is used, the entry is replaced with a Boolean.FALSE. This
	 * is so we know which have already been used and at the end, which need
	 * to be deleted since they weren't used (i.e. the ones that aren't FALSE).
	 */
	protected HashMap getBehaviorsMap() {
		if (fBehaviorsMap == null) {
			List locals = ((List) getJavaClass().primRefValue(sEBehaviorsSF));
			fBehaviorsMap = new HashMap(locals.size());
			Iterator itr = locals.iterator();
			while (itr.hasNext()) {
				EBehavioralFeature bhav = (EBehavioralFeature) itr.next();
				fBehaviorsMap.put(bhav.getName(), bhav);
			}
		}
		return fBehaviorsMap;		
	}
		
	/**
	 * Get it once so that we don't need to keep getting it over and over.
	 */	
	protected List getBehaviorsList() {
		if (fBehaviorsRealList == null)
			fBehaviorsRealList = (List) getJavaClass().primRefValue(sEBehaviorsSF);
		return fBehaviorsRealList;
	}	
	
	/**
	 * The map is keyed by name.
	 * The map is used for a quick lookup of events at the time introspection
	 * of events started.
	 *
	 * Once an event is used, the entry is replaced with a Boolean.FALSE. This
	 * is so we know which have already been used and at the end, which need
	 * to be deleted since they weren't used (i.e. the ones that aren't FALSE).
	 */
	protected HashMap getEventsMap() {
		if (fEventsMap == null) {
			List locals = ((List) getJavaClass().primRefValue(sEventsSF));
			fEventsMap = new HashMap(locals.size());
			Iterator itr = locals.iterator();
			while (itr.hasNext()) {
				JavaEvent event = (JavaEvent) itr.next();
				fEventsMap.put(event.getName(), event);
			}
		}
		return fEventsMap;		
	}
	
	/**
	 * Get it once so that we don't need to keep getting it over and over.
	 */	
	protected List getEventsList() {
		if (fEventsRealList == null)
			fEventsRealList = (List) getJavaClass().primRefValue(sEventsSF);
		return fEventsRealList;
	}		

	public void introspectIfNecessary() {
		if (!fHasIntrospected && !fIsIntrospecting) {
			try {
				introspect();
			} catch (Throwable e) {
				fHasIntrospected = false;
				BeaninfoPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, BeaninfoPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, MessageFormat.format(BeaninfoPlugin.getPlugin().getDescriptor().getResourceString(BeaninfoProperties.INTROSPECTFAILED), new Object[] {getJavaClass().getJavaName()}), e));
			}
		}
	}
	
	public void introspect() {
		fIsIntrospecting = true;
		try {
			// See if are valid kind of class.
			if (getJavaClass().getValueKind() == TypeKind.UNDEFINED) {
				// Not valid, don't let any further introspection occur.
				clearIntrospection(); // Clear out anything introspection has added.
				fHasIntrospected = fHasIntrospectedBehaviors = fHasIntrospectedProperties = fHasIntrospectedEvents = true;
			} else {
				if (!fRetrievedExtensionDocument)
					applyExtensionDocument();	// Apply the extension doc before we do anything.
				BeanDecorator decor = Utilities.getBeanDecorator(getJavaClass());
				if (decor == null || decor.isDoBeaninfo()) {
					IBeanTypeProxy targetType = null;
					ProxyFactoryRegistry registry = getRegistry();
					if (registry != null && registry.isValid())
						targetType = registry.getBeanTypeProxyFactory().getBeanTypeProxy(
							getJavaClass().getQualifiedNameForReflection());
					if (targetType != null) {
						if (targetType.getInitializationError() == null) {
							// If an exception is thrown, treat this as no proxy, however log it because we
							// shouldn't have exceptions during introspection, but if we do it should be logged
							// so it can be corrected.
							try {
								fBeaninfo =
									getProxyConstants().getIntrospectProxy().invoke(
										null,
										new IBeanProxy[] { targetType, getRegistry().getBeanProxyFactory().createBeanProxyWith(false)});
							} catch (ThrowableProxy e) {
								BeaninfoPlugin.getPlugin().getMsgLogger().write(Level.WARNING,
									new Status(
										IStatus.WARNING,
										BeaninfoPlugin.getPlugin().getDescriptor().getUniqueIdentifier(),
										0,
										MessageFormat.format(
											BeaninfoPlugin.getPlugin().getDescriptor().getResourceString(BeaninfoProperties.INTROSPECTFAILED),
											new Object[] { getJavaClass().getJavaName()}),
										e));
							}
						} else {
							// The class itself couldn't be initialized. Just log it, but treat as no proxy.
							BeaninfoPlugin.getPlugin().getMsgLogger().write(Level.WARNING,
								new Status(
									IStatus.WARNING,
									BeaninfoPlugin.getPlugin().getDescriptor().getUniqueIdentifier(),
									0,
									MessageFormat.format(
										BeaninfoPlugin.getPlugin().getDescriptor().getResourceString(BeaninfoProperties.INTROSPECTFAILED),
										new Object[] { getJavaClass().getJavaName()}),
									new ExceptionInInitializerError(targetType.getInitializationError())));
						}
					} else {
						// The class itself could not be found. Just log it, but treat as no proxy.
						BeaninfoPlugin.getPlugin().getMsgLogger().write(Level.WARNING,
							new Status(
								IStatus.WARNING,
								BeaninfoPlugin.getPlugin().getDescriptor().getUniqueIdentifier(),
								0,
								MessageFormat.format(
									BeaninfoPlugin.getPlugin().getDescriptor().getResourceString(BeaninfoProperties.INTROSPECTFAILED),
									new Object[] { getJavaClass().getJavaName()}), null));
					}	
				}
				calculateBeanDescriptor(decor);
				fHasIntrospected = true;
			}
			getAdapterFactory().registerIntrospection(getJavaClass().getQualifiedNameForReflection(), this);
		} finally {
			fIsIntrospecting = false;
		}
	}	
	
	protected void applyExtensionDocument() {
		fRetrievedExtensionDocument = true;
		BeaninfoPlugin.OverridePathSearch searcher = BeaninfoPlugin.getPlugin().getOverrideSearch(getJavaClass().getJavaPackage().getPackageName());
		for (String[] paths = searcher.getNextPath(); paths != null; paths = searcher.getNextPath()) {
			
			ResourceSet rset = getJavaClass().getResourceSet();
			String overridepath = searcher.getUnmatchedPath();
			String overridefile = getJavaClass().getName().replace('.', '$') + ".xmi";	// Turn inner classes from dot notation to file format. //$NON-NLS-1$
			if (overridepath.length() > 0)
				overridefile = overridepath + '/' + overridefile;
			for (int i = 0; i < paths.length; i++) {
				String filename = paths[i] + overridefile;								
				try {				
					Resource diffRes = rset.load(filename);
							
					MofDMTool diffTool = new MofDMTool();
					diffTool.mergingWithDiffFile(diffRes);
					rset.remove(diffRes);	// We don't need it around once we do the merge.
				} catch (FileNotFoundException e) {
					// This is ok. Normally not found.
				} catch (Exception e) {
					// Couldn't load it for some reason.
					BeaninfoPlugin.getPlugin().getMsgLogger().write(Level.WARNING,new Status(IStatus.WARNING, BeaninfoPlugin.PI_BEANINFO, 0, "Error loading file\""+filename+"\"", e)); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
	}
	
	/**
	 * Clear out the introspection because introspection is being closed or removed from the project.
	 * Don't want anything hanging around that we had done.
	 */
	public void clearIntrospection() {
		if (fBeaninfo != null) {
			fBeaninfo = null;
		}
		if (fHasIntrospected) {
			// Clear out the beandecorator if implicitly created.
			Iterator beanItr = getJavaClass().getEDecorators().iterator();
			while (beanItr.hasNext()) {
				EDecorator dec = (EDecorator) beanItr.next();
				if (dec instanceof BeanDecorator) {
					BeanDecorator decor = (BeanDecorator) dec;
					decor.setDescriptorProxy(null);	
					decor.setDecoratorProxy(null);
					if (decor.isImplicitlyCreated() == BeanDecorator.IMPLICIT_DECORATOR) {
						beanItr.remove();
						decor.refDelete();	// Mark it as deleted so we know it is no longer any use.
						break;
					}
				}
			}
		}
				
		if (fHasIntrospectedProperties) {
			// Clear out the attributes that we implicitly created.
			Iterator propItr = ((List) getJavaClass().primRefValue(sEAttributesSF)).iterator();
			while (propItr.hasNext()) {
				EAttribute prop = (EAttribute) propItr.next();
				Iterator pdItr = prop.getEDecorators().iterator();
				while (pdItr.hasNext()) {
					EDecorator dec = (EDecorator) pdItr.next();
					if (dec instanceof PropertyDecorator) {
						PropertyDecorator pd = (PropertyDecorator) dec;
						pd.setDescriptorProxy(null);
						pd.setDecoratorProxy(null);
						if (pd.isImplicitlyCreated() != PropertyDecorator.NOT_IMPLICIT) {
							pdItr.remove();	// Remove it from the property.
							pd.refDelete();	// Mark it as deleted so we know it is no longer any use.
						}
						if (pd.isImplicitlyCreated() == PropertyDecorator.IMPLICIT_DECORATOR_AND_FEATURE) {
							propItr.remove();	// Remove the feature itself
							prop.refDelete();
						}
						break;
					}
				}
			}
				
			// Clear out the references that we implicitly created. Only the decorators can be implicit.
			propItr = ((List) getJavaClass().primRefValue(sEReferencesSF)).iterator();
			while (propItr.hasNext()) {
				EReference prop = (EReference) propItr.next();
				Iterator pdItr = prop.getEDecorators().iterator();
				while (pdItr.hasNext()) {
					EDecorator dec = (EDecorator) pdItr.next();
					if (dec instanceof PropertyDecorator) {
						PropertyDecorator pd = (PropertyDecorator) dec;
						pd.setDescriptorProxy(null);
						pd.setDecoratorProxy(null);
						if (pd.isImplicitlyCreated() != PropertyDecorator.NOT_IMPLICIT) {
							pdItr.remove();	// Remove it from the property decorator.
							pd.refDelete();	// Mark it as deleted so we know it is no longer any use.
						}
						break;
					}
				}
				
			}

			fHasIntrospectedProperties = false;
		}
		
		if (fHasIntrospectedBehaviors) {
			// Clear out the behaviors that we implicitly created.
			Iterator bhavItr = ((List) getJavaClass().primRefValue(sEBehaviorsSF)).iterator();
			while (bhavItr.hasNext()) {
				EBehavioralFeature bHav = (EBehavioralFeature) bhavItr.next();
				Iterator mdItr = bHav.getEDecorators().iterator();
				while (mdItr.hasNext()) {
					EDecorator dec = (EDecorator) mdItr.next();
					if (dec instanceof MethodDecorator) {
						MethodDecorator md = (MethodDecorator) dec;
						md.setDescriptorProxy(null);
						md.setDecoratorProxy(null);
						if (md.isImplicitlyCreated() != MethodDecorator.NOT_IMPLICIT) {
							mdItr.remove();	// Remove it from the behavior.
							md.refDelete();	// Mark it as deleted so we know it is no longer any use.
						}
						if (md.isImplicitlyCreated() == MethodDecorator.IMPLICIT_DECORATOR_AND_FEATURE) {
							bhavItr.remove();	// Remove the behavior itself
							bHav.refDelete();
						}
						break;
					}
				}
			}
			
			fHasIntrospectedBehaviors = false;
		}
	}	
	
	/**
	 * Return the target as a JavaClass 
	 */
	protected JavaClass	getJavaClass() {
		return (JavaClass) getTarget();
	}
	
	/**
	 * Answer the beaninfo constants record
	 */
	protected BeaninfoProxyConstants getProxyConstants() {
		return BeaninfoProxyConstants.getConstants(getRegistry());
	}

	/**
	 * Get the value for the feature passed in. Introspect if necessary,
	 * and then do further calculations only when the request feature is
	 * first called. Some features are too heavy to do just because of 
	 * the class being introspected. Finally, return the value from the
	 * class itself since the introspection filled it in.
	 */
	public Object getValue(RefObject feature) {
		if (feature instanceof EFeature) {
			EFeature eFeature = (EFeature) feature;
			// Test for the feature and do further calculations if needed.
			switch (getJavaClass().eClassJavaClass().getEFeatureId(eFeature)) {
				case JavaRefPackage.JAVA_CLASS__E_ATTRIBUTES :
				case JavaRefPackage.JAVA_CLASS__E_REFERENCES :
					introspectProperties();
					break;
				case JavaRefPackage.JAVA_CLASS__E_ALL_ATTRIBUTES :
					allAttributes(feature);
					break;
				case JavaRefPackage.JAVA_CLASS__E_ALL_REFERENCES :
					allReferences(feature);
					break;
				case JavaRefPackage.JAVA_CLASS__E_BEHAVIORS :
					introspectBehaviors();
					break;
				case JavaRefPackage.JAVA_CLASS__E_ALL_BEHAVIORS :
					allBehaviors(feature);
					break;
				case JavaRefPackage.JAVA_CLASS__EVENTS:
					introspectEvents();
					break;
				case JavaRefPackage.JAVA_CLASS__ALL_EVENTS :
					allEvents(feature);
					break;					
			}
		}
		return getJavaClass().primRefValue(feature);
	}
	
	/**
	 * Fill in the BeanDescriptorDecorator.
	 */
	protected void calculateBeanDescriptor(BeanDecorator decor) {
		// If there already is one, then we
		// will use it. This allows merging with beanfinfo.
		// If this was an implicit one left over from a previous introspection before going stale,
		// this is OK because we always want a bean decorator from introspection. We will reuse it.

		if (decor == null) {
			decor = createBeanDecorator();
			decor.setImplicitlyCreated(BeanDecorator.IMPLICIT_DECORATOR);
			getJavaClass().getEDecorators().add(decor);
		}
		
		if (fBeaninfo != null && decor.isMergeIntrospection()) {
			decor.setDescriptorProxy(getProxyConstants().getBeanDescriptorProxy().invokeCatchThrowableExceptions(fBeaninfo));
			decor.setMergeSuperPropertiesProxy(((IBooleanBeanProxy) getProxyConstants().getIsMergeSuperPropertiesProxy().invokeCatchThrowableExceptions(fBeaninfo)).getBooleanValue());			
			decor.setMergeSuperBehaviorsProxy(((IBooleanBeanProxy) getProxyConstants().getIsMergeSuperMethodsProxy().invokeCatchThrowableExceptions(fBeaninfo)).getBooleanValue());			
		} else {
			decor.setDescriptorProxy(null);
			decor.setMergeSuperEventsProxy(null);
			decor.setMergeSuperPropertiesProxy(null);
			decor.setMergeSuperBehaviorsProxy(null);
		}
		decor.setDecoratorProxy(null);
	}

	/**
	 * introspect the Properties
	 */
	protected void introspectProperties() {
		introspectIfNecessary();			
		if (!fIsIntrospecting && !fIsIntrospectingProperties && !fHasIntrospectedProperties) {
			fIsIntrospectingProperties = true;							
			try {		
				BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
				if (bd == null || bd.isIntrospectProperties()) {
					// bd wants properties to be introspected/reflected
					if (fBeaninfo != null) {
						IArrayBeanProxy props = (IArrayBeanProxy) getProxyConstants().getPropertyDescriptorsProxy().invokeCatchThrowableExceptions(fBeaninfo);
						if (props != null) {
							int propSize = props.getLength();
							for (int i=0; i<propSize; i++)
								try {
									calculateProperty(props.get(i));
								} catch (ThrowableProxy e) {
								}
						}
					} else
						reflectProperties();	// No beaninfo, so use reflection to create properties
				}
				
				// Now go through the list and remove those that should be removed.
				Map oldLocals = getPropertiesMap();
				Iterator itr = getAttributesList().iterator();
				while (itr.hasNext()) {
					EStructuralFeature a = (EStructuralFeature) itr.next();
					PropertyDecorator p = Utilities.getPropertyDecorator(a);
					Object aOld = oldLocals.get(a.getName());
					if (aOld != null && aOld != Boolean.FALSE) {
						// A candidate for removal. It was in the old list and it was not processed.
						if (p != null) {
							int implicit = p.isImplicitlyCreated();
							if (implicit != PropertyDecorator.NOT_IMPLICIT) {
								p.setEDecorates(null);	// Remove from the feature;
								p.refDelete();	// Mark it as deleted.
								p = null;
							}
							
							if (implicit == PropertyDecorator.IMPLICIT_DECORATOR_AND_FEATURE) {
								itr.remove();	// Remove it, this was implicitly created and not processed this time.
								a.refDelete();
							}
						}
					}
					
					if (p != null)
						p.refreshJavaReferences();
				}
				
				// Now go through the references to remove any old property decorators. Since names must be unique between Attributes AND References we can use the same map.
				// We will not remove the reference itself since they are always explicit.
				itr = ((List) getJavaClass().primRefValue(sEReferencesSF)).iterator();
				while (itr.hasNext()) {
					EStructuralFeature a = (EStructuralFeature) itr.next();
					PropertyDecorator p = Utilities.getPropertyDecorator(a);
					Object aOld = oldLocals.get(a.getName());
					if (aOld != null && aOld != Boolean.FALSE) {
						// A candidate for removal. It was in the old list and it was not processed.
						if (p != null && p.isImplicitlyCreated() == PropertyDecorator.IMPLICIT_DECORATOR) {
							p.setEDecorates(null);	// Remove it, this was implicitly created and not processed this time.
							p.refDelete();
							p = null;
						}
					}
					
					if (p != null)
						p.refreshJavaReferences();
				}				
				fHasIntrospectedProperties = true;
			} finally {
				fIsIntrospectingProperties = false;
				fPropertiesMap = null;	// Get rid of accumulated map.
				fAttributesRealList = null;	// Release the real list.
			}
		}
	}
	
	/**
	 * merge all  the Attributes (i.e. supertypes) (properties)
	 */
	protected void allAttributes(RefObject allAttrSF) {
		if (!fIsIntrospecting && !fIsDoingAllAttributes) {
			fIsDoingAllAttributes = true;							
			try {
				introspectIfNecessary();
				EList allAttributes = (EList) getJavaClass().primRefValue(allAttrSF);
				allAttributes.clear();	
				EList localAttributes = getJavaClass().getEAttributes();	// Explicitly requesting possible introspection here because it may not have been done yet.
				EClass superType = getJavaClass().getSupertype();
				if (superType != null) {				
					// Now we need to merge in the supers.
					BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
					
					// We will now merge in either all supers, or if told not to merge. If not merge, then only those without 
					// property descriptors.
					boolean mergeAll = bd == null || bd.isMergeSuperProperties();
					// We have set of all the names added so that we won't add already added properties, by name.
					HashSet keys = new HashSet(100);
					Iterator itr = localAttributes.iterator();
					while (itr.hasNext()) {
						keys.add(((EStructuralFeature) itr.next()).getName());
					}						

					Iterator supers = superType.getEAllAttributes().iterator();
					while (supers.hasNext()) {
						EStructuralFeature attr = (EStructuralFeature) supers.next();
						PropertyDecorator pd = null;
						if (!mergeAll)
							pd = Utilities.getPropertyDecorator(attr);
						// Add only if all of the following are true:
						//   1) The feature's name is not the same as one of locals (so that we can override if they are the same).	
						//   2) If any of the following are true:
						//      a) We want all beaninfos.
						//      b) There is no property decorator (this means it was added through an XMI file and should show up)
						if (!keys.contains(attr.getName()) && (mergeAll || pd == null)) {
							keys.add(attr.getName());
							allAttributes.add(attr);
						}
					}
				}
				allAttributes.addAll(localAttributes);
				ESuperAdapter sa = (ESuperAdapter) getTarget().getAdapter(IESuperAdapter.ADAPTER_KEY);
				sa.setAllAttributesCollectionModified(false);	// Now built, so reset to not changed.
			} finally {
				fIsDoingAllAttributes = false;
			}
		}
	}
	
	/**
	 * merge all  the References (i.e. supertypes) (properties)
	 */
	protected void allReferences(RefObject allRefSF) {
		if (!fIsIntrospecting && !fIsDoingAllReferences) {
			fIsDoingAllReferences = true;							
			try {
				introspectIfNecessary();
				EList allReferences = (EList) getJavaClass().primRefValue(allRefSF);
				allReferences.clear();	
				EList localReferences = getJavaClass().getEReferences();	// Explicitly want to possibly trigger introspection.
				EClass superType = getJavaClass().getSupertype();
				if (superType != null) {				
					// Now we need to merge in the supers.
					BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
					// We will now merge in either all supers, or if told not to merge. If not merge, then only those without 
					// property descriptors.
					boolean mergeAll = bd == null || bd.isMergeSuperProperties();
					// We have set of all the names added so that we won't add already added properties, by name.
					HashSet keys = new HashSet(100);
					Iterator itr = localReferences.iterator();
					while (itr.hasNext()) {
						keys.add(((EStructuralFeature) itr.next()).getName());
					}						
					Iterator supers = superType.getEAllReferences().iterator();
					while (supers.hasNext()) {
						EStructuralFeature ref = (EStructuralFeature) supers.next();
						PropertyDecorator pd = null;
						if (!mergeAll)
							pd = Utilities.getPropertyDecorator(ref);
						// Add only if all of the following are true:
						//   1) The feature's name is not the same as one of locals (so that we can override if they are the same).	
						//   2) If any of the following are true:
						//      a) We want all beaninfos.
						//      b) There is no property decorator (this means it was added through an XMI file and should show up)
						if (!keys.contains(ref.getName()) && (mergeAll || pd == null)) {
							keys.add(ref.getName());
							allReferences.add(ref);
						}
					}
				}
				allReferences.addAll(localReferences);
				ESuperAdapter sa = (ESuperAdapter) getTarget().getAdapter(IESuperAdapter.ADAPTER_KEY);
				sa.setAllReferencesCollectionModified(false);	// Now built, so reset to not changed.
			} finally {
				fIsDoingAllReferences = false;
			}
		}
	}	
		
	
	/**
	 * Fill in the property and its decorator using the propDesc.
	 */
	protected void calculateProperty(IBeanProxy propDesc) {		
		// See if this is an indexed property. If it is, then a few fields will not be set in here, but
		// will instead be set by the calculateIndexedProperty, which will be called.
		boolean indexed = propDesc.getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy("java.beans.IndexedPropertyDescriptor").equals(propDesc.getTypeProxy()); //$NON-NLS-1$
		String name = ((IStringBeanProxy) getProxyConstants().getNameProxy().invokeCatchThrowableExceptions(propDesc)).stringValue();
		boolean changeable;
		EClassifier type = null;
		if (!indexed) {
			changeable = getProxyConstants().getWriteMethodProxy().invokeCatchThrowableExceptions(propDesc) != null;
			IBeanTypeProxy typeProxy = (IBeanTypeProxy) getProxyConstants().getPropertyTypeProxy().invokeCatchThrowableExceptions(propDesc);
			if (typeProxy != null)
				type = Utilities.getJavaClass(typeProxy, getJavaClass().refResource().getResourceSet());
			
		} else {
			changeable = getProxyConstants().getIndexedWriteMethodProxy().invokeCatchThrowableExceptions(propDesc) != null;
			IBeanTypeProxy typeProxy = (IBeanTypeProxy) getProxyConstants().getIndexedPropertyTypeProxy().invokeCatchThrowableExceptions(propDesc);
			if (typeProxy != null)
				type = Utilities.getJavaClass(typeProxy, getJavaClass().refResource().getResourceSet());
			
		}
		
		if (type != null)	
			createProperty(name, indexed, changeable, type, propDesc);	// A valid property descriptor.
	}
	
	/**
	 * Fill in the property and its decorator using the passed in information.
	 */
	public PropertyDecorator createProperty(String name, boolean indexed, boolean changeable, EClassifier type, IBeanProxy propDesc) {	
		// First find if there is already a property of this name, and if there is, is the PropertyDecorator
		// marked to not allow merging in of introspection results.		
		HashMap existingLocals = getPropertiesMap();
		EStructuralFeature prop = null;
		PropertyDecorator pd = null;
		Object p = existingLocals.get(name);
		if (Boolean.FALSE == p)
			return null;	// We've already processed this name, can't process it again.
		if (p != null) {
			// We've found one of the same name. Whether we modify it or use it as is, we put in a
			// special dummy in its place. That marks that we've already processed it and accepted it.
			existingLocals.put(name, Boolean.FALSE);
			
			// If the decorator for this entry says not to merge then return.
			// If there is no PropertyDecorator, then we will merge. If they
			// didn't want to merge then should of created of property decorator and said no merge.
			pd = Utilities.getPropertyDecorator((EStructuralFeature) p);
			if (pd != null && !pd.isMergeIntrospection())
				return null;
			prop = (EStructuralFeature) p;
		}
		
		// Need to test if this is an implicit decorator and it is not of the 
		// same type (i.e. is indexed now but wasn't or visa-versa, then we need
		// to get rid of the decorator and recreate it. If it is not implicit, then
		// we have to use it as is because the user specified, so it won't become
		// an indexed if the user did not created it as an index, and visa-versa.
		if (pd != null && pd.isImplicitlyCreated() == FeatureDecorator.NOT_IMPLICIT) {
			// We can't change the type for explicit.
			indexed = pd instanceof IndexedPropertyDecorator;
		} else
			if (pd != null && pd.isImplicitlyCreated() != FeatureDecorator.NOT_IMPLICIT && 
				((indexed && !(pd instanceof IndexedPropertyDecorator)) ||
				 (!indexed && pd instanceof IndexedPropertyDecorator))) {			 
				prop.getEDecorators().remove(pd);
				pd = null;
			}

		int implicit = pd == null ? FeatureDecorator.IMPLICIT_DECORATOR : FeatureDecorator.NOT_IMPLICIT;		
		EcoreFactory ef = (EcoreFactory) ((EcorePackage) RefRegister.getPackage(EcorePackage.packageURI)).getFactory();		
		if (prop == null) {
			// We will create a new property. New properties are always Attributes. Only through overrides can they be a Reference.
			// We can't have an implicit feature, but an explicit decorator.
			prop = ef.createEAttribute();
			getAttributesList().add(prop);
			implicit = FeatureDecorator.IMPLICIT_DECORATOR_AND_FEATURE;
		}			

		// Now fill it in. Normal id for an attribute is "classname.attributename" but we can't use that
		// for us because that format is used by Java Core for a field and there would be confusion.
		// So we will use '/' instead.
		prop.refSetID(getJavaClass().getName() + BeaninfoJavaReflectionKeyExtension.FEATURE + name);
		prop.setName(name);
		// Sample of what UUID get produced:
		//  "com.ibm.etools.emf.ecore/eClass/eAllAttributes"
		prop.refSetUUID(getJavaClass().getJavaPackage().getName() + "/" + getJavaClass().getName() + "/" + name);		 //$NON-NLS-1$ //$NON-NLS-2$
		prop.refSetIsTransient(false);
		prop.refSetIsVolatile(false);
		prop.refSetIsChangeable(changeable);
		prop.refSetType(type);
		if (!indexed) {
			EMultiplicity eMult = ef.createEMultiplicity();
			eMult.refSetID(getJavaClass().getName() + "." + name + ".multiplicity"); //$NON-NLS-1$ //$NON-NLS-2$
			prop.setEMultiplicity(eMult);		
		} else {
			EMultiplicity eMult = ef.createEMultiplicity();
			eMult.refSetID(getJavaClass().getName() + "." + name + ".multiplicity"); //$NON-NLS-1$ //$NON-NLS-2$
			eMult.setLower(0);
			eMult.setUpper(EMultiplicity.UNBOUNDED);
			eMult.setIsOrdered(true);
			prop.setIsUnique(true);			
			prop.setEMultiplicity(eMult);		
		}

		
		// Now create/fill in the property descriptor for it.
		// If there already is one then we
		// will use it. This allows merging with beanfinfo.		
		if (pd == null) {
			pd = (!indexed) ? createPropertyDecorator() : createIndexedPropertyDecorator();
			pd.setImplicitlyCreated(implicit);
			prop.getEDecorators().add(pd);
		}
		pd.setDescriptorProxy(propDesc);
		pd.setDecoratorProxy(null);
		return pd;
	}
	
					
	/**
	 * Get the MOF Factory for the beaninfo decorator classes.
	 */
	protected BeaninfoFactory getFactory() {
		return ((BeaninfoPackage) RefRegister.getPackage(BeaninfoPackage.packageURI)).getBeaninfoFactory();
	}
	
		
	/**
	 * Create the bean decorator. Can be overridden to sue a different decorator.
	 */
	protected BeanDecorator createBeanDecorator() {
		BeanDecorator bd = getFactory().createBeanDecorator();
		getJavaClass().refContainerExtent().add(bd);
		return bd;
	}
	
	
	/**
	 * Create the property decorator. Can be overridden to sue a different decorator.
	 */
	protected PropertyDecorator createPropertyDecorator() {
		PropertyDecorator pd = getFactory().createPropertyDecorator();
		getJavaClass().refContainerExtent().add(pd);
		return pd;
	} 
	
	/**
	 * Create the indexed property decorator. Can be overridden to sue a different decorator.
	 */
	protected IndexedPropertyDecorator createIndexedPropertyDecorator() {
		IndexedPropertyDecorator id = getFactory().createIndexedPropertyDecorator();
		getJavaClass().refContainerExtent().add(id);
		return id;
	}
	
	/**
	 * Create the method decorator. Can be overridden to sue a different decorator.
	 */
	protected MethodDecorator createMethodDecorator() {
		MethodDecorator md = getFactory().createMethodDecorator();
		getJavaClass().refContainerExtent().add(md);
		return md;
	}
	
	/**
	 * Create the event set decorator. Can be overridden to sue a different decorator.
	 */
	protected EventSetDecorator createEventDecorator() {
		EventSetDecorator ed = getFactory().createEventSetDecorator();
		getJavaClass().refContainerExtent().add(ed);
		return ed;
	}		
	
	/**
	 * Reflect the properties. This requires going through local methods and matching them up to
	 * see if they are properties.
	 */
	protected void reflectProperties() {
		// If we are set to mergeSuperTypeProperties, then we need to get the super properties.
		// This is so that duplicate any from super that we find here. When reflecting we don't
		// allow discovered duplicates unless they are different types.
		HashMap supers = new HashMap(50);		
		BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
		if (bd.isMergeSuperProperties()) {
			EClass superType = getJavaClass().getSupertype();
			if (superType != null) {				
				Iterator superAllItr = superType.getEAllAttributes().iterator();
				while (superAllItr.hasNext()) {
					EStructuralFeature sf = (EStructuralFeature) superAllItr.next();
					supers.put(sf.getName(), sf);
				}
				superAllItr = superType.getEAllReferences().iterator();
				while (superAllItr.hasNext()) {
					EStructuralFeature sf = (EStructuralFeature) superAllItr.next();
					supers.put(sf.getName(), sf);
				}
				// Kludge: The above requests for super properties could of caused a recycle (in case the super
				// was stale). Because of this we need to reintrospect to mark ourselves as not stale.
				introspectIfNecessary();
			}
		}
	
		// If we implement PropertyChangeSupport, then all reflected properties are considered bound.
		boolean isBound = getJavaClass().isAssignableFrom(Utilities.getJavaClass("java.beans.PropertyChangeSupport", getJavaClass().refResource().getResourceSet())); //$NON-NLS-1$
		HashMap props = new HashMap();
		
		Iterator itr = getJavaClass().getPublicMethods().iterator();
		while (itr.hasNext()) {
			Method mthd = (Method) itr.next();
			if (mthd.isStatic() || mthd.isConstructor())
				continue;	// Statics/constructors don't participate as properties
			if (mthd.getName().startsWith("get")) { //$NON-NLS-1$
				String name = java.beans.Introspector.decapitalize(mthd.getName().substring(3));
				PropertyInfo propInfo = (PropertyInfo) props.get(name);
				if (propInfo == null) {
					propInfo = new PropertyInfo();
					if (propInfo.setGetter(mthd, false))
						props.put(name, propInfo);
				} else
					propInfo.setGetter(mthd, false);
			} else if (mthd.getName().startsWith("is")) { //$NON-NLS-1$
				String name = java.beans.Introspector.decapitalize(mthd.getName().substring(2));
				PropertyInfo propInfo = (PropertyInfo) props.get(name);
				if (propInfo == null) {
					propInfo = new PropertyInfo();
					if (propInfo.setGetter(mthd, true))
						props.put(name, propInfo);
				} else
					propInfo.setGetter(mthd, true);
			} else if (mthd.getName().startsWith("set")) { //$NON-NLS-1$
				String name = java.beans.Introspector.decapitalize(mthd.getName().substring(3));
				PropertyInfo propInfo = (PropertyInfo) props.get(name);
				if (propInfo == null) {
					propInfo = new PropertyInfo();
					if (propInfo.setSetter(mthd))
						props.put(name, propInfo);
				} else
					propInfo.setSetter(mthd);
			}	
		}
		
		// Now go through the hash map and create the properties.
		itr = props.entrySet().iterator();
		while (itr.hasNext()) {
			Map.Entry entry = (Map.Entry) itr.next();
			EStructuralFeature sf = (EStructuralFeature) supers.get(entry.getKey());
			// Create it if the sf is not a super or if they are of different types.
			if (sf == null || !sf.refType().equals(((PropertyInfo) entry.getValue()).type))
				((PropertyInfo) entry.getValue()).createProperty((String) entry.getKey(), isBound);
		}
	}
	
	private class PropertyInfo {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
		public EClassifier type, indexedType;
		public boolean constrained;
		public Method getter, setter, indexedGetter, indexedSetter;
		
		public boolean setGetter(Method get, boolean mustBeBoolean) {
			List parms = (List) get.getParametersWithoutReturn();
			if (parms.size() > 1)
				return false;	// Invalid - improper number of parms.
			boolean indexed = parms.size() == 1;
			if (indexed && !((JavaParameter) parms.get(0)).getETypeClassifier().getName().equals("int")) //$NON-NLS-1$
				return false;	// Invalid - a parm that is not an int is invalid for indexed.
			EClassifier retType = get.getReturnType();
			if (retType == null || retType.getName().equals("void")) //$NON-NLS-1$
				return false;	// Invalid - must have a return type
			if (mustBeBoolean && !retType.getName().equals("boolean")) //$NON-NLS-1$
				return false;	// Invalid - it must be a boolean.
			if (indexed) {
				if (indexedType != null) {
					if (indexedType != retType)
						return false;	// Invalid - type is different from previous info.
				}
				if (type != null && !(((JavaHelpers) type).isArray() && ((ArrayType) type).getComponentType() == retType))
					return false;	// Invalid - indexed type doesn't match component type of base type.
			} else {
				if (type != null) {
					if (type != retType)
						return false;	// Invalid - type is different from previous info.
				}
				if (indexedType != null && !(((JavaHelpers) retType).isArray() && ((ArrayType) retType).getComponentType() == indexedType))
					if (type == null) {
						// We had a potential indexed and had not yet found the regular type. We've now found
						// the regular type, and it is not indexed. So it takes priority and will wipe out
						// the indexed type.
						indexedGetter = null;
						indexedSetter = null;
						indexedType = null;
					} else
						return false;	// Invalid - indexed type doesn't match component type of base type we already have
			}
			
			if (indexed) {
				if (indexedGetter != null)
					return false;	// Already have an indexed getter.
				indexedGetter = get;
				indexedType = retType;
			} else {
				if (getter != null)
					return false;	// Already have a getter
				getter = get;
				type = retType;
			}
			return true;
		}
		
		public boolean setSetter(Method set) {
			List parms = (List) set.getParametersWithoutReturn();
			if (parms.size() > 2 || parms.size() < 1)
				return false;	// Invalid - improper number of parms.
			boolean indexed = parms.size() == 2;
			if (indexed && !((JavaParameter) parms.get(0)).getETypeClassifier().getName().equals("int")) //$NON-NLS-1$
				return false;	// Invalid - a parm that is not an int is invalid for indexed.
			EClassifier retType = set.getReturnType();
			if (retType != null && !retType.getName().equals("void")) //$NON-NLS-1$
				return false;	// Invalid - must not have a return type
			EClassifier propType = null;
			if (indexed) {
				propType = ((JavaParameter) parms.get(1)).getETypeClassifier();
				if (indexedType != null) {
					if (indexedType != propType)
						return false;	// Invalid - type is different from previous info.
				}
				if (type != null && !(((JavaHelpers) type).isArray() && ((ArrayType) type).getComponentType() == propType))
					return false;	// Invalid - indexed type doesn't match component type of base type, or base type not an array
			} else {
				propType = ((JavaParameter) parms.get(0)).getETypeClassifier();
				if (type != null) {
					if (type != propType)
						return false;	// Invalid - type is different from previous info.
				}
				if (indexedType != null && !(((JavaHelpers) propType).isArray() && ((ArrayType) propType).getComponentType() == indexedType))
					if (type == null) {
						// We had a potential indexed and had not yet found the regular type. We've now found
						// the regular type, and it is not indexed of the correct type. So it takes priority and will wipe out
						// the indexed type.
						indexedGetter = null;
						indexedSetter = null;
						indexedType = null;
					} else
						return false;	// Invalid - indexed type doesn't match component type of base type from this setter.
			}
			
			if (indexed) {
				if (indexedSetter != null)
					return false;	// Already have an indexed getter.
				indexedSetter = set;
				indexedType = propType;
			} else {
				if (setter != null)
					return false;	// Already have a getter
				setter = set;
				type = propType;
			}
			
			if (set.getJavaExceptions().contains(Utilities.getJavaClass("java.beans.PropertyVetoException", getJavaClass().refResource().getResourceSet()))) //$NON-NLS-1$
				constrained = true;
			return true;
		}
		
		public void createProperty(String name, boolean isBound) {
			boolean indexed = indexedType != null;
			if (indexed && type == null)
				return;	// A potential indexed, but never found the getter/setter of the regular type.
				
			PropertyDecorator prop = BeaninfoClassAdapter.this.createProperty(name, indexed, (!indexed) ? (setter != null) : (setter != null || indexedSetter != null), type, null);
			if (prop == null)
				return;	// Reflection not wanted.
			
			indexed = prop instanceof IndexedPropertyDecorator;	// It could of been forced back to not indexed if explicitly set.
				
			// Now fill in the property decorator info. If our decorator was not implicit, then we need
			// to use a proxy so that the use can override specific settings while we supply the defaults.
			// If our decorator was implicitly created, then we know that there are no user data and we
			// can use our decorator directly.
			if (prop.isImplicitlyCreated() == FeatureDecorator.NOT_IMPLICIT) {
				PropertyDecorator propProxy = indexed ? createIndexedPropertyDecorator() : createPropertyDecorator();	// Create a proxy.
				prop.setDecoratorProxy(propProxy);
				prop = propProxy;
			}
				
			prop.setIsBound(isBound);
			prop.setIsConstrained(constrained);
			if (getter != null)
				prop.setReadMethod(getter);
			else
				prop.unsetReadMethod();
			if (setter != null)
				prop.setWriteMethod(setter);
			else
				prop.unsetWriteMethod();
			if (indexed) {
				IndexedPropertyDecorator iprop = (IndexedPropertyDecorator) prop;
				if (indexedGetter != null)
					iprop.setIndexedReadMethod(indexedGetter);
				else
					iprop.unsetIndexedReadMethod();
				if (indexedSetter != null)
					iprop.setIndexedWriteMethod(indexedSetter);
				else
					iprop.unsetIndexedWriteMethod();
			}
				
		}
	};
	
	/**
	 * introspect the behaviors (methods)
	 */
	protected void introspectBehaviors() {
		introspectIfNecessary();			
		if (!fIsIntrospecting && !fIsIntrospectingBehaviors && !fHasIntrospectedBehaviors) {
			fIsIntrospectingBehaviors = true;							
			try {		
				fNewBehaviors = new HashSet(50);				
				BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
				if (bd == null || bd.isIntrospectBehaviors()) {
					// bd wants behaviors to be introspected/reflected
					if (fBeaninfo != null) {
						IArrayBeanProxy mthds = (IArrayBeanProxy) getProxyConstants().getMethodDescriptorsProxy().invokeCatchThrowableExceptions(fBeaninfo);
						if (mthds != null) {
							int mthdSize = mthds.getLength();
							for (int i=0; i<mthdSize; i++)
								try {
									calculateBehavior(mthds.get(i));
								} catch (ThrowableProxy e) {
								}
						}
					} else
						reflectBehaviors();	// No beaninfo, so use reflection to create behaviors
				}

				// Now go through the list and remove those that should be removed.
				Map oldLocals = getBehaviorsMap();
				Iterator itr = getBehaviorsList().iterator();
				while (itr.hasNext()) {
					EBehavioralFeature a = (EBehavioralFeature) itr.next();
					MethodDecorator m = Utilities.getMethodDecorator(a);
					Object aOld = oldLocals.get(a.getName());
					if (aOld != null && aOld != Boolean.FALSE) {
						// A candidate for removal. It was in the old list and it was not processed.
						if (m != null) {
							int implicit = m.isImplicitlyCreated();
							if (implicit != MethodDecorator.NOT_IMPLICIT) {
								m.setEDecorates(null);	// Remove it because it was implicit.
								m.refDelete();
								m = null;
							}
							if (implicit == MethodDecorator.IMPLICIT_DECORATOR_AND_FEATURE) {
								itr.remove();	// The feature was implicit too.
								a.refDelete();
								a = null;
							}
						}
					}
					
					if (m != null)
						m.refreshJavaReferences();
					if (a instanceof MethodProxy)
						((MethodProxy) a).refreshJavaReferences();
				}
				
				fHasIntrospectedBehaviors = true;
			} finally {
				fIsIntrospectingBehaviors = false;
				fBehaviorsMap = null;	// Clear out the temp lists.
				fBehaviorsRealList = null;
				fNewBehaviors = null;

			}			
		}
	}
	
	/**
	 * Fill in the behavior and its decorator using the mthdDesc.
	 */
	protected void calculateBehavior(IBeanProxy bhavDesc) {
		String name = ((IStringBeanProxy) getProxyConstants().getNameProxy().invokeCatchThrowableExceptions(bhavDesc)).stringValue();
		createBehavior(name, null, bhavDesc);
	}
	
	/**
	 * Fill in the behavior and its decorator using the passed in information.
	 */
	public MethodDecorator createBehavior(String name, Method method, IBeanProxy mthdDesc) {	
		// First find if there is already a behavior of this name, and if there is, is the MethodDecorator
		// marked to not allow merging in of introspection results.
		HashMap existingLocals = getBehaviorsMap();
		EBehavioralFeature bhav = null;
		MethodDecorator md = null;
		Object b = null;
		if (name != null)
			b = existingLocals.get(name);
		else {
			// First try by method name. If found and not FALSE, use it. If it is null, then go and use it
			// that name to create a new entry. If it is FALSE, try by id.
			name = method.getName();
			b = existingLocals.get(name);
			if (b == Boolean.FALSE) {
				name = method.refID();
				b = existingLocals.get(name);
			}
		}
		if (Boolean.FALSE == b)
			return null;	// We've already processed this behavior, can't process it again.			
		if (b != null) {
			// We've found one of the same behavior. Whether we modify it or use it as is, we put in a
			// special dummy in its place. That marks that we've already processed it and accepted it.
			existingLocals.put(name, Boolean.FALSE);
			
			// If the decorator for this entry says not to merge then return.
			// If there is no decorator, then we will merge. If they didn't want to
			// merge, then they should of created a decorator with no merge on it.
			md = Utilities.getMethodDecorator((EBehavioralFeature) b);
			if (md != null && !md.isMergeIntrospection())
				return null;
			bhav = (EBehavioralFeature) b;
		}
		
		// Need to find the method and method id.
		String methodID = null;
		if (method == null) {
			// No method sent, create a proxy to it.
			JavaRefFactory f = (JavaRefFactory) ((JavaRefPackage) RefRegister.getPackage(JavaRefPackage.packageURI)).getFactory();
			method = f.createMethod();
			URI uri = Utilities.getMethodURI((IMethodProxy) getProxyConstants().getMethodProxy().invokeCatchThrowableExceptions(mthdDesc));
			methodID = uri.getRef();
			((InternalProxy) method).refSetProxyURI(uri);
		} else
			methodID = method.refID();		
			
		int implicit = md == null ? FeatureDecorator.IMPLICIT_DECORATOR : FeatureDecorator.NOT_IMPLICIT;
		if (bhav == null) {
			// We will create a new MethodProxy.
			bhav = getFactory().createMethodProxy();
			// We have to make sure the name we give it is unique.
			// Only need to do this on new ones because we already assured that reused ones are unique.
			if (fNewBehaviors.contains(name))
				name = stripClass(methodID);
			getBehaviorsList().add(bhav);
			implicit = FeatureDecorator.IMPLICIT_DECORATOR_AND_FEATURE;	// Can't have an implicit feature but explicit decorator.
		}			
		
		// Now fill it in.
		if (bhav instanceof MethodProxy)
			((MethodProxy) bhav).setMethod(method);
		bhav.refSetID(getJavaClass().getName()+BeaninfoJavaReflectionKeyExtension.BEHAVIOR+name);
		bhav.setName(name);
		fNewBehaviors.add(name);
		// Sample of what UUID get produced:
		//  "com.ibm.etools.emf.ecore/eClass/eAllAttributes"
		bhav.refSetUUID(getJavaClass().getJavaPackage().getName() + "/" + getJavaClass().getName() + BeaninfoJavaReflectionKeyExtension.BEHAVIOR + name); //$NON-NLS-1$
		
		// Now create/fill in the method decorator for it.
		// If there already is one then we
		// will use it. This allows merging with beaninfo.		
		if (md == null) {
			md = createMethodDecorator();
			md.setImplicitlyCreated(implicit);
			bhav.getEDecorators().add(md);
		}
		md.setDescriptorProxy(mthdDesc);
		md.setDecoratorProxy(null);
		return md;
	}
	
	/**
	 * Strip the class name off of the method id.
	 */
	protected String stripClass(String methodID) {
		return methodID.substring(methodID.indexOf(ReflectionAdaptor.C_CLASS_MEMBER_DELIMITER)+1);
	}
	
	/**
	 * Reflect the behaviors. This requires going through local public methods and creating a 
	 * method proxy for it.
	 */
	protected void reflectBehaviors() {
		// If we are set to mergeSuperTypeBehaviors, then we need to get the super behaviors.
		// This is so that duplicate any from super that we find here. When reflecting we don't
		// allow discovered duplicates unless they are different signatures.
		HashMap supers = new HashMap(50);		
		BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
		if (bd.isMergeSuperBehaviors()) {
			EClass superType = getJavaClass().getSupertype();
			if (superType != null) {				
				Iterator superAllItr = superType.getEAllBehaviors().iterator();
				while (superAllItr.hasNext()) {
					EBehavioralFeature sf = (EBehavioralFeature) superAllItr.next();
					supers.put(sf.getName(), sf);
				}
				// Kludge: The above requests for super properties could of caused a recycle (in case the super
				// was stale). Because of this we need to reintrospect to mark ourselves as not stale.
				introspectIfNecessary();
			}
		}
		
		Iterator itr = getJavaClass().getPublicMethods().iterator();
		while (itr.hasNext()) {
			Method mthd = (Method) itr.next();
			if (mthd.isStatic() || mthd.isConstructor())
				continue;	// Statics/constructors don't participate as behaviors			
			EBehavioralFeature bf = (EBehavioralFeature) supers.get(mthd.getName());
			// See if same signature, if so then don't duplicate it.
			if (bf instanceof MethodProxy && sameSignature(mthd.refID(), ((MethodProxy) bf).getMethod().refID()))
				continue;
			// See if there is one of the same signature instead of name.
			bf = (EBehavioralFeature) supers.get(stripClass(mthd.refID()));
			if (bf != null)
				continue;
				
			createBehavior(null, mthd, null);	// Don't pass a name, try to create it by name, only use ID if there is more than one of the same name.
		}
	}	
			
	protected boolean sameSignature(String m1ID, String m2ID) {
		// See if the signatures match except for the class name at the beginning of each.
		int m1Sig = m1ID.indexOf(ReflectionAdaptor.C_CLASS_MEMBER_DELIMITER);
		int m2Sig = m2ID.indexOf(ReflectionAdaptor.C_CLASS_MEMBER_DELIMITER);		
		return m1ID.regionMatches(m1Sig+1, m2ID, m2Sig+1, m1ID.length()-m1Sig-1);
	}
	
	/**
	 * merge all  the Behaviors(i.e. supertypes)
	 */
	protected void allBehaviors(RefObject allAttrSF) {
		if (!fIsIntrospecting && !fIsDoingAllBehaviors) {
			fIsDoingAllBehaviors = true;							
			try {
				introspectIfNecessary();				
				EList allBehaviors = (EList) getJavaClass().primRefValue(allAttrSF);
				allBehaviors.clear();	
				EList behaviors = getJavaClass().getEBehaviors();	// Explicitly want to trigger introspection
				EClass superType = getJavaClass().getSupertype();
				if (superType != null) {				
					// Now we need to merge in the supers.
					BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
					// We will now merge in either all supers, or if told not to merge. If not merge, then only those without 
					// property descriptors.
					boolean mergeAll = bd == null || bd.isMergeSuperBehaviors();

					// We have set of all the local features added so that we won't add features from super classes that were already added.
					HashMap keys = new HashMap(100);
					Iterator itr = behaviors.iterator();
					while (itr.hasNext()) {
						addKey(keys, (EBehavioralFeature) itr.next());
					}						
					Iterator supers = superType.getEAllBehaviors().iterator();
					while (supers.hasNext()) {
						// We know the super allBehaviors has already been vetted for dups within itself, so
						// only need to compare with dups against this one's locals.
						EBehavioralFeature bhav = (EBehavioralFeature) supers.next();
						MethodDecorator md = null;
						if (!mergeAll)
							md = Utilities.getMethodDecorator(bhav);
						// Add only if:
						//   1) This is not a dup to one that was added as local.
						//                 AND any one of the following are true
						//      a) We want all beaninfos.
						//      b) There is no method decorator (this means it was added through an XMI file and should show up)
						//
						if (!isDup(keys, bhav) && (mergeAll || md == null)) {
							allBehaviors.add(bhav);
						}
					}
				}
				allBehaviors.addAll(behaviors);
				ESuperAdapter sa = (ESuperAdapter) getTarget().getAdapter(IESuperAdapter.ADAPTER_KEY);
				sa.setAllBehaviorsCollectionModified(false);	// Now built, so reset to not changed.				
			} finally {
				fIsDoingAllBehaviors = false;
			}
		}
	}
	
	/**
	 * Add the key to the list. If the feature is a dup, add the longname also.
	 */
	private void addKey(HashMap keys, EBehavioralFeature feature) {
		String name = feature.getName();
		Object existing = keys.get(name);
		if (existing == null) {
			keys.put(name, feature);
		} else {
			if (existing instanceof EBehavioralFeature) {
				// This is the first time a dup name was found. Turn the value into a long name so
				// future comparisons can easily be made for a dup with this first entry.
				existing = formLongName((EBehavioralFeature) existing);
				keys.put(name, existing);	// Set it to the longname for future comparisons.
			}
			String longName = formLongName(feature);
			if (longName == null)
				return;	// It's not a Method or MethodProxy, so can't form longname. So just accept it.
			if (longName.equals(existing))
				return;	// It's a dup to the existing short named one
			if (keys.containsKey(longName))
				return;	// It's a dup to another one.
			keys.put(longName, longName);
		}
	}
	
	/**
	 * Check to see if this feature is a dup to one already in the list.
	 */
	private boolean isDup(HashMap keys, EBehavioralFeature feature) {
		String name = feature.getName();
		Object existing = keys.get(name);
		if (existing == null)
			return false;	// Not a dup.
		else {
			if (existing instanceof EBehavioralFeature) {
				// This is the first time a dup name was found. Turn the value into a long name so
				// future comparisons can easily be made for a dup with this first entry.
				existing = formLongName((EBehavioralFeature) existing);
				keys.put(name, existing);	// Set it to the longname for future comparisons.
			}
			String longName = formLongName(feature);
			if (longName == null)
				return false;	// It's not a Method or MethodProxy, so can't form longname. So just accept it.
			if (longName.equals(existing))
				return true;	// It's a dup to the existing short named one
			if (keys.containsKey(longName))
				return true;	// It's a dup to another one.
			return false;
		}
	}
	
	
	/**
	 * introspect the events
	 */
	protected void introspectEvents() {
		introspectIfNecessary();			
		if (!fIsIntrospecting && !fIsIntrospectingEvents && !fHasIntrospectedEvents) {
			fIsIntrospectingEvents = true;							
			try {					
				BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
				if (bd == null || bd.isIntrospectEvents()) {
					// bd wants events to be introspected/reflected
					if (fBeaninfo != null) {
						IArrayBeanProxy events = (IArrayBeanProxy) getProxyConstants().getEventSetDescriptorsProxy().invokeCatchThrowableExceptions(fBeaninfo);
						if (events != null) {
							int eventSize = events.getLength();
							for (int i=0; i<eventSize; i++)
								try {
									calculateEvent(events.get(i));
								} catch (ThrowableProxy e) {
								}
						}
					} else
						reflectEvents();	// No beaninfo, so use reflection to create events
				}

				// Now go through the list and remove those that should be removed.
				Map oldLocals = getEventsMap();
				Iterator itr = getEventsList().iterator();
				while (itr.hasNext()) {
					JavaEvent a = (JavaEvent) itr.next();
					EventSetDecorator e = Utilities.getEventSetDecorator(a);					
					Object aOld = oldLocals.get(a.getName());
					if (aOld != null && aOld != Boolean.FALSE) {
						// A candidate for removal. It was in the old list and it was not processed.
						if (e != null) {
							int implicit = e.isImplicitlyCreated();
							if (implicit != EventSetDecorator.NOT_IMPLICIT) {
								e.setEDecorates(null);	// Remove it because it was implicit.
								e.refDelete();
								e = null;
							}
							if (implicit == EventSetDecorator.IMPLICIT_DECORATOR_AND_FEATURE) {
								itr.remove();	// The feature was implicit too.
								a.refDelete();
								a = null;
							}
						}
					}
					
					if (e != null)
						e.refreshJavaReferences();
				}
				
				fHasIntrospectedEvents = true;
			} finally {
				fIsIntrospectingEvents = false;
				fEventsMap = null;	// Clear out the temp lists.
				fEventsRealList = null;
			}			
		}
	}
		
	/**
	 * Fill in the event and its decorator using the eventDesc.
	 */
	protected void calculateEvent(IBeanProxy eventDesc) {
		String name = ((IStringBeanProxy) getProxyConstants().getNameProxy().invokeCatchThrowableExceptions(eventDesc)).stringValue();
		createEvent(name, eventDesc);
	}	
	
	/**
	 * Fill in the event and its decorator using the passed in information.
	 */
	public EventSetDecorator createEvent(String name, IBeanProxy eventDesc) {	
		// First find if there is already an event of this name, and if there is, is the EventSetDecorator
		// marked to not allow merging in of introspection results.
		HashMap existingLocals = getEventsMap();
		JavaEvent event = null;
		EventSetDecorator ed = null;
		Object b = existingLocals.get(name);
		if (Boolean.FALSE == b)
			return null;	// We've already processed this event, can't process it again.			
		if (b != null) {
			// We've found one of the same event. Whether we modify it or use it as is, we put in a
			// special dummy in its place. That marks that we've already processed it and accepted it.
			existingLocals.put(name, Boolean.FALSE);
			
			// If the decorator for this entry says not to merge then return.
			// If there is no decorator, then we will merge. If they didn't want to
			// merge, then they should of created a decorator with no merge on it.
			ed = Utilities.getEventSetDecorator((JavaEvent) b);
			if (ed != null && !ed.isMergeIntrospection())
				return null;
			event = (JavaEvent) b;
		}	
			
		int implicit = ed == null ? FeatureDecorator.IMPLICIT_DECORATOR : FeatureDecorator.NOT_IMPLICIT;
		if (event == null) {
			// We will create a new Event.
			event = getFactory().createBeanEvent();
			getEventsList().add(event);
			implicit = FeatureDecorator.IMPLICIT_DECORATOR_AND_FEATURE;	// Can't have an implicit feature but explicit decorator.
		}			
		
		// Now fill it in.
		event.refSetID(getJavaClass().getName()+BeaninfoJavaReflectionKeyExtension.EVENT+name);
		event.setName(name);
		// Sample of what UUID get produced:
		event.refSetUUID(getJavaClass().getJavaPackage().getName() + "/" + getJavaClass().getName() + BeaninfoJavaReflectionKeyExtension.EVENT + name); //$NON-NLS-1$
		
		// Now create/fill in the event decorator for it.
		// If there already is one then we
		// will use it. This allows merging with beaninfo.		
		if (ed == null) {
			ed = createEventDecorator();
			ed.setImplicitlyCreated(implicit);
			event.getEDecorators().add(ed);
		}
		ed.setDescriptorProxy(eventDesc);
		ed.setDecoratorProxy(null);
		return ed;
	}	
	
	/**
	 * Reflect the events. This requires going through local public methods and creating an 
	 * event for the discovered events.
	 */
	protected void reflectEvents() {
		// If we are set to mergeSuperTypeEvents, then we need to get the super events.
		// This is so that duplicate any from super that we find here. When reflecting we don't
		// allow discovered duplicates.
		HashMap supers = new HashMap(50);		
		BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
		if (bd.isMergeSuperEvents()) {
			JavaClass superType = (JavaClass) getJavaClass().getSupertype();
			if (superType != null) {				
				Iterator superAllItr = superType.getAllEvents().iterator();
				while (superAllItr.hasNext()) {
					JavaEvent se = (JavaEvent) superAllItr.next();
					supers.put(se.getName(), se);
				}
				// Kludge: The above requests for super events could of caused a recycle (in case the super
				// was stale). Because of this we need to reintrospect to mark ourselves as not stale.
				introspectIfNecessary();
			}
		}
		
		HashMap events = new HashMap();
		eventListenerClass = (JavaClass) JavaClassImpl.reflect("java.util.EventListener", getJavaClass());	// Initialize, needed for building eventinfos. //$NON-NLS-1$
		tooManyExceptionClass = (JavaClass) JavaClassImpl.reflect("java.util.TooManyListenersException", getJavaClass());	// Initialize, needed for building eventinfos. //$NON-NLS-1$
		Iterator itr = getJavaClass().getPublicMethods().iterator();
		while (itr.hasNext()) {
			Method mthd = (Method) itr.next();
			if (mthd.isStatic() || mthd.isConstructor())
				continue;	// Statics/constructors don't participate in events.
			String key = validEventAdder(mthd);
			if (key != null) {
				EventInfo eventInfo = (EventInfo) events.get(key);
				if (eventInfo == null) {
					eventInfo = new EventInfo();
					eventInfo.setAdder(mthd);
					events.put(key, eventInfo);
				} else
					eventInfo.setAdder(mthd);
			} else {
				key = validEventRemove(mthd);
				if (key != null) {
					EventInfo eventInfo = (EventInfo) events.get(key);
					if (eventInfo == null) {
						eventInfo = new EventInfo();
						eventInfo.setRemover(mthd);
						events.put(key, eventInfo);
					} else
						eventInfo.setRemover(mthd);
				}
			}
		}

		eventListenerClass = null;	// No longer need it.
				
		// Now actually create the events.
		HashSet eventNames = new HashSet(events.size());	// Set of found event names, to prevent duplicates
		Iterator evtItr = events.entrySet().iterator();
		while (evtItr.hasNext()) {
			Map.Entry eventMap = (Map.Entry) evtItr.next();
			EventInfo ei = (EventInfo) eventMap.getValue();
			if (ei.isValidInfo()) {
				String eventName = getEventName((String) eventMap.getKey());
				if (eventNames.contains(eventName))
					continue;	// Aleady created it. (Note: Introspector actually takes last one over previous dups, but the order is totally undefined, so choosing first is just as good or bad.
					
				JavaEvent superEvent = (JavaEvent) supers.get(eventName);
				if (superEvent != null)
					continue;	// Don't override a super event.
					
				if (ei.createEvent(eventName))
					eventNames.add(eventName);	// It was validly created.
			}
		}
		
		tooManyExceptionClass = null;	// No longer need it.
	}
	
	/**
	 * merge all the Events (i.e. supertypes)
	 */
	protected void allEvents(RefObject allEventSF) {
		if (!fIsIntrospecting && !fIsDoingAllEvents) {
			fIsDoingAllEvents = true;							
			try {
				introspectIfNecessary();
				EList allEvents = (EList) getJavaClass().primRefValue(allEventSF);
				allEvents.clear();	
				EList localEvents = getJavaClass().getEvents();	// Explicitly want to possibly trigger introspection.
				JavaClass superType = getJavaClass().getSupertype();
				if (superType != null) {				
					// Now we need to merge in the supers.
					BeanDecorator bd = Utilities.getBeanDecorator(getJavaClass());
					// We will now merge in either all supers, or if told not to merge. If not merge, then only those without 
					// property descriptors.
					boolean mergeAll = bd == null || bd.isMergeSuperEvents();
					// We have set of all the names added so that we won't add already added properties, by name.
					HashSet keys = new HashSet(100);
					Iterator itr = localEvents.iterator();
					while (itr.hasNext()) {
						keys.add(((JavaEvent) itr.next()).getName());
					}						
					Iterator supers = superType.getAllEvents().iterator();
					while (supers.hasNext()) {
						JavaEvent evt = (JavaEvent) supers.next();
						EventSetDecorator ed = null;
						if (!mergeAll)
							ed = Utilities.getEventSetDecorator(evt);
						// Add only if all of the following are true:
						//   1) The feature's name is not the same as one of locals (so that we can override if they are the same).	
						//   2) If any of the following are true:
						//      a) We want all beaninfos.
						//      b) There is no event set decorator (this means it was added through an XMI file and should show up)
						if (!keys.contains(evt.getName()) && (mergeAll || ed == null)) {
							keys.add(evt.getName());
							allEvents.add(evt);
						}
					}
				}
				allEvents.addAll(localEvents);
				BeaninfoSuperAdapter sa = (BeaninfoSuperAdapter) getTarget().getAdapter(BeaninfoSuperAdapter.ADAPTER_KEY);
				sa.setAllEventsCollectionModified(false);	// Now built, so reset to not changed.
			} finally {
				fIsDoingAllEvents = false;
			}
		}
	}	
	
	
	
	private JavaClass 
		eventListenerClass,	// Event Listener class. Needed for validation.
		tooManyExceptionClass;	// Too many listeners exception.

	/*
	 * Pass in the key, it will be used to form the name.
	 */
	protected String getEventName(String key) {
		return key.substring(0, key.indexOf(':'));
	}

	/*	
	 * Answers event key if valid, null if not valid.
	 */
	protected String validEventAdder(Method method) {
		String name = method.getName();
		if (!name.startsWith("add") || !name.endsWith("Listener")) //$NON-NLS-1$ //$NON-NLS-2$
			return null;	// Not valid format for an add listener name.
			
		List parms = (List) method.getParametersWithoutReturn();
		if (parms.size() != 1)
			return null;	// Invalid - improper number of parms.
			
		EClassifier returnType = method.getReturnType();
		if (returnType == null || !returnType.getName().equals("void")) //$NON-NLS-1$
			return null;	// Invalid - must be void return type.
			
		EClassifier parmType = ((JavaParameter) parms.get(0)).getETypeClassifier();
		if (!BeaninfoClassAdapter.this.eventListenerClass.isAssignableFrom(parmType))
			return null;	// Parm must be inherit from EventListener

		// This is a unique containing event name and listener type
		// This is so we can have a unique key for two events with the same
		// name but different listener type. (This matches Introspector so that we aren't
		// coming up with different results.
		return java.beans.Introspector.decapitalize(name.substring(3, name.length()-8)) + ':' + ((JavaHelpers) parmType).getQualifiedName();
	}
	
	/*
	 * Answers event key if valid, null if not valid.
	 */
	protected String validEventRemove(Method method) {
		String name = method.getName();
		if (!name.startsWith("remove") || !name.endsWith("Listener")) //$NON-NLS-1$ //$NON-NLS-2$
			return null;	// Not valid format for a remove listener name.
			
		List parms = (List) method.getParametersWithoutReturn();
		if (parms.size() != 1)
			return null;	// Invalid - improper number of parms.
			
		EClassifier returnType = method.getReturnType();
		if (returnType == null || !returnType.getName().equals("void")) //$NON-NLS-1$
			return null;	// Invalid - must be void return type.
			
		EClassifier parmType = ((JavaParameter) parms.get(0)).getETypeClassifier();
		if (!BeaninfoClassAdapter.this.eventListenerClass.isAssignableFrom(parmType))
			return null;	// Parm must be inherit from EventListener

		// This is a unique containing event name and listener type
		// This is so we can have a unique key for two events with the same
		// name but different listener type. (This matches Introspector so that we aren't
		// coming up with different results).
		return java.beans.Introspector.decapitalize(name.substring(6, name.length()-8)) + ':' + ((JavaHelpers) parmType).getQualifiedName();
	}
	
		
	private class EventInfo {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
		public Method addListenerMethod;
		public Method removeListenerMethod;
				
		public void setAdder(Method addMethod) {
			addListenerMethod = addMethod;
		}
				
		public void setRemover(Method removeMethod) {
			removeListenerMethod = removeMethod;
		}
		
		// Answer whether this is a valid event info.
		public boolean isValidInfo() {
			return (addListenerMethod != null && removeListenerMethod != null);
		}
				
		public boolean createEvent(String name) {				
			EventSetDecorator ed = BeaninfoClassAdapter.this.createEvent(name, null);
			if (ed == null)
				return false;	// Reflection not wanted.
		
			ed.setAddListenerMethod(addListenerMethod);
			ed.setRemoveListenerMethod(removeListenerMethod);
			
			// See if unicast.
			Iterator itr = addListenerMethod.getJavaExceptions().iterator();
			while (itr.hasNext()) {
				if (itr.next() == BeaninfoClassAdapter.this.tooManyExceptionClass) {
					ed.setIsUnicast(true);
					break;
				}
			}
			
			// Set the listener type.
			List parms = addListenerMethod.getParametersWithoutReturn();
			ed.setListenerType((JavaClass) ((JavaParameter) parms.get(0)).getETypeClassifier());
			
			// We'll let listener methods get retrieved dynamically when needed.
			
			return true;
		}

	}				
	
	/**
	 * Mark this factory as the stale factory.
	 */
	public void markStaleFactory(ProxyFactoryRegistry stale) {
		if (fStaleFactory == null) {
			// It's not stale so make it stale.
			fHasIntrospected = fHasIntrospectedProperties = fHasIntrospectedBehaviors = fHasIntrospectedEvents = false;	// So that next access will re-introspect
			fStaleFactory = new WeakReference(stale);
			if (fBeaninfo != null) {
				fBeaninfo.getProxyFactoryRegistry().releaseProxy(fBeaninfo);	// Dispose of the beaninfo since we will need to recreate it
				fBeaninfo = null;
			}
			
			// Need to mark the esuperadapter that things have changed so that any 
			// subtype will know to reuse the parent for anything that requires knowing parent info.
			Adapter a = getTarget().getAdapter(ESuperAdapter.ADAPTER_KEY);
			// Simulate that this objects super has changed. This will make all subclasses
			// think about the super has changed and next retrieving anything that involves the
			// super will cause a rebuild to occur.
			Notification note = new NotificationImpl(getTarget(), Notifier.TOUCH, sESuperSF, null, null, -1);
			a.notifyChanged(note);
			// Do the same with BeaninfoSuperAdapter so that events also will be rebuilt.
			a = getTarget().getAdapter(BeaninfoSuperAdapter.ADAPTER_KEY);			
			a.notifyChanged(note);			
		}
	}
	
	/**
	 * Form a longname for the addkey function.
	 */
	private String formLongName(EBehavioralFeature feature) {
		Method mthd = null;
		if (feature instanceof Method)
			mthd = (Method) feature;
		else if (feature instanceof MethodProxy)
			mthd = ((MethodProxy) feature).getMethod();
		else
			return null;	// Don't know what it is.
			
		StringBuffer longName = new StringBuffer(100);
		longName.append(feature.getName());	// Feature Name
		longName.append(':');
		longName.append(mthd.getName());	// Method Name
		longName.append('(');
		Iterator itr = mthd.getParameters().iterator();
		boolean first = true;
		while (itr.hasNext()) {
			JavaParameter parm = (JavaParameter) itr.next();
			if (parm.isReturn())
				continue;
			if (!first)
				longName.append(',');
			else
				first = false;
			longName.append(parm.getJavaType().getQualifiedName());
		}
		
		return longName.toString();
	}	
	/**
	 * @see com.ibm.etools.emf.notify.Adapter#notifyChanged(Notification)
	 */
	public void notifyChanged(Notification msg) {
		// In case of removing adapter, make sure we are first removed from the factory so it doesn't know about us anymore.
		if (msg.getEventType() == Notifier.REMOVING_ADAPTER)
			getAdapterFactory().removeAdapter(this);
	}

}