package com.ibm.etools.beaninfo.export;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.util.Collection;
import java.util.Iterator;

import com.ibm.etools.beaninfo.*;
import com.ibm.etools.beaninfo.BeanDecorator;
import com.ibm.etools.beaninfo.FeatureDecorator;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.diff.MofDMTool;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ecore.EAttribute;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.emf.ecore.utilities.Association;
import com.ibm.etools.emf.ecore.utilities.copy.CopyGroup;
import com.ibm.etools.emf.ecore.utilities.copy.EtoolsCopyUtility;
import com.ibm.etools.emf.ref.InternalProxy;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.impl.ExtentImpl;
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.emf.resource.impl.ContextImpl;
import com.ibm.etools.emf.resource.impl.ResourceSetImpl;
import com.ibm.etools.java.*;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.JavaRefFactory;

/**
 * This class manages the export process of a particular bean.
 * Used for creating the diff file for a bean.
 * @version 	1.0
 * @author
 */
public class ExportManager {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	protected Resource rs;
	protected JavaClass fromClass;
	protected JavaClass jclass;
	protected MofDMTool diffTool;
	
	/**
	 * Construct with a from java class. A working copy
	 * is made of the class. Do not send in an array type.
	 * No check is made, but it won't work.
	 */
	public ExportManager(JavaClass fromClass) {
		this.fromClass = fromClass;
		
		ResourceSet rset = new ResourceSetImpl();
		Context context = new ContextImpl();
		context.setResourceSet(rset);
		
		// Create a new resource that has the uri as the one the java class came from.
		// This is necessary so that the diff program creates the correct hrefs.
		String uri = fromClass.refResource().getURI().toString();
		rs = ResourceFactoryRegister.getFactory(uri, rset).makeResource(uri, new ExtentImpl());	// Due to bug JavaXMIFactory need to pass extent too.
		rset.add(rs);
		
		// Now fully do introspection on this guy so that we can work with it fully introspected.
		fromClass.getEAttributes();
		fromClass.getEReferences();
		fromClass.getEBehaviors();
		fromClass.getEvents();
		
		// Add an empty copy of the javaclass to the resource so that the diff will see changes from now on.
		JavaRefPackage pkg = (JavaRefPackage) fromClass.refMetaObject().refContainer();
		jclass = pkg.getJavaRefFactory().createJavaClass();
		jclass.refSetID(fromClass.refID());
		rs.getExtent().add(jclass);
		rs.getExtent().remove(rs.getExtent().getObjectByType((EMetaObject) pkg.refMetaObject()));	// A package is automatically created, we don't want it.
		diffTool = new MofDMTool(rs.getExtent());		
	}
	
	public void createDiff(String rootDirectory) {
		String classname = fromClass.getQualifiedNameForReflection();
		classname = classname.replace('.', '/')+".xmi"; //$NON-NLS-1$
		rootDirectory = rootDirectory.replace('\\', '/');
		if (rootDirectory.charAt(rootDirectory.length()-1) != '/')
			rootDirectory += '/';
		diffTool.differencing(rootDirectory+classname);
	}
	
	/**
	 * Answer the copied class.
	 */
	public JavaClass getCopyiedClass() {
		return jclass;
	}
		
	/**
	 * Force all of the decorators to be overrides and
	 * no introspection required. Will take the introspection
	 * results and set them as actual settings on the decorators,
	 * and turn off introspection in the copied decorators.
	 */
	public void forceAllOverride() {
		// Force the bean decorator to override everything and prevent introspection.
		forceBeanDecoratorOveride();
		
		// We now need to copy all attributes too, and their property decorator.
		List copyAttrs = jclass.getEAttributes();
		Iterator itr = fromClass.getEAttributes().iterator();
		while (itr.hasNext()) {
			EAttribute attr = (EAttribute) itr.next();
			forceAttributeOverride(copyAttrs, attr);
		}
		
		// We now need to copy all references too, and their property decorator.
		List copyRefs = jclass.getEReferences();
		itr = fromClass.getEReferences().iterator();
		while (itr.hasNext()) {
			EReference ref = (EReference) itr.next();
			forceReferenceOverride(copyRefs, ref);
		}
		
		// We now need to copy all behaviors too, and their method decorator.
		List copyBehavs = jclass.getEBehaviors();
		itr = fromClass.getEBehaviors().iterator();
		while (itr.hasNext()) {
			EBehavioralFeature bhav = (EBehavioralFeature) itr.next();
			forceBehaviorOverride(copyBehavs, bhav);
		}
		
		// When events work, will need to do that to.
	}

	/**
	 * Force a full override of the behavior. It will add it to the
	 * EBehaviors list passed in. This will turn off merging with introspection.
	 * The copied EBehavior will be returned and it or the decorator on it
	 * can be modified to turn anything back on.
	 */
	public EBehavioralFeature forceBehaviorOverride(List copyBehavs, EBehavioralFeature bhav) {
		EBehavioralFeature copyBehav = (EBehavioralFeature) copyModelElement(bhav);
		copyBehavs.add(copyBehav);
		MethodDecorator md = Utilities.getMethodDecorator(bhav);
		if (md != null) {
			MethodDecorator copymd = (MethodDecorator) md.createIntrospectedCopy();
			copymd.setMergeIntrospection(false);
			rs.getExtent().add(copymd);
			copymd.refSetID();	// Force an id if not already set. The differencing tool requires it.
			copymd.setEDecorates(copyBehav);
		}
		
		return copyBehav;
	}

	/**
	 * Force a full override of the reference. It will add it to the
	 * EReferences list passed in. This will turn off merging with introspection.
	 * The copied EReference will be returned and it or the decorator on it
	 * can be modified to turn anything back on.
	 */
	public EReference forceReferenceOverride(List copyRefs, EReference ref) {
		EReference copyRef = (EReference) copyModelElement(ref);
		copyRefs.add(copyRef);
		PropertyDecorator pd = Utilities.getPropertyDecorator(ref);
		if (pd != null) {
			PropertyDecorator copypd = (PropertyDecorator) pd.createIntrospectedCopy();
			copypd.setMergeIntrospection(false);
			rs.getExtent().add(copypd);
			copypd.refSetID();	// Force an id if not already set. The differencing tool requires it.
			copypd.setEDecorates(copyRef);
		}
		
		return copyRef;
	}

	/**
	 * Force a full override of the attribute. It will add it to the
	 * EAttributes list passed in. This will turn off merging with introspection.
	 * The copied EAttribute will be returned and it or the decorator on it
	 * can be modified to turn anything back on.
	 */
	public EAttribute forceAttributeOverride(List copyAttrs, EAttribute attr) {
		EAttribute copyAttr = (EAttribute) copyModelElement(attr);
		copyAttrs.add(copyAttr);
		PropertyDecorator pd = Utilities.getPropertyDecorator(attr);
		if (pd != null) {
			PropertyDecorator copypd = (PropertyDecorator) pd.createIntrospectedCopy();
			copypd.setMergeIntrospection(false);
			rs.getExtent().add(copypd);
			copypd.refSetID();	// Force an id if not already set. The differencing tool requires it.
			copypd.setEDecorates(copyAttr);
		}
		
		return copyAttr;
	}

	/**
	 * Force a full override of the bean decorator. This will turn off 
	 * introspection entirely. Users can modify the resulting bean decorator if
	 * they wish to turn anything back on.
	 */
	public BeanDecorator forceBeanDecoratorOveride() {
		BeanDecorator bd = Utilities.getBeanDecorator(fromClass);
		if (bd != null) {
			BeanDecorator copybd = (BeanDecorator) bd.createIntrospectedCopy();
			copybd.setIntrospectBehaviors(false);	// Don't introspect behaviors/events/properties, they will be fully defined in this diff file.
			copybd.setIntrospectEvents(false);
			copybd.setIntrospectProperties(false);
			copybd.setMergeIntrospection(false);
			copybd.setDoBeaninfo(false);
			rs.getExtent().add(copybd);
			copybd.refSetID();	// Force an id if not already set. The differencing tool requires it.
			copybd.setEDecorates(jclass);
			return copybd;
		}
		
		return null;
	}
	
	/*
	 * Want to copy the model element without the decorators.
	 */
	protected EModelElement copyModelElement(EModelElement model) {
		// We don't want the decorators, but there is bug wrt/decorators
		// in copy util if they aren't in the group being copied. So we need
		// to create a group containing them, do the copy, remove the decorators
		// from the copy, and return the copy.

		EtoolsCopyUtility copyUtil = new EtoolsCopyUtility();		
		CopyGroup cg = new CopyGroup();
		cg.setPreserveIds(true);
		cg.add(model);
		Iterator itr = model.getEDecorators().iterator();
		while (itr.hasNext())
			cg.add((EDecorator) itr.next());
			
		copyUtil.copy(cg);

		EModelElement newModel = (EModelElement) copyUtil.getCopy(model);
		newModel.getEDecorators().clear();
		if (newModel.refID() == null)
			newModel.refSetID(rs.makeHref(newModel));
		return newModel;
	}
}
