package com.ibm.etools.beaninfo.vm;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.beans.*;
import java.util.*;
/**
 * This class is the beaninfo class that is created when
 * beaninfo modeling introspects a bean. Its purpose is
 * to gather together and analyze the beaninfo. For example,
 * it checks with the supertype beaninfo to see if all of
 * the supertype's descriptors are included in this list.
 * If they are, then it knows that it does a straight
 * inheritance of the supertype's descriptors, and those
 * descriptors can be removed from the list. This makes
 * it easier on the model side so that there isn't a
 * great proliferation of descriptors all describing the
 * same properties. In that case they can be merged from 
 * the supertype model. If some are not found, then that
 * means this beaninfo is trying to hide some of them,
 * and in that case this is the definitive list and 
 * inheritance shouldn't be used on the model side.
 *
 * The test for seeing if the super feature is included
 * in the beaninfo is first to see if the the feature
 * is in the beaninfo by name, if it is then it is
 * marked as included. Then a check is made on equality,
 * if they are equal, then the feature is removed from the
 * beaninfo list, but the merge flag is still left on.
 * This allows merging to occur but it also allows 
 * overrides to occur.
 */

public class ModelingBeanInfo extends SimpleBeanInfo {
	public static final String copyright = "(c) Copyright IBM Corporation 2001, 2002."; //$NON-NLS-1$
	// The following fields indicate if the super info should be merged
	// in on the model side.
	protected boolean
		fMergeSuperEvents = false,
		fMergeSuperMethods = false,
		fMergeSuperProperties = false;
		
	protected final BeanInfo fTargetBeanInfo;	// The beaninfo being modeled.
	
	protected EventSetDescriptor[] fEventSets;
	protected MethodDescriptor[] fMethods;	
	protected PropertyDescriptor[] fProperties;
	

	/**
	 * Method used to do introspection and create the appropriate ModelingBeanInfo
	 *
	 * This will always introspect.
	 */
	public static ModelingBeanInfo introspect(Class introspectClass) throws IntrospectionException {
		return introspect(introspectClass, true);
	}
	
	/**
	 * Method used to do introspection and create the appropriate ModelingBeanInfo
	 *
	 * introspectIfNoBeanInfo: If this is true, then if no explicit beaninfo was found for this
	 *   class, then introspection will be done anyway. The Introspector will use
	 *   reflection for local methods/events/properties of this class and then add
	 *   in the results of the superclass introspection. If this parameter is false,
	 *   then if the explicit beaninfo is not found, then no introspection will be
	 *   done and null will be returned.
	 */
	public static ModelingBeanInfo introspect(Class introspectClass, boolean introspectIfNoBeanInfo) throws IntrospectionException {
		if (!introspectIfNoBeanInfo) {
			// Need to check if the beaninfo is explicitly supplied.
			// If not, then we return null.
			// The checks will be the same that Introspector will use.
			
			boolean found = false;
			// 1. Is there a BeanInfo class in the same package
			if (!classExists(introspectClass.getName()+"BeanInfo", introspectClass)) { //$NON-NLS-1$
				// 2. Is this class a BeanInfo class for itself.
				if (!(BeanInfo.class).isAssignableFrom(introspectClass)) {
					// 3. Can this class be found in the Beaninfo searchpath.
					String[] searchPath = Introspector.getBeanInfoSearchPath();
					int startClassname = introspectClass.getName().lastIndexOf(".")+1; //$NON-NLS-1$
					String biName = "."+introspectClass.getName().substring(startClassname)+"BeanInfo"; //$NON-NLS-1$ //$NON-NLS-2$
					for (int i=0; i<searchPath.length; i++) {
						if (classExists(searchPath[i]+biName, introspectClass)) {
							found = true;
							break;
						}
					}
				} else
					found = true;
			} else
				found = true;
			
			if (!found)
				return null;
		}
		
		BeanInfo bInfo = Introspector.getBeanInfo(introspectClass);
		Class superClass = introspectClass.getSuperclass();
		if (superClass == null)
			return new ModelingBeanInfo(bInfo);
		else
			return new ModelingBeanInfo(bInfo, Introspector.getBeanInfo(superClass));
	}
	
	/**
	 * See if this class exists, first in the class loader of the sent class,
	 * then in the system loader, then the bootstrap loader, and finally the
	 * current thread context class loader.
	 */
	protected static boolean classExists(String className, Class fromClass) {
		if (fromClass.getClassLoader() != null)
			try {
				fromClass.getClassLoader().loadClass(className);
				return true;
			} catch (ClassNotFoundException e) {
			}
		if (ClassLoader.getSystemClassLoader() != null)
			try {
				ClassLoader.getSystemClassLoader().loadClass(className);
				return true;
			} catch (ClassNotFoundException e) {
			}
		try {
			Class.forName(className);
			return true;
		} catch (ClassNotFoundException e) {
		}
		
		try {
	        // Use the classloader from the current Thread.
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			Class cls = cl.loadClass(className);
			return true;
		} catch (ClassNotFoundException e) {
		}
		
		return false;
		
	}
	
	/**
	 * Used only for Object since that is the only bean that doesn't
	 * have a superclass. Superclass beaninfo required for all other
	 * classes. If this is constructed then this means no merge
	 * and the list is definitive.
	 */
	protected ModelingBeanInfo(BeanInfo beanInfo) {
		fTargetBeanInfo = beanInfo;
	}	
	
	protected ModelingBeanInfo(BeanInfo beanInfo, BeanInfo superBeanInfo) {
		this(beanInfo);
		
		// Now go through the beaninfo to determine the merge state.
		// The default is no merge.
		
		fEventSets = (EventSetDescriptor[]) stripList(beanInfo.getEventSetDescriptors(), superBeanInfo.getEventSetDescriptors());
		fMergeSuperEvents = (fEventSets != null);	// There were some left over, so merge.
		
		fMethods = (MethodDescriptor[]) stripList(beanInfo.getMethodDescriptors(), superBeanInfo.getMethodDescriptors());
		fMergeSuperMethods = (fMethods != null);	// There were some left over, so merge.
	
		fProperties = (PropertyDescriptor[]) stripList(beanInfo.getPropertyDescriptors(), superBeanInfo.getPropertyDescriptors());
		fMergeSuperProperties = (fProperties != null);	// There were some left over, so merge.
		
	}
	
	/**
	 * This is an internal method to go through the super list
	 * and remove any identical ones from the full list. As soon
	 * as one in super is not found, then no change is made and
	 * it is assumed the list is definitive.
	 *
	 * It will return the new array to use, and it means that
	 * the merge flag should be set to true. If it returns null,
	 * then use the full list as is and the merge flag should
	 * be set to false.
	 */
	protected FeatureDescriptor[] stripList(FeatureDescriptor[] fullList, FeatureDescriptor[] stripList) {
		// The process is to create a boolean list mirroring the fullList.
		// This boolean list indicates if the corresponding (by index)
		// entry from the fullList is to be copied to the final computed
		// list.
		// A Hashmap is created where the key is the feature name from the fullList
		// and the value is the index into the fullList. Unfortunately, with
		// MethodDescriptor's the "name" is not unique if there is more than
		// one method with the same name but different arguments. So in this
		// case the value will be an ArrayList containing all of the indexes of
		// the matching entries.
		//
		// Then the stripList will be stepped through and see if there is
		// an entry in the Hashmap for it. If there is an entry, then
		// the entry is checked to see if it is semantically equal.
		// If it is, then the boolean list entry is marked so that
		// the entry won't be copied over and the counter
		// of the number of entries in the fullList copy is decremented.
		// If they aren't semantically equal, then we know that this is
		// an override. In that case, the entry is kept, but we don't
		// prevent merge later. If there are more than one with the
		// same name (i.e. the Hashmap value is an ArrayList), then 
		// Only if any one of them is semantically equal we will remove
		// the entry.
		//
		// If the stripList entry is not found in the Hashmap, then the
		// routine is immediately ended with null returned because it means
		// that there are more in the stripList that aren't in the fullList.
		//
		// If we get all of the way through, then we know that what is left
		// is the new list.
		//
		// We will create a new list of all that is left.
		//
		// All of this is based upon the assumption that the list can
		// get quite large (which it does for javax.swing) and that
		// a simple n-squared order search would be too slow.
		
		if (
			(fullList == null ||	fullList.length == 0) || 
			(stripList == null || stripList.length == 0) || 
			fullList.length < stripList.length)
			return null;	// With this we know already that not all of the stripList is in fullList, or there is nothing to inherit.
			
		// Create a working copy of the FeatureDescriptorEquality for fullList and stripList and just reuse them
		FeatureDescriptorEquality workingStrip = FeatureDescriptorEquality.createEquality(stripList[0]);
		FeatureDescriptorEquality workingFull = FeatureDescriptorEquality.createEquality(fullList[0]);		
		
		boolean[] dontCopy = new boolean[fullList.length];
		int fullSize = fullList.length;
		HashMap fullMap = new HashMap(fullSize);
		for (int i=0; i<fullList.length; i++) {
			String key = fullList[i].getName();
			Object value = fullMap.get(key);
			if (value == null)
				fullMap.put(fullList[i].getName(), new Integer(i));
			else {
				if (!(value instanceof ArrayList)) {
					ArrayList newValue = new ArrayList(3);
					newValue.add(value);
					fullMap.put(key, newValue);
					value = newValue;
				}
				((ArrayList) value).add(new Integer(i));
			}
				
		}

		for (int i=0; i < stripList.length; i++) {
			Object index = fullMap.get(stripList[i].getName());
			if (index == null)
				return null;	// Entry not found in full list.
			workingStrip.setFeature(stripList[i]);
			if (!(index instanceof ArrayList)) {			
				workingFull.setFeature(fullList[((Integer)index).intValue()]);
				if (workingFull.equals(workingStrip)) {
					// They are semantically identical, so remove from list				
					fullSize--;
					dontCopy[((Integer)index).intValue()] = true;	// Remove it from the list
				}
			} else {
				ArrayList r = (ArrayList) index;
				for (int ri=0; ri<r.size(); ri++) {
					Integer rIndex = (Integer) r.get(ri);
					workingFull.setFeature(fullList[rIndex.intValue()]);
					if (workingFull.equals(workingStrip)) {
						// They are semantically identical, so remove from list				
						fullSize--;
						dontCopy[rIndex.intValue()] = true;	// Remove it from the list
						break;
					}
				}
			}		
		}
		
		FeatureDescriptor[] result = (FeatureDescriptor[]) java.lang.reflect.Array.newInstance(fullList.getClass().getComponentType(), fullSize);
		if (fullSize == 0) {
			// We've removed them all. This could only happen if the lists where identical
			// because if fullList was smaller we would of already left the routine. So
			// fullList must of been the same size or larger than stripList. If stripList
			// was larger we would of already left in the loop. So the lists must of matched
			// identically.
			//
			// We will return a 0-length array so that we know that merge is to be done,
			// but there are no entries at this level.
			return result;
		}
		
		// Copy over the remaining entries.
		int iTo = 0;
		for (int i=0; i<fullList.length; i++)
			if (!dontCopy[i])
				result[iTo++] = fullList[i];
		
		return result;
	}
	
	/**
	 * If this returns true, then all of the super class's
	 * events should be merged in. If it returns false,
	 * then the events returned are it, there are no
	 * others.
	 */
	public boolean isMergeSuperEvents() {
		return fMergeSuperEvents;
	}
	
	/**
	 * If this returns true, then all of the super class's
	 * methods should be merged in. If it returns false,
	 * then the methods returned are it, there are no
	 * others.
	 */	
	public boolean isMergeSuperMethods() {
		return fMergeSuperMethods;
	}
	
	
	/**
	 * If this returns true, then all of the super class's
	 * properties should be merged in. If it returns false,
	 * then the properties returned are it, there are no
	 * others.
	 */	
	public boolean isMergeSuperProperties() {
		return fMergeSuperProperties;
	}
	
    public BeanInfo[] getAdditionalBeanInfo() {
    	return fTargetBeanInfo.getAdditionalBeanInfo();
    }
    
    public BeanDescriptor getBeanDescriptor() {
    	return fTargetBeanInfo.getBeanDescriptor();
    }
    
    public EventSetDescriptor[] getEventSetDescriptors() {
    	return fMergeSuperEvents ?
    		fEventSets : fTargetBeanInfo.getEventSetDescriptors();
    }
    
    public java.awt.Image getIcon(int iconKind) {
    	return fTargetBeanInfo.getIcon(iconKind);
    }
    
    public MethodDescriptor[] getMethodDescriptors() {
    	return fMergeSuperMethods ?
    		fMethods : fTargetBeanInfo.getMethodDescriptors();
    }
    
    public PropertyDescriptor[] getPropertyDescriptors() {
    	return fMergeSuperProperties ?
    		fProperties : fTargetBeanInfo.getPropertyDescriptors();
    }
}