package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cdm.*;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.CompoundCommand;
import com.ibm.etools.cde.commands.*;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import java.util.*;
import com.ibm.etools.cde.emf.IEMFListener;
import com.ibm.etools.cde.emf.EMFListenerAdapter;
import com.ibm.etools.emf.notify.Notifier;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
/**
 * This is a static policy for dealing with annotations.
 * It provides utility methods so that annotations are handled
 * consistently.
 */
public class AnnotationPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private static final RefStructuralFeature 
		SF_ANNOTATES,
		SF_ANNOTATIONS,
		SF_DIAGRAM,
		SF_VISUALINFOS;
	
	static {
		SF_ANNOTATES = CDEUtilities.CDM_PACKAGE.getAnnotationEMF_Annotates();
		SF_ANNOTATIONS = CDEUtilities.CDM_PACKAGE.getDiagramData_Annotations();
		SF_DIAGRAM = CDEUtilities.CDM_PACKAGE.getVisualInfo_Diagram();
		SF_VISUALINFOS = CDEUtilities.CDM_PACKAGE.getAnnotation_VisualInfos();
	}
	
	/**
	 * Create an annotation. It will return the new annotation.
	 * This allows the caller to customize the annotation that will be added when
	 * commands are executed. Since the annotation is new, there is no need for the
	 * customization to be done through commands too.
	 *
	 * Then the annotation needs to be passed into the createAnnotationCommand method
	 * so that a command is created to add it into the system.
	 */
	public static Annotation createAnnotation(Object modelObject) {
		CDMFactory fact = CDEUtilities.CDM_PACKAGE.getCDMFactory();
		if (modelObject instanceof RefObject)
			return fact.createAnnotationEMF();
		else
			return fact.createAnnotationGeneric();
	}
	
	/**
	 * This will return a command to add the annotations into the system.
	 * It will return null if there are no annotations to add. This will
	 * use the default add annotations command that was in the EditDomain.
	 *
	 * Note: This is not to be used in CreateRequest in ContainerEditPolicy,
	 * use the getCreateRequestCommand() in this class to do it, so that the
	 * order is preserved correctly. This is to be used only when annotations
	 * by themselves are being added, and the model object is already in the model.
	 */
	public static Command getDefaultAddAnnotationsCommand(List annotations, EditDomain domain) {
		if (annotations.isEmpty())
			return null;

		try {
			AddAnnotationsCommand applyCmd = (AddAnnotationsCommand) domain.getDefaultAddAnnotationsCommandClass().newInstance();
			applyCmd.setDomain(domain);
			applyCmd.setAnnotations(annotations);
			return applyCmd;
		} catch(InstantiationException e) {
			e.printStackTrace();
		} catch(IllegalAccessException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * This will return a command to apply a new model object to the annotation.
	 * The model should be live so that the connection can be succesfully made.
	 */
	public static Command getApplyModelToAnnotationCommand(Object modelObject, Annotation annotation, AnnotationLinkagePolicy linkage) {
		return new ApplyModelToAnnotationCommand(modelObject, annotation, linkage);
	}
	
	/**
	 * Return a command to delete the annotations from the DiagramData that
	 * they are in. It will also delete any VisualInfos associated with the
	 * annotations. It is assumed the list contains only Annotations.
	 *
	 * It will return null if there are no annotations to delete.
	 *
	 * NOTE: This is not to be used in DeleteDependentRequest in ContainerEditPolicy,
	 * use the getDeleteDependentRequest() in this class to do it, so that the order
	 * is preserved correctly. This is to be used only when annotations by themselves
	 * are being deleted, but the model object is still in the model (i.e. the model object
	 * no longer has an annotation).
	 */
	public static Command getDeleteCommand(List annotations, DiagramData diagramData) {
		if (annotations.isEmpty())
			return null;
		CommandBuilder cBldr = new CommandBuilder(""); //$NON-NLS-1$
		
		List vis = new ArrayList();
		Iterator itr = annotations.iterator();
		while (itr.hasNext()) {
			Annotation annotation = (Annotation) itr.next();
			
			if (annotation instanceof AnnotationEMF) {
				// We want to unset the annotates field. This is so that the annotation is no longer linked to the
				// model object.
				cBldr.cancelAttributeSetting(annotation, SF_ANNOTATES);
			}
			// We don't do anything like this for AnnotationGeneric because not needed. When removed from the DiagramData it will be un-linked.
			
			// Add the VisualInfos to the vis list so that they can be removed from the diagrams later.
			vis.addAll(annotation.getVisualInfos());
		}
		
		// Now remove the annotations. In non-EMF case, this will un-register the ID, and anyone listening will stop listening.
		cBldr.cancelAttributeSettings(diagramData, SF_ANNOTATIONS, annotations);
		
		// Now remove all of the VisualInfos from the Diagrams.
		cBldr.cancelGroupAttributeSetting(vis, SF_DIAGRAM);
		
		return cBldr.getCommand();
	}
	
	/**
	 * This command is used in the DeleteDependentRequest of the ContainerEditPolicy. It makes sure
	 * that the annotations and visual infos are deleted at the right time.
	 */
	public static Command getDeleteDependentCommand(List annotations, Command deleteDependentCommand, DiagramData diagramData) {
		CompoundCommand c = new CompoundCommand();
		c.append(deleteDependentCommand);
		c.append(getDeleteCommand(annotations, diagramData));
		return c.unwrap();
	}
	
	/**
	 * Update the list with all annotations that this model object and children have attached to them.
	 *
	 * For the non-EMF case, this may not return anything because the connections between model and 
	 * annotation may not yet be live, nor may it be possible to find the children of the model object.
	 * The developer in the non-EMF case will have to know this and use another mechanism to find all
	 * of the children.
	 *
	 * It will return the same list, but it allows for a simple top level usage of 
	 * creating the list in the call itself (e.g. List a = getAllAnnotations(new ArrayList(), ...))
	 */
	public static List getAllAnnotations(List annotations, Object modelObject, AnnotationLinkagePolicy policy) {
		Annotation ann = policy.getAnnotation(modelObject);
		if (ann != null)
			annotations.add(ann);
			
		if (modelObject instanceof RefObject) {
			RefObject ro = (RefObject) modelObject;
			Iterator itr = ro.refContains().iterator();
			while (itr.hasNext()) {
				Object n = itr.next();
				if (n instanceof RefObject)
					getAllAnnotations(annotations, n, policy);
			}
		} else {
			// We will try to find containment.
			GenericAnnotationLinkagePolicy gPolicy = (GenericAnnotationLinkagePolicy) policy;
			Iterator itr = gPolicy.getContainedChildren(modelObject).iterator();
			while (itr.hasNext()) {
				getAllAnnotations(annotations, itr.next(), policy);
			}
		}
		
		return annotations;
	}
	
	/**
	 * This command is used in the CreateRequest of the ContainerEditPolicy. It makes sure that
	 * the annotations are added at the right time.
	 */
	public static Command getCreateRequestCommand(List annotations, Command childCreateCommand, EditDomain domain) {
		CompoundCommand c = new CompoundCommand(childCreateCommand.getLabel());
		c.append(getDefaultAddAnnotationsCommand(annotations, domain));
		c.append(childCreateCommand);
		return c.unwrap();
	}
	
	/**
	 * This will return the command to delete the VisualInfo passed in, and if there are no other VisualInfos
	 * or KeyedValues for the annotation, delete the annotation too.
	 *
	 * NOTE: It is important that this command is used to cancel the VisualInfo because it will make sure
	 * that linkages are correctly cleaned up.
	 */
	public static Command cancelVisualInfo(VisualInfo vi) {
		Annotation annotation = (Annotation) vi.refContainer();
		if (annotation.getVisualInfos().size() == 1 && annotation.getKeyedValues().size() == 0) {
			// This is the only Visual Info, and there are no key values, then get rid of the
			// entire annotation.
			return getDeleteCommand(Collections.nCopies(1, annotation), (DiagramData) annotation.refContainer());
		}
		
		// So just cancel the visual info.
		CommandBuilder cmdBldr = new CommandBuilder(null);
		cmdBldr.cancelAttributeSetting(annotation, SF_VISUALINFOS, vi);
		cmdBldr.cancelAttributeSetting(vi, SF_DIAGRAM);
		return cmdBldr.getCommand();
	}
	
	/**
	 * This will return the command to delete the keyed valued from the annotation, and if there are no VisualInfos,
	 * and no other keyed values, it will also cancel the annotation.
	 *
	 * Note: It is important that this command is used to cancel settings on the annotation because it will
	 * cleanup correctly.
	 */
	public static Command cancelAnnotationSetting(Annotation ann, String key) {
		if (!ann.isSetKeyedValue(key))
			return null;	// Not set, so can't cancel.
			
		if (ann.getKeyedValues().size() == 1)
			return getDeleteCommand(Collections.nCopies(1, ann), (DiagramData) ann.refContainer());	// Cancel the entire visual info
		
		// There is more than one, so only cancel the one setting.
		CancelKeyedValueCommand cmd = new CancelKeyedValueCommand();
		cmd.setTarget(ann);
		cmd.setKey(key);
		return cmd;
	}
	
	/**
	 * Create the appropriate command to apply the Annotation setting.
	 * It will create the Annotation if necessary.
	 *
	 * Note: This is important to be used if the annotation may not yet be available. It will make
	 * sure that it is created correctly. If the annotation already exists, then a straight ApplyKeyedValueCommand
	 * could be used, however this is simpler in that it can be applied without worrying if it exists or not.
	 */
	public static Command applyAnnotationSetting(Object modelObject, KeyedValue newValue, EditDomain domain) {
		return new ApplyAnnotationCommand(modelObject, newValue, domain);
	}
	
	/**
	 * Return the IPropertyDescriptors that are applicable to the annotation on the given model object.
	 */
	public static List getAnnotationPropertyDescriptors(Object modelObject, EditDomain domain) {
		Annotation annotation = domain.getAnnotationLinkagePolicy().getAnnotation(modelObject);
		if (annotation != null) {
			ArrayList descs = new ArrayList(2);
			Iterator itr = annotation.getKeyedValues().iterator();
			while (itr.hasNext()) {
				KeyedValue kv = (KeyedValue) itr.next();
				IPropertyDescriptor desc = domain.getKeyedPropertyDescriptor(kv.getKey());
				if (desc != null)
					descs.add(desc);
			}
			return descs;
		}
		return Collections.EMPTY_LIST;
	}
	
	public static final RefObject SF_PARENT_ANNOTATION;
	static {
		SF_PARENT_ANNOTATION = CDEUtilities.CDM_PACKAGE.getAnnotationEMF_Annotates().refOppositeEnd();
	}
	
	/**
	 * A compound listener that listens for Annotation changes. It treats it like it is actually
	 * a property of the model object.
	 *
	 * Note: This shouldn't be used with the same IEMFListener for with VisualInfoPolicy.VisualInfoListener
	 * because when annotations are added/removed that listener will get duplicate notifications, one from
	 * each of these two listeners. If both are desired for the same IEMFListener, then VisualInfoListener
	 * should be used with annotationKeys flag set to true. Then only one notification will be sent.
	 *
	 * It notifies:
	 *   a) Annotations add/removed for the model of interest. Notifier won't be available (because the
	 *      model could be a non-EMF model). SF will be parentAnnotationAttribute.
	 *   b) Any Annotation changes, including KeyValues. Notifier will be Annotation.
	 *   c) If this is an EMF model and notifyModel was set to true, then all model notifications are sent through too.
	 */
	public static class AnnotationListener implements IEMFListener, GenericAnnotationLinkagePolicy.AnnotationLinkageListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected IEMFListener listener;
		protected Object model;
		protected String modelID;
		protected GenericAnnotationLinkagePolicy linkage;
		protected Annotation annotation;
		protected EMFListenerAdapter
			modelAdapter,	// When EMF model
			annotationAdapter;
		protected boolean modelToo;
		
		public AnnotationListener(Object model, EditDomain domain, IEMFListener listener) {
			this(model, domain, listener, true);
		}
				
		public AnnotationListener(Object model, EditDomain domain, IEMFListener listener, boolean modelToo) {
			this.model = model;
			this.modelToo = modelToo;
			this.listener = listener;
			if (model instanceof RefObject)
				setupModelAdapter(domain.getAnnotationLinkagePolicy());
			else
				setupModelListener((GenericAnnotationLinkagePolicy) domain.getAnnotationLinkagePolicy());
		}
				
		/**
		 * Remove all listening.
		 */
		public void removeListening() {
			if (model != null) {
				listener = null;	// We are removing so we don't care to notify more.
				if (model instanceof RefObject) {
					((Notifier) model).removeAdapter(modelAdapter);
					modelAdapter = null;
				} else {
					linkage.removeAnnotationLinkageListener(modelID, this);
					modelID = null;
					linkage = null;
				}
				removeAnnotationAdapter();
				model = null;
			}
		}
		
		private void setupModelAdapter(AnnotationLinkagePolicy link) {
			Notifier ro = (Notifier) model;
			modelAdapter = new EMFListenerAdapter(this);
			modelAdapter.setTarget(ro);
			ro.addAdapter(modelAdapter);
			setupAnnotationAdapter(link.getAnnotation(model));
		}
		
		private void setupModelListener(GenericAnnotationLinkagePolicy linkage) {
			this.linkage = linkage;
			modelID = linkage.getIDFromModel(model);
			linkage.addAnnotationLinkageListener(modelID, this);
			setupAnnotationAdapter(linkage.getAnnotation(model));
		}

		private void setupAnnotationAdapter(Annotation annotation) {
			if (annotationAdapter != null) {
				removeAnnotationAdapter();
			}
			
			this.annotation = annotation;
			if (annotation != null) {
				annotationAdapter = new EMFListenerAdapter(this);
				annotationAdapter.setKeyedValueListening(true);
				annotationAdapter.setTarget(annotation);
				annotation.addAdapter(annotationAdapter);
			}
		}
		
		private void removeAnnotationAdapter() {
			if (annotationAdapter != null) {
				EMFListenerAdapter a = annotationAdapter;
				annotationAdapter = null;	// So we don't get a loop on remove.
				annotation.removeAdapter(a);
				annotation = null;
			}
		}
					
		public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
			if (listener == null)
				return;
			if (notifier == model) {
				if (eventType == Notifier.REMOVING_ADAPTER)
					removeListening();	// My adapter is being removed from the model, so remove all listening.
				else if (sf == SF_PARENT_ANNOTATION) {
					switch (eventType) {
						case Notifier.SET:
							setupAnnotationAdapter((Annotation) newValue);
						case Notifier.TOUCH:							
							listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
							break;
						case Notifier.UNSET:
							setupAnnotationAdapter(null);
							listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
							break;
					}
				} else if (modelToo)
					listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
			} else if (notifier == annotation) {
				if (eventType == Notifier.REMOVING_ADAPTER)
					removeAnnotationAdapter();	// My adapter is being removed from the annotation, so remove listening
				else {
					listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
				}
			}				
		}
		
		public void annotationLinkageChanged(GenericAnnotationLinkagePolicy.AnnotationLinkageEvent event) {
			if (listener == null)
				return;
			if (event.getType() == event.REMOVING_LISTENER)
				removeListening();	// My adapter is being removed from the model, so remove all listening.
			else {
				switch (event.getType()) {
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.SET:
						setupAnnotationAdapter(((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getNewAnnotation());
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.TOUCH:							
						listener.notifyChanged(null, event.getType() == GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.SET ? Notifier.SET : Notifier.TOUCH, SF_PARENT_ANNOTATION, ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getOldAnnotation(), ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getNewAnnotation(), Notifier.NO_INDEX);
						break;
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.UNSET:
						setupAnnotationAdapter(null);
						listener.notifyChanged(null, Notifier.UNSET, SF_PARENT_ANNOTATION, ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getOldAnnotation(), ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getNewAnnotation(), Notifier.NO_INDEX);
						break;
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.SET_ID:
						modelID = (String) event.getSource();	// Get the new id and save it.
						break;
				}
			}
		}
	}

	 				
}