package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import org.eclipse.swt.graphics.Image;
import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.ui.plugin.*;
import org.eclipse.jface.resource.ImageDescriptor;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Hashtable;

import com.ibm.etools.logging.util.*;
import com.ibm.etools.logging.util.IPluginHelper;

import org.eclipse.jface.resource.ImageRegistry;

/**
 * This is the top-level class of the Common Diagram Editor plugin
 */
public final class CDEPlugin extends AbstractUIPlugin implements IPluginHelper{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String SHOW_XML = "SHOW_XML"; //$NON-NLS-1$
	
	private MsgLogger internalMsgLogger = null;

	// Strings needed for the "Adapter by type" extension point.
	public static final String 
		ADAPTER_BY_TYPE_ID = "adapter",	// Extension point id //$NON-NLS-1$
		ADAPTER_ELEMENT = "adapter",	// Adapter element name //$NON-NLS-1$
		DECORATOR_TYPE_CLASS = "typeclass",	// decorator type class key //$NON-NLS-1$
		ADAPTER_CLASS = "adapterclass";	// adapter class key		 //$NON-NLS-1$
		
	private static CDEPlugin CDE_PLUGIN = null;	
	
	public CDEPlugin(IPluginDescriptor pluginDescriptor) {	
		super(pluginDescriptor);
		CDE_PLUGIN = this;
	}
	
	/**
	 * Accessor to return the plugin as the OCM plugin.
	 */
	public static CDEPlugin getPlugin() {
		return CDE_PLUGIN;
	}	
	
	public String getPluginID() {
		return getDescriptor().getUniqueIdentifier();
	}
	
	/**
	 * Handle getting a class from the string. The format of the string is either:
	 *   a) pluginName/className - Find class within the plugin
	 *   b) className - Find class within EMFPlugin classpath.
	 * Note:
	 * Will throw ClassNotFoundException.
	 */
	static public Class getClassFromString(String pluginFormatedClassName) throws ClassNotFoundException {
		return getClassFromString(null, pluginFormatedClassName);
	}
	
	/**
	 * Handle getting a class from the string. The format of the string is either:
	 *   a) pluginName/className - Find class within the plugin
	 *   b) className - Find class within the plugin classpath of the plugin descriptor passed in.
	 *
	 *   If of the form "className:data" or "pluginName/className:data" the ":data" portion will be stripped off.
	 * Note:
	 * Will throw ClassNotFoundException.
	 */
	 static public Class getClassFromString(IPluginDescriptor descriptor, String pluginFormatedClassName) throws ClassNotFoundException {

		int	slashNdx = pluginFormatedClassName.indexOf('/');
		int colonNdx = pluginFormatedClassName.indexOf(':', slashNdx+1);
		String className = colonNdx == -1 ? pluginFormatedClassName : pluginFormatedClassName.substring(0, colonNdx);	// Strip off data portion
		if (slashNdx != -1) {
			String pluginName = className.substring(0, slashNdx);
			className = className.substring(slashNdx+1);
			IPluginRegistry registry = Platform.getPluginRegistry();
			if ( registry.getPluginDescriptor(pluginName) != null ) {
				ClassLoader classLoader = registry.getPluginDescriptor(pluginName).getPluginClassLoader();
				return classLoader.loadClass(className);
			}	
		}

		// No plugin name, use plugin classpath of passed in descriptor.
		return (descriptor == null ? CDE_PLUGIN.getDescriptor() : descriptor).getPluginClassLoader().loadClass(className);
	 }
	 
	/**
	 * This creates an instance of the class from a classname. If the class is an instanceof IExecutableExtension
	 * then the init data, if any, will be passed as a string to it. The config element and attribute name
	 * will be sent in as nulls because this isn't be created from a configuraton element. The default ctor
	 * will be used to instantiate it.
	 *
	 * This is used for things like CellValidatorClassname to allow passing in additional information. For example
	 * the minmax validator can have "min,max" passed in as initialization data to configure the validator for this.
	 */
	static public Object createInstance(IPluginDescriptor descriptor, String pluginFormatedClassName) throws ClassNotFoundException, InstantiationException, IllegalAccessException, CoreException {
		Class c = getClassFromString(descriptor, pluginFormatedClassName);
		Object o = c.newInstance();
		setInitializationData(o, pluginFormatedClassName, null);
		return o;
	 }
	 
	/**
	 * Set initialization data on the object if it is IExecutableExtension. If the className is
	 * passed in, it will look for the first ':' and use the string after that as the data.
	 * If initdata is passed in, it will be used in place of the data from the className.
	 * If there is no initdata in the className and the initData is null, then null is used as the class data.
	 */
	static public void setInitializationData(Object o, String className, Object initData) throws CoreException {
		if (o instanceof IExecutableExtension) {
			int colonNdx = className.indexOf(':');
			((IExecutableExtension) o).setInitializationData(null, null, initData == null ? (colonNdx == -1 ? null : className.substring(colonNdx+1)) : initData);
		}
	 }
	 
	/**
	 * Handle getting an image from a string that represents a URL. I.E. A url
	 * will be made of the string and then the image accessed from that.
	 *
	 * NOTE: This method just creates it. It is the responsibility of the
	 * caller to dispose of the image when it is no longer needed.
	 */
	static public Image getImageFromURLString(String url) {
		try {
			return ImageDescriptor.createFromURL(new URL(url)).createImage();
		} catch (MalformedURLException e) {
			return ImageDescriptor.getMissingImageDescriptor().createImage();
		}	
	}	

	/*
	 * The levels of logging for the MsgLogger are defined in
	 * the class com.ibm.etools.logging.util.Level
	 */
	public MsgLogger getMsgLogger(){
		if(internalMsgLogger==null)
			internalMsgLogger = (MsgLogger) MsgLogger.getFactory().getLogger(
												PluginHelperImpl.getMsgLoggerName(CDE_PLUGIN), 
												CDE_PLUGIN);
		return internalMsgLogger;
	}

	/**
	 * @see IPluginHelper#getMsgLoggerConfig(Plugin)
	 */
	public Hashtable getMsgLoggerConfig(Plugin plugin) {
		return new PluginHelperImpl().getMsgLoggerConfig(this);
	}

	/**
	 * @see IPluginHelper#setMsgLoggerConfig(Hashtable)
	 */
	public void setMsgLoggerConfig(Hashtable msgLoggerConfig) {
		getMsgLogger().setMsgLoggerConfig(msgLoggerConfig);
	}

}


