package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.*;
import com.ibm.etools.cde.commands.*;
import com.ibm.etools.gef.*;
import com.ibm.etools.gef.requests.*;
import com.ibm.etools.cdm.model.*;
import com.ibm.etools.cdm.*;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.gef.editpolicies.AbstractEditPolicy;
import com.ibm.etools.cde.emf.*;
import java.util.*;
import org.eclipse.core.runtime.IAdaptable;
/**
 * XYLayoutEditPolicy for DiagramFigures. The constraint info is stored
 * in the DiagramFigure.
 */
public class DiagramFigureXYLayoutEditPolicy extends XYLayoutEditPolicy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ContainerPolicy containerPolicy;		// Handles the containment functions
	
/**
 * Create with the container policy for handling DiagramFigures
 */
public DiagramFigureXYLayoutEditPolicy(ContainerPolicy containerPolicy) {
	this.containerPolicy = containerPolicy;
}

public void activate() {
	super.activate();
	containerPolicy.setContainer(getHost().getModel());
}

public void deactivate() {
	super.deactivate();
	containerPolicy.setContainer(null);
}
	
/**
 * The child editpart is about to be added to the parent.
 * The child is an existing child that was orphaned from a previous parent.
 */
protected Command createAddCommand(EditPart childEditPart, Object constraint) {
	// We have to first see if we can add the component. If we can, then we can continue.
	// The child in this case should be a DiagramFigure.
	KeyedValueHolder child = (KeyedValueHolder) childEditPart.getModel();
	GroupRequest grpReq = new GroupRequest(RequestConstants.REQ_ADD);
	grpReq.setEditParts(childEditPart);
	Command addContributionCmd = containerPolicy.getCommand(grpReq);
	if (addContributionCmd == null || !addContributionCmd.canExecute())
		return UnexecutableCommand.INSTANCE;	// It can't be added.
		
	CompoundCommand command = new CompoundCommand("");		 //$NON-NLS-1$
	command.append(primChangeConstraintCommand(child, (Rectangle) constraint, true, true));
	command.append(addContributionCmd);

	return command.unwrap();
}


protected Command createChangeConstraintCommand(EditPart childEditPart, Object constraint, boolean moved, boolean resized) {
	return primChangeConstraintCommand((KeyedValueHolder) childEditPart.getModel(), (Rectangle) constraint, moved, resized);
}

/**
 * A new child is being added to the parent. It isn't yet in the model.
 */
protected Command getCreateCommand(CreateRequest aRequest) {
	DiagramFigure child = (DiagramFigure) aRequest.getNewObject();
	
	Command createContributionCmd = containerPolicy.getCommand(aRequest);
	if (createContributionCmd == null || !createContributionCmd.canExecute())
		return UnexecutableCommand.INSTANCE;	// It can't be created

	Rectangle constraint = (Rectangle) translateToModelConstraint(getConstraintFor(aRequest));			
	CompoundCommand command = new CompoundCommand("");		 //$NON-NLS-1$
	command.append(primChangeConstraintCommand(child, constraint, true, isChildResizeable(getCreateConstraintHandler(child))));
	command.append(createContributionCmd);

	return command.unwrap();
}

/*
 * Get the IConstraintHandler for a created child.
 */
protected IConstraintHandler getCreateConstraintHandler(Object child) {
	IModelAdapterFactory factory = CDEUtilities.getModelAdapterFactory(getHost());
	return factory != null ? (IConstraintHandler) factory.getAdapter(child, IConstraintHandler.class) : null;
}

/*
 * Get the IConstraintHandler for the edit part.
 */
protected IConstraintHandler getConstraintHandler(EditPart editpart) {
	return (IConstraintHandler) ((IAdaptable) editpart).getAdapter(IConstraintHandler.class);
}


/**
 * A child is being deleted from the model.
 */
protected Command getDeleteDependantCommand(Request aRequest) {
	Command deleteContributionCmd = containerPolicy.getCommand(aRequest);
	if ( deleteContributionCmd == null )
		return UnexecutableCommand.INSTANCE;	// It can't be deleted
		
	return deleteContributionCmd;		
}

/**
 * getOrphanChildCommand: About to remove a child from the model
 * so that it can be added someplace else.
 *
 * Remove the visual constraint since it may not be appropriate in 
 * the new position.
 */
protected Command getOrphanChildrenCommand(Request aRequest) {
	// Now get the orphan command for the child.
	Command orphanContributionCmd = containerPolicy.getCommand(aRequest);
	if (orphanContributionCmd == null || !orphanContributionCmd.canExecute())
		return UnexecutableCommand.INSTANCE;	// It can't be orphaned		

	CompoundCommand command = new CompoundCommand("");	 //$NON-NLS-1$
	GroupRequest gr = (GroupRequest) aRequest;
	Iterator children = gr.getEditParts().iterator();
	while(children.hasNext()) {
		EditPart childEditPart = (EditPart) children.next();
		DiagramFigure child = (DiagramFigure) childEditPart.getModel();
		// Delete the visual constraint, if any.
		CancelVisualConstraintCommand delCommand = new CancelVisualConstraintCommand();
		delCommand.setTarget(child);
		command.append(delCommand);
	}
	command.append(orphanContributionCmd);
	return command;
}

protected Command primChangeConstraintCommand(KeyedValueHolder info, Rectangle constraint, boolean pointChanged, boolean sizeChanged) {
	KeyedValue constraintKV = info.getKeyedValue(CDMModelConstants.VISUAL_CONSTRAINT_KEY);
	Rectangle lConstraint = constraintKV instanceof KeyedConstraint ? ((KeyedConstraint) constraintKV).getValue() : null;
	pointChanged |= (lConstraint == null);	// The location also changed if there wasn't currently a setting.
	sizeChanged |= (lConstraint == null);	// The size also changed if there wasn't currently a setting.
	if (pointChanged || sizeChanged) {
		// We have actually moved/sized some.
		// Update the keyed value
		
		ApplyVisualConstraintCommand viUpdate = new ApplyVisualConstraintCommand(""); //$NON-NLS-1$
		viUpdate.setTarget(info);
		if (pointChanged && sizeChanged)
			viUpdate.setRectangle(constraint);
		else if (pointChanged)
			viUpdate.setLocation(new Point(constraint.x, constraint.y));
		else
			viUpdate.setSize(new Dimension(constraint.width, constraint.height));
		return viUpdate;
	}
	
	return null;
}

public Command getCommand(Request request){
	if (REQ_DELETE.equals(request.getType()) || REQ_ANCESTOR_DELETED.equals(request.getType()))
		return containerPolicy.getCommand(request);
	return super.getCommand(request);
}

/**
 * Update ViewObject LayoutConstraints
 * based on model changes. This edit policy will of added a child
 * editpolicy that listens for changes in the VisualConstraint. It will
 * call this method to let us know that the constraint has changed.
 * It will also be called when the child editpolicy has been activated
 * with the current contents of the constraint so that the initial
 * setting may be made.
 */
protected void refreshFromEditPart (EditPart child, KeyedValue constraintValue) {
	com.ibm.etools.draw2d.geometry.Rectangle rect = null;
	Rectangle constraint = (constraintValue instanceof KeyedConstraint) ? ((KeyedConstraint) constraintValue).getValue() : null;
	if (constraint == null)
		rect = new com.ibm.etools.draw2d.geometry.Rectangle(Integer.MIN_VALUE, Integer.MIN_VALUE, -1, -1);	// No constraint, so set to complete default.
	else
		rect = (com.ibm.etools.draw2d.geometry.Rectangle) translateToFigureConstraint(constraint);

	setConstraintToFigure(child, rect);
}

protected void setConstraintToFigure(EditPart child, com.ibm.etools.draw2d.geometry.Rectangle figureConstraint) {
	IConstraintHandler handler = getConstraintHandler(child);
	if (handler != null) {
		int x = figureConstraint.x;	// So that any changes are restored after contribute.
		int y = figureConstraint.y;
		handler.contributeFigureSize(figureConstraint);
		figureConstraint.setLocation(x, y);
	}
	((GraphicalEditPart) getHost()).setLayoutConstraint(child, ((GraphicalEditPart) child).getFigure(), figureConstraint);
}

/**
 * Translate from the figure constraint (it is assumed that zoom/grid already taken out of the picture).
 */
protected Object translateToModelConstraint(Object figureConstraint){
	com.ibm.etools.draw2d.geometry.Rectangle r = (com.ibm.etools.draw2d.geometry.Rectangle) figureConstraint;	
	return new Rectangle(r.x,r.y,r.width,r.height);
}

/**
 * Translate to figure constraint, need to take zoom into account.
 */
protected Object translateToFigureConstraint(Object modelConstraint){
	if (modelConstraint == null)
		return null;
	Rectangle r = (Rectangle) modelConstraint;
	com.ibm.etools.draw2d.geometry.Rectangle figureConstraint = new com.ibm.etools.draw2d.geometry.Rectangle(r.x,r.y,r.width,r.height);
	ZoomController zoomController = getZoomController();
	if (zoomController != null) {
		if (figureConstraint.x != Integer.MIN_VALUE)
			figureConstraint.x = zoomController.zoomCoordinate(figureConstraint.x);
		if (figureConstraint.y != Integer.MIN_VALUE)
			figureConstraint.y = zoomController.zoomCoordinate(figureConstraint.y);
	}
	return figureConstraint;
}

protected boolean isChildResizeable(EditPart childEditpart) {
	return isChildResizeable(getConstraintHandler(childEditpart));
}

protected boolean isChildResizeable(IConstraintHandler handler) {
	return (handler != null) ? handler.isResizeable() : false;
}

protected Object getChildConstraint(EditPart child) {
	return getChildConstraint(child, ((KeyedValueHolder) child.getModel()).getKeyedValue(CDMModelConstants.VISUAL_CONSTRAINT_KEY));
}

protected Object getChildConstraint(EditPart child, KeyedValue kv) {
	return kv instanceof KeyedConstraint ? new Rectangle(((KeyedConstraint) kv).getValue()) : new Rectangle(Integer.MIN_VALUE, Integer.MIN_VALUE, -1, -1);
}

/**
 * A child has been added. We need to add in the listener for Visual Constraint changes. We will
 * also have an editpolicy on the child to handle this.
 */
public static final String VISUAL_CONSTRAINT_REFRESH_POLICY = "com.ibm.etools.cde.visualConstraintRefreshPolicy"; //$NON-NLS-1$
protected static RefObject constraintSF = KeyedValueNotificationHelper.getFeature(CDMModelConstants.VISUAL_CONSTRAINT_KEY);
protected class VisualConstraintRefreshPolicy extends AbstractEditPolicy implements IEMFListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		private EMFListenerAdapter adapter;
		
		public void activate() {
			super.activate();
			// Add listener to visual constraint.
			adapter = getAdapter();
			KeyedValueHolder target = (KeyedValueHolder) getHost().getModel();
			adapter.setTarget(target);
			target.addAdapter(adapter);
			// Signal an initial change so that this child gets positioned correctly.
			KeyedValue kv = target.getKeyedValue(CDMModelConstants.VISUAL_CONSTRAINT_KEY);
			refreshFromEditPart(getHost(), kv);
		}
		
		public void deactivate() {
			super.deactivate();
			if (adapter != null) 
				adapter.getTarget().removeAdapter(adapter);
			adapter = null;
		}
		
		public EditPart getTargetEditPart(Request request){
			return null;
		}
		
		protected EMFListenerAdapter getAdapter() {
			if (adapter == null) {
				adapter = new EMFListenerAdapter(this, true);
			}
			return adapter;
		}
		
		public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {						
			if (sf == constraintSF) {
				switch (eventType) {								
					case Notifier.SET:	// It was changed.
					case Notifier.TOUCH:	// It was touched.
						refreshFromEditPart(getHost(), (KeyedValue) newValue);
						break;
																
					case Notifier.UNSET:	// It was removed
						refreshFromEditPart(getHost(), null);
						break;
				}
			}
		}
		
};
protected void decorateChild(EditPart child){
	super.decorateChild(child);
	child.installEditPolicy(VISUAL_CONSTRAINT_REFRESH_POLICY, new VisualConstraintRefreshPolicy());
}

protected void undecorateChild(EditPart child){
	super.undecorateChild(child);
	child.removeEditPolicy(VISUAL_CONSTRAINT_REFRESH_POLICY);
}

}


