package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPartViewer;
import org.eclipse.ui.IEditorPart;
import com.ibm.etools.cde.emf.EMFGraphicalEditorPart;
import com.ibm.etools.draw2d.*;
/**
 * The grid controller. It maintains the current grid
 * information.
 *
 * It also notifies when any of the grid properties
 * change.
 *
 * To find the GridController, it is stored in the
 * EditDomain under the viewer data for the viewer
 * in question. The key is the GRID_KEY defined within
 * this class.
 *
 * For Grid to work the following must be done:
 *
 *  1) The layoutEditPolicy upon activation will need to:
 *     a) Create a GridFigure and add it at the end of the
 *        EditPart's contentPane,
 *        it should pass in the zoomController if it also
 *        can process zooming.
 *     b) Add itself as a listener to the GridController for
 *        the current viewer.
 *     c) Register the edit part as being gridable with the GridController.      
 *  2) Upon any GridController notification, it should call repaint on the GridFigure
 *     so that it is rescheduled to 
 *     draw the new grid. For grid visibility, it should set the GridFigure visibility
 *     appropriately.
 *  4) Upon deactivation it should:
 *     a) remove itself as a listener from the
 *        GridController.
 *     b) Remove the GridFigure from the 
 *        EditPart contentPane.
 *     c) Remove the edit part from the registered list.
 *
 */
public class GridController {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String GRID_KEY = "com.ibm.etools.cde.gridkey"; //$NON-NLS-1$

	protected List listeners = new ArrayList(5);
	protected HashSet registeredEPs = new HashSet(2);
	
	private boolean showGrid = false;
	private int gridWidth = 20;
	private int gridHeight = 20;
	private int gridMargin = 0; 
	
	/**
	 * Static helper method to return the grid controller from the active editor, if any in the primary viewer is griddable
	 */
	public static GridController getGridController(IEditorPart activeEditor){
	
		EditPartViewer viewer = (EditPartViewer) activeEditor.getAdapter(com.ibm.etools.gef.GraphicalViewer.class);
		if (viewer != null) {
			EditDomain dom = (EditDomain) viewer.getEditDomain();
			return (GridController) dom.getViewerData(viewer, GRID_KEY);
		}
		return null;
	}
	
	/**
	 * Static helper method to return the grid controller from the editpart, if any in the viewer is griddable.
	 */
	public static GridController getGridController(EditPart editPart){	
		EditPartViewer viewer = editPart.getRoot().getViewer();
		EditDomain dom = (EditDomain) viewer.getEditDomain();
		return (GridController) dom.getViewerData(viewer, GRID_KEY);
	}
		
	/**
	 * Add a Grid Listener.
	 * Note: Not thread safe, do not add/remove a listener
	 * during grid notification.
	 */
	public void addGridListener(IGridListener listener) {
		listeners.add(listener);
	}
	
	/**	
	 * Remove a Grid Listener.
	 * Note: Not thread safe, do not add/remove a listener
	 * during grid notification.
	 */
	public void removeGridListener(IGridListener listener) {
		listeners.remove(listener);
	}
	
	
	/**
	 * Get the grid height.
	 */
	public int getGridHeight() {
		return gridHeight;
	}
	
	/**
	 * Set the grid height.
	 */
	public void setGridHeight(int gridHeight) {
		int oldGridHeight = this.gridHeight;
		this.gridHeight = gridHeight;
		for (int i=0; i<listeners.size(); i++)
			((IGridListener) listeners.get(i)).gridHeightChanged(gridHeight, oldGridHeight);
	}
	
	/**
	 * Get the grid width.
	 */
	public int getGridWidth() {
		return gridWidth;
	}
	
	/**
	 * Set the grid width.
	 */
	public void setGridWidth(int gridWidth) {
		int oldGridWidth = this.gridWidth;
		this.gridWidth = gridWidth;
		for (int i=0; i<listeners.size(); i++)
			((IGridListener) listeners.get(i)).gridWidthChanged(gridWidth, oldGridWidth);
	}
	
	/**
	 * Get the grid margin.
	 */
	public int getGridMargin() {
		return gridMargin;
	}
	
	/**
	 * Set the grid margin.
	 */
	public void setGridMargin(int gridMargin) {
		int oldGridMargin = this.gridMargin;
		this.gridMargin = gridMargin;
		for (int i=0; i<listeners.size(); i++)
			((IGridListener) listeners.get(i)).gridMarginChanged(gridMargin, oldGridMargin);
	}		
	
	/**
	 * Get the grid visibility.
	 */
	public boolean isGridShowing() {
		return showGrid;
	}
	
	/**
	 * Set the grid visibility.
	 */
	public void setGridShowing(boolean showGrid) {
		this.showGrid = showGrid;
		for (int i=0; i<listeners.size(); i++)
			((IGridListener) listeners.get(i)).gridVisibilityChanged(showGrid);
	}
	
	/**
	 * Register the Editpart as having a grid on it.
	 */
	public void registerEditPart(EditPart ep) {
		registeredEPs.add(ep);
	}
	
	/**
	 * Is this EditPart registered?
	 */
	public boolean isEditPartRegistered(EditPart ep) {
		return registeredEPs.contains(ep);
	}
	
	/**
	 * Unregister the EditPart.
	 */
	public void unregisterEditPart(EditPart ep) {
		registeredEPs.remove(ep);
	}
}