package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.notify.impl.NotificationImpl;
import com.ibm.etools.cdm.*;
/**
 * This class contains static helper methods to
 * help with KeyedValue notifications. Such as
 * making notifications of KeyedValue changes
 * look like standard property changes.
 */
public class KeyedValueNotificationHelper {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected static HashMap keyedSF = new HashMap();	// Map of keys to Structural Features for those keys.
	protected static final EStructuralFeature keyedValues;
	static {
		keyedValues = CDEUtilities.CDM_PACKAGE.getKeyedValueHolder_KeyedValues();
	}
	
	/**
	 * This is used to return a structural feature for a key. It allows equality
	 * testing within the notify method for a keyed feature only. refName()
	 * will return the key as the value.
	 *
	 * This method will return a cached copy for each key so that
	 * it can be assured that this same feature will always be
	 * returned for this key.
	 */
	public static RefStructuralFeature getFeature(String key) {
		EAttribute f = (EAttribute) keyedSF.get(key);
		if (f == null) {
			EcoreFactory fact = ((EcorePackage) RefRegister.getPackage(EcorePackage.packageURI)).getEcoreFactory();
			f = fact.createEAttribute();
			f.setName(key);
			f.setETypeClassifier(keyedValues.getETypeClassifier());	// Set the type to be a KeyedValue.
			keyedSF.put(key, f);
		}
		return f;
	}
	
	/**
	 * Adapter that are interested in knowing when a specific keyedValue
	 * has been changed should call this method. It will call back on 
	 * the adapter's notify method with the appropriate keyed SF.
	 *
	 * It will call notifyChanged back on your adapter with:
	 *   The correct transaction type (i.e. SET/UNSET/TOUCH),
	 *   The SF will be the feature from getFeature above,
	 *   The oldValue and newValue will be the correct KeyedValue.
	 *
	 * If this is not a KeyedValue feature, then it will return false.
	 *
	 * Usage in an adapter is:
	 *   notifyChanged(notifier, event, sf, old, new, pos) {
	 *     if (KeyedValueNofificationHelper.notifyChanged(this, notifier, event, sf, old, new, pos))
	 *       return;  // It was handled
	 */
	 public static boolean notifyChanged(Adapter adapter, Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int position) {
	 	if (sf != keyedValues)
	 		return false;
	 		
	 	// Adds and removes will be treated as Set and Unset because we are treating these as single valued features.
	 	RefStructuralFeature ksf;
		switch (eventType) {
			case Notifier.ADD_MANY:
				// There where several added at once, notify for each.
				Iterator itr = ((List) newValue).iterator();
				while (itr.hasNext()) {
					KeyedValue kv = (KeyedValue) itr.next();
					ksf = getFeature(kv.getKey());
					adapter.notifyChanged(new NotificationImpl(notifier, Notifier.SET, ksf, null, kv, Notifier.NO_INDEX));
				}
				break;
				
			case Notifier.TOUCH:	// It was touched.
			case Notifier.ADD:	// It was added.
			case Notifier.SET:	// It was replaced.
				ksf = getFeature(((KeyedValue) newValue).getKey());
				adapter.notifyChanged(new NotificationImpl(notifier, eventType != Notifier.TOUCH ? Notifier.SET : Notifier.TOUCH, ksf, oldValue, newValue, Notifier.NO_INDEX));
				break;
				
			case Notifier.REMOVE_MANY:
				// There where several removed at once, notify for each.
				itr = ((List) oldValue).iterator();
				while (itr.hasNext()) {
					KeyedValue kv = (KeyedValue) itr.next();
					ksf = getFeature(kv.getKey());
					adapter.notifyChanged(new NotificationImpl(notifier, Notifier.UNSET, ksf, kv, null, Notifier.NO_INDEX));
				}
				break;
				
			case Notifier.REMOVE:
				ksf = getFeature(((KeyedValue) oldValue).getKey());
				adapter.notifyChanged(new NotificationImpl(notifier, Notifier.UNSET, ksf, oldValue, null, Notifier.NO_INDEX));
				break;
				
			case Notifier.MOVE:
				break;	// Moving a keyedvalue from one position to another is not considered a change to notify about.
				
			default:
				return false;	// We didn't handle it.
		}
		
		return true;
	 }
					
}