package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editpolicies.BendpointEditPolicy;
import com.ibm.etools.gef.requests.BendpointRequest;
import com.ibm.etools.gef.GraphicalEditPart;
import com.ibm.etools.draw2d.Connection;
import com.ibm.etools.draw2d.AbsoluteBendpoint;
import com.ibm.etools.cdm.*;
import com.ibm.etools.cdm.model.CDMModelConstants;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.gef.EditPartViewer;
import com.ibm.etools.cde.emf.IEMFListener;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.cdm.model.Point;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.cde.commands.CancelKeyedValueCommand;
import com.ibm.etools.cdm.model.CDMModelConstants;
/**
 * This editpolicy handles changing the bendpoints of a connection.
 * It is used when the bendpoints are stored in a visual info of
 * connection model, which is the host of this policy.
 * 
 * It will handle the bendpoints entirely. It will listen for changes
 * to the bendpoints, and when they do, refresh the connection.
 */

public class VisualBendpointEditPolicy extends BendpointEditPolicy implements IZoomListener, IEMFListener {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected ZoomController zoomController;
	protected VisualInfoPolicy.VisualInfoListener viListener;
	
	public void activate() {
		super.activate();
		
		zoomController = ZoomController.getZoomController(getHost());
		if (zoomController != null) {
			zoomController.addZoomListener(this);
			zoomChanged(zoomController.getZoomValue(), 0);
		}

		// Add listener to visual constraint.
		EditPartViewer viewer = getHost().getRoot().getViewer();
		EditDomain dom = (EditDomain) viewer.getEditDomain();
		viListener = new VisualInfoPolicy.VisualInfoListener(getHost().getModel(), dom.getDiagram(viewer), dom, this);		
		refreshBendpoints();
	}
		
	public void deactivate() {
		if (zoomController != null) {
			zoomController.removeZoomListener(this);
			zoomController = null;
		}		
		
		if (viListener!= null) 
			viListener.removeListening();
		viListener = null;
		
		super.deactivate();
	}
	
	protected static RefObject bendpointsSF = KeyedValueNotificationHelper.getFeature(CDMModelConstants.VISUAL_BENDPOINTS_KEY);
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {						
		if (sf == bendpointsSF) {
			// The bendpoints keyedvalue was changed
			switch (eventType) {								
				case Notifier.SET:	// It was changed.
				case Notifier.TOUCH:	// It was touched.
					refreshBendpoints((KeyedValue) newValue);
					break;
															
				case Notifier.UNSET:	// It was removed
					refreshBendpoints(null);
					break;
			}
		} else if (sf == VisualInfoPolicy.SF_VISUALINFOS) {
			// A visual info was either added or removed
			refreshBendpoints();
		} else if (sf == VisualInfoPolicy.SF_PARENT_ANNOTATION) {
			// An annotation was either added or removed
			refreshBendpoints();
		}
	}

	
	public void zoomChanged(int newZoom, int oldZoom) {
		refreshBendpoints();	// Zoom has changed, the entire bendpoints must be re-calaculated.
	}
	
	protected void refreshBendpoints() {
		// Get the bendpoints, if any, and send them to the refresh.
		KeyedValue kv = null;
		VisualInfo vi = VisualInfoPolicy.getVisualInfo(getHost());
		if (vi != null) {
			kv = vi.getKeyedValue(CDMModelConstants.VISUAL_BENDPOINTS_KEY);
		}			
		refreshBendpoints(kv);	
	}
	
	protected void refreshBendpoints(KeyedValue kv) {
		List figureConstraint = Collections.EMPTY_LIST;
		if (kv instanceof KeyedPoints) {
			KeyedPoints kp = (KeyedPoints) kv;
			List modelConstraint = kp.getPoints();
			figureConstraint = new ArrayList();
			for (int i=0; i<modelConstraint.size(); i++) {
				Point point = (Point) modelConstraint.get(i);
				AbsoluteBendpoint bp = new AbsoluteBendpoint(zoomCoordinate(point.x), zoomCoordinate(point.y));
				figureConstraint.add(bp);
			}
		}
		((Connection) ((GraphicalEditPart) getHost()).getFigure()).setRoutingConstraint(figureConstraint);
	}
	
	protected Command getCreateBendpointCommand(BendpointRequest request) {
		// The absolute location is the coordinate on the graph viewer
		// We may be zoomed up in which case we need to adjust by the zoom factor, so we get a copy to make changes.
		com.ibm.etools.draw2d.geometry.Point absoluteLocation = request.getLocation().getCopy();
		unzoomPoint(absoluteLocation);
		Point modelPoint = new Point(absoluteLocation.x, absoluteLocation.y);
		
		List currentPoints = getCurrentPoints();
		if (currentPoints == null) {
			currentPoints = new ArrayList(1);
			currentPoints.add(modelPoint);
		} else {
			currentPoints = new ArrayList(currentPoints);	// We will be modifying it.
			currentPoints.add(request.getIndex(), modelPoint);
		}
		
		return changeBendpointsCommand(currentPoints);
	}
	
	protected Command getMoveBendpointCommand(BendpointRequest request) {
		List currentPoints = (List) getCurrentPoints();
		if (currentPoints == null)
			return null;	// We have no points to move.
			
		currentPoints = (List) new ArrayList(currentPoints);	// We will be modifying it.
			
		// The absolute location is the coordinate on the graph viewer
		// We may be zoomed up in which case we need to adjust by the zoom factor, so we get a copy to make changes.
		com.ibm.etools.draw2d.geometry.Point absoluteLocation = request.getLocation().getCopy();
		unzoomPoint(absoluteLocation);
		Point modelPoint = new Point(absoluteLocation.x, absoluteLocation.y);
		currentPoints.set(request.getIndex(), modelPoint);
		return changeBendpointsCommand(currentPoints);
	}
	
	protected Command getDeleteBendpointCommand(BendpointRequest request) {
		int index = request.getIndex();
		List currentPoints = (List) getCurrentPoints();
		if (currentPoints == null || index >= currentPoints.size())
			return null;	// We have no points to delete, or the point is outside the list.
			
		currentPoints = new ArrayList(currentPoints);	// We will be modifying it.
		currentPoints.remove(index);
		return changeBendpointsCommand(currentPoints);
	}



	/**
	 * Given the list of points, create change request.
	 * The points are in model constraint format, not GEF format.
	 */
	protected Command changeBendpointsCommand(List points) {
		EditPartViewer viewer = getHost().getRoot().getViewer();
		List oldPoints = getCurrentPoints();
		if (!points.equals(oldPoints)) {
			// We have actually changed some.
			// Update the keyed value
			if (!points.isEmpty()) {
				CDMFactory fact = CDEUtilities.CDM_PACKAGE.getCDMFactory();
				KeyedPoints newConstraint = fact.createKeyedPoints();
				newConstraint.setKey(CDMModelConstants.VISUAL_BENDPOINTS_KEY);
				newConstraint.getPoints().addAll(points);
				EditDomain dom = EditDomain.getEditDomain(getHost());
				return VisualInfoPolicy.applyVisualInfoSetting(getHost().getModel(), newConstraint, dom, dom.getDiagram(viewer));
			} else {
				// The points list is empty, so just remove the setting
				VisualInfo vi = VisualInfoPolicy.getVisualInfo(getHost());
				return VisualInfoPolicy.cancelVisualInfoSetting(vi, CDMModelConstants.VISUAL_BENDPOINTS_KEY);
			}
		}
		
		return null;
	
	}
	
	/**
	 * Get the current list.
	 */
	protected List getCurrentPoints() {
		EditPartViewer viewer = getHost().getRoot().getViewer();
		VisualInfo vi = VisualInfoPolicy.getVisualInfo(getHost().getModel(), viewer);
		KeyedValue bendpointsKV = vi != null ? vi.getKeyedValue(CDMModelConstants.VISUAL_BENDPOINTS_KEY) : null;
		List oldPoints = bendpointsKV instanceof KeyedPoints ? ((KeyedPoints) bendpointsKV).getPoints() : null;
		return oldPoints;
	}
	
	/**
	 * Adjust the point for the zoom factor.
	 */
	protected void unzoomPoint(com.ibm.etools.draw2d.geometry.Point p) {
		if (zoomController != null) {
			p.setLocation(zoomController.unzoomCoordinate(p.x), zoomController.unzoomCoordinate(p.y));
		}
	}
	
	/**
	 * Adjust the coordinate for the zoom factor.
	 */
	protected int zoomCoordinate(int coor) {
		return (zoomController == null) ? coor : zoomController.zoomCoordinate(coor);
	}

}