package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.gef.editpolicies.AbstractEditPolicy;
import java.util.*;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPartListener;
import com.ibm.etools.gef.GraphicalEditPart;
import com.ibm.etools.draw2d.IFigure;
import org.eclipse.core.runtime.IAdaptable;
import com.ibm.etools.draw2d.geometry.Rectangle;
import com.ibm.etools.gef.Request;
/**
 * This is an editpolicy that is placed onto a VisualComponent, and that has
 * children that are visual components. What it does is control the figure constraints
 * based upon the children visual component notifications. As the children change position and
 * size, this will receive the changes and change the child's figure constraint to match.
 *
 *
 * It will get the IVisualComponent from the children editparts by asking through the getAdapter() interface.
 * The children should return their visual component. If they need to change the visual component for some
 * reason, they need to deactivate and reactivate their edit policy with the key CONSTRAINT_REFRESH_POLICY
 * (see the declaration in this class). That will cause the visual component to be recycled.
 *
 * NOTE: It is NOT a LayoutEditPolicy. This is because it will not handle add/moving/changing/deleting
 * of children. That is the responsibility of the LayoutEditPolicy. However, this is a sub-function of
 * the LayoutEditPolicy. The layout edit policy usually handles this function too. But for visual components
 * this isn't necessary because they handle those functions themselves. So there will be a LayoutEditPolicy
 * that works in conjunction with this. Its only function will be to handle the models. When the
 * constraints are applied to the model, the model will apply them to visual component, who will then 
 * notify back to this policy that it has changed.
 */
public class VisualComponentsLayoutPolicy extends AbstractEditPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	// Key to use for the edit policy when it is installed on the container.
	public static final String LAYOUT_POLICY = "com.ibm.etools.visualcomponentslayoutpolicy"; //$NON-NLS-1$
	public static final String CONSTRAINT_REFRESH_POLICY = "com.ibm.etools.componentconstraintrefreshpolicy"; //$NON-NLS-1$
	
	private EditPartListener listener;
	
	public void activate(){
		setListener(createListener());
		decorateChildren();
		super.activate();
	}
	
	public void deactivate(){
		undecorateChildren();
		setListener(null);
		super.deactivate();
	}

	public EditPart getTargetEditPart(Request request){
		return null;
	}
	
	protected EditPartListener createListener(){
		return new EditPartListener.Stub(){
			public void childAdded(EditPart child, int index){
				decorateChild(child);
			}
			public void removingChild(EditPart child, int index){
				undecorateChild(child);
			}
		};
	}
	
	protected void setListener(EditPartListener listener){
		if (this.listener != null)
			getHost().removeEditPartListener(this.listener);
		this.listener = listener;
		if (this.listener != null)
			getHost().addEditPartListener(this.listener);
	}
	
	protected void decorateChildren(){
		Iterator children = getHost().getChildren().iterator();
		while (children.hasNext())
			decorateChild((EditPart) children.next());
	}
	
	protected void decorateChild(EditPart child){
		child.installEditPolicy(CONSTRAINT_REFRESH_POLICY, new ConstraintRefreshPolicy());
	}
	
	protected void undecorateChildren(){
		Iterator children = getHost().getChildren().iterator();
		while (children.hasNext())
			undecorateChild((EditPart) children.next());
	}
	
	protected void undecorateChild(EditPart child){
		child.removeEditPolicy(CONSTRAINT_REFRESH_POLICY);
	}
	
	protected class ConstraintRefreshPolicy extends AbstractEditPolicy implements IVisualComponentListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		IVisualComponent visualComponent;
		
		public void activate() {
			super.activate();
			// Add listener to visual component, the visual component is retrieved through the getAdapter interface from the editpart.
			// Signal an initial change so that this child gets positioned correctly.
			visualComponent = (IVisualComponent) ((IAdaptable) getHost()).getAdapter(IVisualComponent.class);
			if (visualComponent != null) {
				visualComponent.addComponentListener(this);
				signalRefresh();
			}
		}
		
		public void deactivate() {
			super.deactivate();
			if (visualComponent!= null) 
				visualComponent.removeComponentListener(this);
			visualComponent = null;
		}
		
		public EditPart getTargetEditPart(Request request){
			return null;
		}
		
		protected void signalRefresh() {
			// Get the current bounds and set the figure's constraint.
			Rectangle bounds = visualComponent.getBounds().getCopy();
			((GraphicalEditPart) VisualComponentsLayoutPolicy.this.getHost()).setLayoutConstraint(getHost(), ((GraphicalEditPart) getHost()).getFigure(), bounds);
		}
		
		public void componentMoved(int x, int y) {
			IFigure child = ((GraphicalEditPart) getHost()).getFigure();
			IFigure parent = ((GraphicalEditPart) VisualComponentsLayoutPolicy.this.getHost()).getFigure();
			Rectangle bounds = ((Rectangle) parent.getLayoutManager().getConstraint(child)).getCopy();
			bounds.setLocation(x, y);
			parent.setConstraint(child, bounds);
		}
		
		public void componentResized(int width, int height) {
			IFigure child = ((GraphicalEditPart) getHost()).getFigure();
			IFigure parent = ((GraphicalEditPart) VisualComponentsLayoutPolicy.this.getHost()).getFigure();
			Rectangle bounds = ((Rectangle) parent.getLayoutManager().getConstraint(child)).getCopy();
			bounds.setSize(width, height);
			parent.setConstraint(child, bounds);
		}

		public void componentRefreshed() {
			signalRefresh();
		}
		
		public void componentShown() {
		}

		public void componentHidden() {
		}
		
	}

}