package com.ibm.etools.cde;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cdm.*;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.EditPartViewer;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.cde.commands.*;
import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.cde.emf.*;
import com.ibm.etools.emf.notify.Notifier;
import java.util.Iterator;
import java.util.List;
/**
 * A helper policy for handling the VisualInfo.
 * It takes care of finding it, updating it, getting
 * notifications back of changes to it.
 *
 * It is not instantiated, all of the methods are statics.
 *
 * There is a inner class, VisualInfoListener, which when created
 * will listen for annotation add/removes, visual info add/removes, and
 * any visual info changes. This can be used for listening to VisualInfo changes.
 */
public class VisualInfoPolicy {
	
	/**
	 * Return the VisualInfo for the specified EditPart.
	 */
	public static VisualInfo getVisualInfo(EditPart ep) {
		return getVisualInfo(ep.getModel(), ep.getRoot().getViewer());
	}
		
	/**
	 * Return the VisualInfo for the specified object for the
	 * specified viewer. This is usually used on ADD/CREATE because
	 * we don't yet have an editpart to find the viewer.
	 */
	public static VisualInfo getVisualInfo(Object model, EditPartViewer viewer) {
		EditDomain dom = (EditDomain) viewer.getEditDomain();
		AnnotationLinkagePolicy al = dom.getAnnotationLinkagePolicy();
		
		// Get the Annotation for the editpart.
		Annotation annot = al.getAnnotation(model)		;
		if (annot == null)
			return null;	// There is no annotation to store the VisualInfo
			
		// Get the diagram for the viewer that the object is/will be in.
		Diagram diagram = dom.getDiagram(viewer);
		VisualInfo vi = annot.getVisualInfo(diagram);
		return vi;
	}
	
	
	/**
	 * Create the appropriate command to apply the Visual Info setting.
	 * It will create the VisualInfo/Annotation if necessary.
	 *
	 * Note: This is important to be used if the annotation/visual info may not yet be available. It will make
	 * sure that they are created correctly. If the visual info already exists, then a straight ApplyKeyedValueCommand
	 * could be used, however this is simpler in that it can be applied without worrying if it exists or not.
	 */
	public static Command applyVisualInfoSetting(Object modelObject, Object newValue, EditDomain domain, Diagram diagram) {
		return new ApplyVisualInfoCommand(modelObject, newValue, domain, diagram);
	}
	
	/**
	 * Cancel a VisualInfo KeyValue.
	 * It will do more in that if that is the only keyedValue. This will remove it from the
	 * diagram, thereby saving storage.
	 */
	public static Command cancelVisualInfoSetting(VisualInfo vi, String key) {
		if (!vi.isSetKeyedValue(key))
			return null;	// Not set, so can't cancel.
			
		if (vi.getKeyedValues().size() == 1)
			return AnnotationPolicy.cancelVisualInfo(vi);	// Cancel the entire visual info
		
		// There is more than one, so only cancel the one setting.
		CancelKeyedValueCommand cmd = new CancelKeyedValueCommand();
		cmd.setTarget(vi);
		cmd.setKey(key);
		return cmd;
	}
	
	/**
	 * To completely remove a VisualInfo.
	 *
	 * Note: It is important that this command be used to completely remove a VisualInfo. If it isn't
	 * used, it won't be correctly removed from the model.
	 */
	public static Command cancelVisualInfo(VisualInfo vi) {
		return AnnotationPolicy.cancelVisualInfo(vi);
	}
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final RefObject 
		SF_PARENT_ANNOTATION,
		SF_VISUALINFOS;
	static {
		SF_PARENT_ANNOTATION = CDEUtilities.CDM_PACKAGE.getAnnotationEMF_Annotates().refOppositeEnd();
		SF_VISUALINFOS = CDEUtilities.CDM_PACKAGE.getAnnotation_VisualInfos();
	}
	
	/**
	 * A compound listener that listens for VisualInfo changes. It treats it like it is actually
	 * a property of the model object.
	 *
	 * Note: This shouldn't be used with the same IEMFListener for with AnnotationPolicy.AnnotationListener
	 * because when annotations are added/removed that listener will get duplicate notifications, one from
	 * each of these two listeners. If both are desired for the same IEMFListener, then VisualInfoListener
	 * should be used with annotationKeys flag set to true. Then only one notification will be sent.
	 *
	 * It notifies:
	 *   a) Any VisualInfo change, including KeyValues. Notifier will be VisualInfo.
	 *   b) VisualInfo add/removed for the diagram of interest. Notifier will be an Annotation, SF will be 
	 *      visualInfos.
	 *   c) Annotations add/removed for the model of interest. Notifier won't be available (because the
	 *      model could be a non-EMF model). SF will be parentAnnotationAttribute.
	 *   d) If turned on through annotationKeys ctor, any Annotation changes, including KeyValues. Notifier will be Annotation.
	 */
	public static class VisualInfoListener implements IEMFListener, GenericAnnotationLinkagePolicy.AnnotationLinkageListener {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected IEMFListener listener;
		protected Object model;
		protected String modelID;
		protected GenericAnnotationLinkagePolicy linkage;
		protected Diagram diagram;
		protected Annotation annotation;
		protected boolean annotationKeys;
		protected VisualInfo visualInfo;
		protected EMFListenerAdapter
			modelAdapter,	// When EMF model
			annotationAdapter,
			visualInfoAdapter;
		
		public VisualInfoListener(Object model, Diagram diagram, EditDomain domain, IEMFListener listener) {
			this.model = model;
			this.diagram = diagram;
			this.listener = listener;
			if (model instanceof RefObject)
				setupModelAdapter();
			else
				setupModelListener((GenericAnnotationLinkagePolicy) domain.getAnnotationLinkagePolicy());
		}
		
		public VisualInfoListener(Object model, Diagram diagram, EditDomain domain, IEMFListener listener, boolean annotationKeys) {
			this(model, diagram, domain, listener);
			this.annotationKeys = annotationKeys;
		}
		
		/**
		 * Remove all listening.
		 */
		public void removeListening() {
			if (model != null) {
				listener = null;	// We are removing so we don't care to notify more.
				if (model instanceof RefObject) {
					((Notifier) model).removeAdapter(modelAdapter);
					modelAdapter = null;
				} else {
					linkage.removeAnnotationLinkageListener(modelID, this);
					modelID = null;
					linkage = null;
				}
				removeAnnotationAdapter();
				model = null;
				diagram = null;
			}
		}
		
		private void setupModelAdapter() {
			Notifier ro = (Notifier) model;
			modelAdapter = new EMFListenerAdapter(this);
			modelAdapter.setTarget(ro);
			ro.addAdapter(modelAdapter);
			setupAnnotationAdapter((Annotation) ((RefObject) model).refValue(CDEUtilities.CDM_PACKAGE.getAnnotationEMF_Annotates().refOppositeEnd()));	// The opposite end is the feature for modelobject to annotation
		}
		
		private void setupModelListener(GenericAnnotationLinkagePolicy linkage) {
			this.linkage = linkage;
			modelID = linkage.getIDFromModel(model);
			linkage.addAnnotationLinkageListener(modelID, this);
			setupAnnotationAdapter(linkage.getAnnotation(model));
		}

		private void setupAnnotationAdapter(Annotation annotation) {
			if (annotationAdapter != null) {
				removeAnnotationAdapter();
			}
			
			this.annotation = annotation;
			if (annotation != null) {
				annotationAdapter = new EMFListenerAdapter(this);
				annotationAdapter.setTarget(annotation);
				annotation.addAdapter(annotationAdapter);
				setupVisualInfoAdapter(annotation.getVisualInfo(diagram));
			}
		}
		
		private void removeAnnotationAdapter() {
			if (annotationAdapter != null) {
				EMFListenerAdapter a = annotationAdapter;
				annotationAdapter = null;	// So we don't get a loop on remove.
				annotation.removeAdapter(a);
				annotation = null;
				
				removeVisualInfoAdapter();
			}
		}
		
		private void setupVisualInfoAdapter(VisualInfo visualInfo) {
			if (visualInfoAdapter != null) {
				removeVisualInfoAdapter();
			}
			
			this.visualInfo = visualInfo;
			if (visualInfo != null) {
				visualInfoAdapter = new EMFListenerAdapter(this, true);
				visualInfoAdapter.setTarget(visualInfo);
				visualInfo.addAdapter(visualInfoAdapter);
			}
		}
		
		private void removeVisualInfoAdapter() {
			if (visualInfoAdapter != null) {
				EMFListenerAdapter a = visualInfoAdapter;
				visualInfoAdapter = null;	// So we don't get a loop on remove.
				visualInfo.removeAdapter(a);
				visualInfo = null;
			}
		}
			
		public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {
			if (listener == null)
				return;
			if (notifier == model) {
				if (eventType == Notifier.REMOVING_ADAPTER)
					removeListening();	// My adapter is being removed from the model, so remove all listening.
				else if (sf == SF_PARENT_ANNOTATION) {
					switch (eventType) {
						case Notifier.SET:
							setupAnnotationAdapter((Annotation) newValue);
						case Notifier.TOUCH:							
							listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
							break;
						case Notifier.UNSET:
							setupAnnotationAdapter(null);
							listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
							break;
					}
				}
			} else if (notifier == annotation) {
				if (eventType == Notifier.REMOVING_ADAPTER)
					removeAnnotationAdapter();	// My adapter is being removed from the annotation, so remove listening
				else if (sf == SF_VISUALINFOS) {
					switch (eventType) {
						case Notifier.ADD:
						case Notifier.TOUCH:
							VisualInfo vi = (VisualInfo) newValue;
							if (vi.getDiagram() == diagram) {
								if (eventType == Notifier.ADD)
									setupVisualInfoAdapter(vi);
								listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
							}
							break;
						case Notifier.ADD_MANY:
							Iterator itr = ((List) newValue).iterator();
							while (itr.hasNext()) {
								vi = (VisualInfo) itr.next();
								if (vi.getDiagram() == diagram) {
									setupVisualInfoAdapter(vi);
									listener.notifyChanged(notifier, Notifier.ADD, sf, null, vi, Notifier.NO_INDEX);
									break;
								}
							}
							break;
						case Notifier.REMOVE:
							vi = (VisualInfo) oldValue;
							if (vi.getDiagram() == diagram) {
								setupVisualInfoAdapter(null);
								listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
							}
							break;
						case Notifier.REMOVE_MANY:
							itr = ((List) oldValue).iterator();
							while (itr.hasNext()) {
								vi = (VisualInfo) itr.next();
								if (vi.getDiagram() == diagram) {
									setupVisualInfoAdapter(null);
									listener.notifyChanged(notifier, Notifier.REMOVE, sf, vi, null, Notifier.NO_INDEX);
									break;
								}
							}
							break;
					}
				} else if (annotationKeys) {
					// Also interested in any annotation change
					listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
				}
			} else if (notifier == visualInfo) {
				if (eventType == Notifier.REMOVING_ADAPTER)
					removeVisualInfoAdapter();
				listener.notifyChanged(notifier, eventType, sf, oldValue, newValue, pos);
			}				
		}
		
		public void annotationLinkageChanged(GenericAnnotationLinkagePolicy.AnnotationLinkageEvent event) {
			if (listener == null)
				return;
			if (event.getType() == event.REMOVING_LISTENER)
				removeListening();	// My adapter is being removed from the model, so remove all listening.
			else {
				switch (event.getType()) {
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.SET:
						setupAnnotationAdapter(((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getNewAnnotation());
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.TOUCH:							
						listener.notifyChanged(null, event.getType() == GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.SET ? Notifier.SET : Notifier.TOUCH, SF_PARENT_ANNOTATION, ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getOldAnnotation(), ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getNewAnnotation(), Notifier.NO_INDEX);
						break;
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.UNSET:
						setupAnnotationAdapter(null);
						listener.notifyChanged(null, Notifier.UNSET, SF_PARENT_ANNOTATION, ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getOldAnnotation(), ((GenericAnnotationLinkagePolicy.AnnotationLinkageChangeEvent) event).getNewAnnotation(), Notifier.NO_INDEX);
						break;
					case GenericAnnotationLinkagePolicy.AnnotationLinkageEvent.SET_ID:
						modelID = (String) event.getSource();	// Get the new id and save it.
						break;
				}
			}
		}
	}	
		
}