package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.emf.ref.*;
import java.util.*;
import com.ibm.etools.emf.notify.Notifier;

/**
 * Command to apply MOF attributes at indexed locations.
 *
 * All of the settings will be between the given index and the
 * next item. So if current setting was:
 *   0 1 2 3 4
 *
 * And the settings were, with an insertion index of 2:
 *   a b c
 *
 * The result would be:
 *   0 1 a b c 2 3 4
 *
 * The undo will remove the entries.
 *
 * The default index is -1, which is add all at the end.
 *
 * You can also set a insertionValue, and it will insert before that value,
 * if it is found, else it will add at the end. Insertion index and Insertion value
 * are mutually exclusive.
 *
 */
public class ApplyAttributeSettingCommand extends AbstractApplyAttributeCommand {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Object oldValue;
	protected boolean oldSet;
	protected int fInsertionIndex = -1;
	protected Object fInsertBeforeValue;
	
public ApplyAttributeSettingCommand(String name) {
	super(name);
}	

public ApplyAttributeSettingCommand() {
	super();
}		


public void execute() {		
	RefObject target = getTarget();	
	if (!feature.refIsMany()) {
		//Remember previous values for single-valued attributes.
		if (oldSet = target.refIsSet(feature))
			oldValue = target.refValue(feature);
		try {
			target.refSetValue(feature, getAttributeSettingValues().get(0));
		} catch (RuntimeException e) {
			// Try to put the old one back.
			undo();
			throw e;
		}
	} else {
		int insertAt = fInsertionIndex;
		List featureList = (List) target.refValue(feature);
		if (fInsertBeforeValue != null) {
			// We want to insert before a value. Find the position
			// of that value.
			insertAt = featureList.indexOf(fInsertBeforeValue);
		}

		try {
			if (insertAt == -1) {
				featureList.addAll(getAttributeSettingValues());
			} else {
				featureList.addAll(insertAt, getAttributeSettingValues());
			}
		} catch (RuntimeException e) {
			// Try to clean up to a pre-execute state. Some may of already been added, need to remove them.
			featureList.removeAll(getAttributeSettingValues());
			throw e;
		}
	}
}

public void undo() {	
	RefObject target = getTarget();
	
	if (feature.refIsMany()) {
		((List) target.refValue(feature)).removeAll(getAttributeSettingValues());
	} else if (!oldSet) {
		target.refUnsetValue(feature);			
	} else {
		target.refSetValue(feature, oldValue);
	}
	
	oldValue = null;	// Don't hold onto it.	
}

/**
 * Set the insertion index. -1 means at to the end.
 */
public void setInsertionIndex(int newIndex) {
	fInsertionIndex = newIndex;
	fInsertBeforeValue = null;	// Reset it to indicate index type requested.
}
/**
 * Set the insert before value. This is mutually exclusive with the
 * setInsertionIndex() method.
 */
public void setInsertBeforeValue(Object insertBefore) {
	fInsertionIndex = -1;	// Reset it to indicate insertion value requested.
	fInsertBeforeValue = insertBefore;
}

}
