package com.ibm.etools.cde.commands;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import java.util.*;

import com.ibm.etools.emf.ref.RefObject;

public class CancelAttributeSettingCommand extends AbstractAttributeCommand {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected Object oldValue;	// Old value when not many feature.
	protected boolean oldSet;	// Was Old actually set?
	protected int[] oldIndexes;	// Indexes of removed values so that they may be put back in correct locations.
		
	public CancelAttributeSettingCommand(String name) {
		super(name);
	}
	public CancelAttributeSettingCommand() {
		super();
	}	
	
	public boolean canExecute() {
		// Can execute if super.canExecute() and it is either a single-valued feature or
		// it is multi-valued and there are settings to cancel out.
		return super.canExecute() && (!feature.refIsMany() || !getAttributeSettingValues().isEmpty());
	}
	
	public void execute() {
		RefObject target = getTarget();
		if (!feature.refIsMany()) {
			if (oldSet = target.refIsSet(feature))
				oldValue = target.refValue(feature);
			try {
				target.refUnsetValue(feature);
			} catch (RuntimeException e) {
				// Try to clean up and put old value back.
				undo();
				throw e;
			}
		} else {
			List values = getAttributeSettingValues();
			ArrayList orderedValues = new ArrayList(values.size());	// The values will be reordered into their order within the settings list.
			oldIndexes = new int[values.size()];
			int oldI = -1;
			// Walk through the setting list and remove values as they are found. Save their index so that they 
			// can be put back in the same place on undo.
			ListIterator itr = ((List) target.refValue(feature)).listIterator();
			Object v = null;
			try {
				while (!values.isEmpty() && itr.hasNext()) {
					v = itr.next();
					if (values.remove(v)) {
						// It was found in the list to be canceled.												
						oldIndexes[++oldI] = itr.previousIndex();	// Loc to restore it to
						// Actual value to add back is the one from the setting, not the one from values
						// because the one from values is semantically equal, while the one from the setting
						// is the actual identity to restore.
						orderedValues.add(v);
						itr.remove();								
					}			
				}
			
				setAttributeSettingValue(orderedValues);	// Save the ordered values for undo.
			} catch (RuntimeException e) {
				// An itr.remove() failed. Try to clean up.
				setAttributeSettingValue(orderedValues);
				undo();	// Add back all those so far removed.
				values.addAll(orderedValues);	// Add back those that were removed.
				setAttributeSettingValue(values);
				throw e;
			}
		}
	}
		
	public void undo() {
		RefObject target = getTarget();
		if (!feature.refIsMany()) {
				if (oldSet) 
					try {		
						target.refSetValue(feature, oldValue);
					} catch (RuntimeException e) {
						// Try to restore to executed state.
						target.refUnsetValue(feature);
						throw e;
					}					
				else
					target.refUnsetValue(feature);
		} else {
			if (getAttributeSettingValues().isEmpty())
				return;	// Nothing to undo. This could happen if the cancel execute didn't find any to remove.			
			// Walk backwards through the list adding the oldValues back in at their appropriate place.
			// Going backwards so that the index values match them in the same order as they came out.
			int oldI = getAttributeSettingValues().size();
			ListIterator oldItr = getAttributeSettingValues().listIterator(oldI);
			int nextI = oldIndexes[--oldI];			
			ListIterator itr = ((List) target.refValue(feature)).listIterator(nextI);	// Start with last one to restore from oldIndexes.
			while (true) {
				if (nextI == itr.nextIndex()) {
					// We found a spot to add
					itr.add(oldItr.previous());
					if (--oldI < 0)
						break;	// We have no more to restore
					nextI = oldIndexes[oldI];
				}
				itr.previous();
			} 
		}
		
		oldIndexes = null;	// Don't hold onto it.
		oldValue = null;	// Don't hold onto it.
	}		
}
