package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.gef.requests.*;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.gef.Request;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cdm.*;
import java.util.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.cde.*;
/**
 * Container Policy for EMF (MOF) containment relationship.
 * The structural feature is to be passed in from subclasses.
 * This allows the subclass to determine the feature to use, in cases
 * where it may change depending upon conditions.
 */
public abstract class AbstractEMFContainerPolicy extends ContainerPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public AbstractEMFContainerPolicy(EditDomain domain) {
		super(domain);
	}
	
	/**
	 * Is this a valid child of this container.
	 */
	protected abstract boolean isValidChild(Object child);
	
	/**
	 * Is parent acceptable. Sometimes even though the parent may accept
	 * a child, the child may not accept the parent.
	 */
	protected boolean isParentAcceptable(Object child) {
		IModelAdapterFactory fact = CDEUtilities.getModelAdapterFactory(domain);
		if (fact != null) {
			IContainmentHandler handler = (IContainmentHandler) fact.getAdapter(child, IContainmentHandler.class);
			if (handler != null)
				return handler.isParentValid(container);
		}
		return true;
	}
	
	/**
	 * Create a  child.
	 * NOTE: Should not be getting create request on a non-composite containment.
	 * That doesn't make sense, it should be an add instead. So we don't test for that.
	 */
	protected Command getCreateCommand(Object child, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		if (!isValidChild(child) || !isParentAcceptable(child))
			return UnexecutableCommand.INSTANCE;
		return primCreateCommand(child, positionBeforeChild, containmentSF);
	}
	
	/**
	 * Internal create a child where we have the child has already been tested for validity.
	 */
	protected Command primCreateCommand(Object child, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		if (!containmentSF.refIsMany() && ((RefObject) container).refIsSet(containmentSF))
			return UnexecutableCommand.INSTANCE;	// This is a single valued feature, and it is already set.
			
		CommandBuilder cBld = new CommandBuilder("");		 //$NON-NLS-1$
		cBld.applyAttributeSetting((RefObject) container, containmentSF, child, positionBeforeChild);
		List annotations = AnnotationPolicy.getAllAnnotations(new ArrayList(), child, domain.getAnnotationLinkagePolicy());		
		return AnnotationPolicy.getCreateRequestCommand(annotations, cBld.getCommand(), domain);
	}
	
	/**
	 * Add a  child.
	 */
	protected Command getAddCommand(List children, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		Iterator itr = children.iterator();
		while (itr.hasNext()) {
			Object child = itr.next();
			if (!isValidChild(child) || !isParentAcceptable(child))
				return UnexecutableCommand.INSTANCE;
		}
		
		return primAddCommand(children, positionBeforeChild, containmentSF);
	}
	
	/**
	 * Internal add children where we have the children already tested for validity..
	 */
	protected Command primAddCommand(List children, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		if (!containmentSF.refIsMany() && (((RefObject) container).refIsSet(containmentSF) || children.size() > 1))
			return UnexecutableCommand.INSTANCE;	// This is a single valued feature, and it is already set or there is more than one child being added.

		CommandBuilder cBld = new CommandBuilder(""); //$NON-NLS-1$
		cBld.applyAttributeSettings((RefObject) container, containmentSF, children, positionBeforeChild);
		return cBld.getCommand();
	}
	
	/**
	 * Delete a  child.
	 */
	protected Command getDeleteDependentCommand(Object child, RefStructuralFeature containmentSF) {
		CommandBuilder cBld = new CommandBuilder(""); //$NON-NLS-1$
		cBld.cancelAttributeSetting((RefObject) container, containmentSF, child);
		if (containmentSF.refIsComposite()) {
			// A composite feature means we have annotations to worry about.
			List annotations = AnnotationPolicy.getAllAnnotations(new ArrayList(), child, domain.getAnnotationLinkagePolicy());
			return AnnotationPolicy.getDeleteDependentCommand(annotations, cBld.getCommand(), domain.getDiagramData());
		} 
		return cBld.getCommand();	// No annotations if not composite because the child is owned by someone else and is not going away.
	}
	
	/**
	 * Orphan  children.
	 */
	protected Command getOrphanChildrenCommand(List children, RefStructuralFeature containmentSF) {
		CommandBuilder cBld = new CommandBuilder(""); //$NON-NLS-1$
		cBld.cancelAttributeSettings((RefObject) container, containmentSF, children);
		return cBld.getCommand();
	}
	
	/**
	 * Move the  children.
	 */
	protected Command getMoveChildrenCommand(List children, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		CommandBuilder cBld = new CommandBuilder("");		 //$NON-NLS-1$
		if (children.contains(positionBeforeChild))
			return UnexecutableCommand.INSTANCE;
		// We clone the list because cancelAttributeSettings fools around with the list itself.
		cBld.cancelAttributeSettings((RefObject) container, containmentSF, new ArrayList(children));
		cBld.applyAttributeSettings((RefObject) container, containmentSF, children, positionBeforeChild);
		return cBld.getCommand();
	}			

}


