package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import com.ibm.etools.emf.ref.RefRegister;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EClassifier;
import com.ibm.etools.emf.ecore.EDecorator;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.emf.resource.impl.ContextImpl;
import com.ibm.etools.emf.resource.Resource;
import com.ibm.etools.emf.resource.ResourceFactory;
import com.ibm.etools.emf.ref.Extent;
import com.ibm.etools.emf.ref.KeyFactoryRegister;
import com.ibm.etools.emf.ref.Key;
import com.ibm.etools.emf.resource.HrefFactoryRegister;
import java.util.Iterator;
import java.util.HashMap;
import com.ibm.etools.cde.decorators.*;
import com.ibm.etools.cde.utility.*;
import org.eclipse.ui.views.properties.IPropertySheetEntry;
import com.ibm.etools.gef.EditPart;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;
import org.eclipse.jface.viewers.ILabelProvider;
import com.ibm.etools.cde.CDEPlugin;
import java.lang.reflect.Method;
import java.text.MessageFormat;
import org.eclipse.core.runtime.*;

import com.ibm.etools.logging.util.Level;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.emf.resource.ResourceFactoryRegister;
/**
 * This class is used to get access decorators for a specific
 * EClassifier. It will look up the parent chain to find the decorators.
 * This allows for the inheritance of settings. It uses the ClassDecoratorFeatureAccess
 * philosophy to find the decorator. These methods are utility methods.
 * If no decorators that have a setting are found, then the default decorator
 * will be used or the type requested will be found.
 *
 * If a decorator is found, and if the setting in question is not set
 * on that decorator, then an inherited decorator setting will be used
 * instead.
 *
 * It will also allow access to certain common specific settings, such as DisplayName, etc.
 * In those cases it provides some special overrides in that certain settings are not to
 * be overridden.
 * The following settings are not inherited:
 *   a) DisplayName: If there is not a decorator for the class, then there
 *                   is no displayname. That is because displayname of
 *                   subclasses shouldn't be the same as an inherited class.
 *   b) ShortDescription: Same reason as display name.
 *   c) Expert: Same reason as above.
 *   d) Hidden: Same reason as above.
 *
 * There will be an instance of this class stored in the EditDomain under the CLASS_DESCRIPTOR_DECORATOR_POLICY_KEY
 * specified here. There is a static helper method in this class to get the policy from any editpart,
 * or from the edit domain.
 *
 * The reason it is an instance and stored in the EditDomain is because each editor may want to have a different
 * default ClassDescriptorDecorator for classes that don't have a decorator anywhere in their heirarchy.
 */
public class ClassDescriptorDecoratorPolicy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String CLASS_DESCRIPTOR_DECORATOR_POLICY_KEY = "com.ibm.etools.cde.classdescriptordecoratorpolicykey"; //$NON-NLS-1$
	
	/**
	 * Helper method to get the policy from an editpart.
	 */
	public static ClassDescriptorDecoratorPolicy getPolicy(EditPart editpart) {
		return getPolicy(EditDomain.getEditDomain(editpart));
	}
	
	/**
	 * Helper method to get the policy from the editdomain.
	 */
	public static ClassDescriptorDecoratorPolicy getPolicy(EditDomain domain) {
		return (ClassDescriptorDecoratorPolicy) domain.getData(CLASS_DESCRIPTOR_DECORATOR_POLICY_KEY);
	}
	
	/**
	 * Helper method to set the ClassDescriptorDecoratorPolicy into the EditDomain.
	 */
	public static void setClassDescriptorDecorator(EditDomain domain, ClassDescriptorDecoratorPolicy policy) {
		policy.editDomain = domain;
		domain.setData(CLASS_DESCRIPTOR_DECORATOR_POLICY_KEY, policy);
	}
	
	public ClassDescriptorDecoratorPolicy() {	
		// Create a default ClassDescriptorDecorator. This is the one used if none is found.
		
		DecoratorsFactory dFact = ((DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI)).getDecoratorsFactory();
		UtilityFactory uFact = ((UtilityPackage) RefRegister.getPackage(UtilityPackage.packageURI)).getUtilityFactory();
		
		// Initialize a default decorator for the metaclass that
		// will be used if a decorator is not specified for a specific class
		ClassDescriptorDecorator classDecor = dFact.createClassDescriptorDecorator();
		classDecor.setGraphViewClassname("com.ibm.etools.cde/com.ibm.etools.cde.emf.DefaultGraphicalEditPart"); //$NON-NLS-1$
		classDecor.setTreeViewClassname("com.ibm.etools.cde/com.ibm.etools.cde.emf.DefaultTreeEditPart"); //$NON-NLS-1$
		classDecor.setGraphic(uFact.createGIFFileGraphic("platform:/plugin/com.ibm.etools.cde/images/somepart.gif")); //$NON-NLS-1$
		classDecor.setLabelProviderClassname("com.ibm.etools.cde/com.ibm.etools.cde.emf.DefaultLabelProvider"); //$NON-NLS-1$
		registerDefaultDecorator(ClassDescriptorDecorator.class, classDecor);
	}
	
	public ClassDescriptorDecoratorPolicy(ClassDescriptorDecorator defaultDecorator) {
		registerDefaultDecorator(ClassDescriptorDecorator.class, defaultDecorator);
	}
	
	private static final DecoratorsPackage sDecoratorPackage = (DecoratorsPackage) RefRegister.getPackage(DecoratorsPackage.packageURI);
	

	private EditDomain editDomain;
	/*
	 * The default decorators will be stored in this resource. The id will be 
	 * the class.toString(). This is stored here in a resource so that they
	 * can be loaded and modified in the future through documents.
	 */
	public static final String DEFAULT_DECOR_RESOURCE_NAME = "defaultDecoratorResource.xmi"; //$NON-NLS-1$
	private Extent defaultDecorExtent;
	private Key idKey;
	private ResourceSet rset;
	{
		ContextImpl context = new ContextImpl();
		rset = ResourceFactoryRegister.getResourceSetFactory().makeResourceSet();
		context.setResourceSet(rset);
		Resource defaultDecorResource = ResourceFactoryRegister.getFactory(DEFAULT_DECOR_RESOURCE_NAME).makeResource(DEFAULT_DECOR_RESOURCE_NAME);
		defaultDecorExtent = defaultDecorResource.getExtent();
		
		String keyId = HrefFactoryRegister.getFactory().getKeyInterface();
		idKey = KeyFactoryRegister.getFactory().makeKey("com.ibm.etools.emf.ref.IDKey"); //$NON-NLS-1$

		if (idKey != null) {
			defaultDecorExtent.addKey(idKey);
			idKey.setExtent(defaultDecorExtent);
		}
		rset.add(defaultDecorResource);
	}
		
	public void registerDefaultDecorator(Class decClass, EDecorator defaultDecor) {
		defaultDecor.refSetID(decClass.toString());
		idKey.add(defaultDecor);
	}
	
	public EDecorator getDefaultDecorator(Class decClass) {
		return (EDecorator) idKey.get(decClass.toString());
	}
	
	/**
	 * Find the decorator for this class only. Do not handle inheritance.
	 */
	protected ClassDescriptorDecorator findDecorator(EClassifier eClass) {
		Iterator itr = eClass.getEDecorators().iterator();
		while (itr.hasNext()) {
			Object next = itr.next();
			if (next instanceof ClassDescriptorDecorator)
				return (ClassDescriptorDecorator) next;
		}
		return null;
	}
	
	/**
	 * Find the decorator of the type requested with the feature set for this class,
	 * return default of that type if none found.
	 */
	public EDecorator findDecorator(EClassifier eClass, Class decoratorType, RefStructuralFeature feature) {
		EDecorator decr = ClassDecoratorFeatureAccess.getDecoratorWithFeature(eClass, decoratorType, feature);
		return (decr != null) ? decr : (EDecorator) getDefaultDecorator(decoratorType);
	}

	/**
	 * Find the decorator of the type requested with the KeyedValue key set for this class,
	 * return default of that type if none found.
	 */
	public EDecorator findDecorator(EClassifier eClass, Class decoratorType, String key) {
		EDecorator decr = ClassDecoratorFeatureAccess.getDecoratorWithKeyedFeature(eClass, decoratorType, key);
		return (decr != null) ? decr : (EDecorator) getDefaultDecorator(decoratorType);
	}


	/**
	 * Return the customizer class. This is an inheritable setting.
	 */
	public String getCustomizerClassname(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_CustomizerClassname());
		return decr.getCustomizerClassname();
	}
	
	/**
	 * Return the default palette. This is an inheritable setting.
	 */
	public String getDefaultPalette(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_DefaultPalette());
		return decr.isSetDefaultPalette() ?
			decr.getDefaultPalette() : null;
	}
	
	/**
	 * Return the display name. This is not an inheritable setting, so default won't be used.
	 */
	public String getDisplayName(EClassifier eClass) {
		ClassDescriptorDecorator decor = findDecorator(eClass);
		return (decor != null && decor.isSetDisplayNameString()) ? decor.getDisplayNameString().getStringValue() : null;
	}
	
	/**
	 * Return the short description. This is not an inheritable setting, so default won't be used.
	 */
	public String getDescription(EClassifier eClass) {
		ClassDescriptorDecorator decor = findDecorator(eClass);
		return (decor != null && decor.isSetDescriptionString()) ? decor.getDescriptionString().getStringValue() : null;
	}
	
	/**
	 * Return the graph view class. This is an inheritable setting.
	 */
	public String getGraphViewClassname(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_GraphViewClassname());
		return decr.getGraphViewClassname();
	}
	
	/**
	 * Return the model adapter class. This is an inheritable setting.
	 */
	public String getModelAdapterClassname(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_ModelAdapterClassname());
		return decr.getModelAdapterClassname();
	}
	
	/**
	 * Return the icon. This is an inheritable setting.
	 */
	public IGraphic getIcon(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_Graphic());
		return decr.isSetGraphic() ? decr.getGraphic() : null;
	}
	
	/**
	 * Return the tree view class. This is an inheritable setting.
	 */
	public String getTreeViewClassname(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_TreeViewClassname());
		return decr.getTreeViewClassname();
	}
	/**
	 * Return the expert setting. This is not an inheritable setting, so default won't be used.
	 */
	public boolean isExpert(EClassifier eClass) {
		ClassDescriptorDecorator decor = findDecorator(eClass);
		return (decor != null) ? decor.isFiltered(IPropertySheetEntry.FILTER_ID_EXPERT) : false;
	}		
	/**
	 * Return the hidden setting. This is not an inheritable setting, so default won't be used.
	 */
	public boolean isHidden(EClassifier eClass) {
		ClassDescriptorDecorator decor = findDecorator(eClass);
		return (decor != null) ? decor.isHidden() : false;
	}
	
	/**
	 * Return the labelProvider class. This is an inheritable setting.
	 */
	public String getLabelProviderClassname(EClassifier eClass) {
		ClassDescriptorDecorator decr = (ClassDescriptorDecorator) findDecorator(eClass, ClassDescriptorDecorator.class, sDecoratorPackage.getClassDescriptorDecorator_LabelProviderClassname());
		return decr.getLabelProviderClassname();
	}
	
	
	/**
	 * Return the labelProvider itself. 
	 */
	public ILabelProvider getLabelProvider(EClassifier eClass) {
		String classNameAndData = getLabelProviderClassname(eClass);
		if (classNameAndData == null)
			return null;
		
		try {	
			Class clazz = CDEPlugin.getClassFromString(classNameAndData);
					
			try {
				ILabelProvider provider = (ILabelProvider) clazz.newInstance();
				
				CDEPlugin.setInitializationData(provider, classNameAndData, null);
				if (provider instanceof INeedData)
					((INeedData) provider).setData(editDomain);
				return provider;
			} catch (Exception exc) {
				String msg = MessageFormat.format(CDEMessages.getString(CDEMessages.OBJECT_NOINSTANTIATE), new Object[] {clazz});		
				CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, msg, exc));
			}
		} catch (ClassNotFoundException e) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", e)); //$NON-NLS-1$
		}
		
		return null;
	}
}
			