package com.ibm.etools.cde.emf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.ArrayList;
import java.util.List;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.emf.ecore.utilities.copy.*;
import com.ibm.etools.emf.ecore.utilities.copy.CopyGroup;
import com.ibm.etools.emf.ecore.utilities.copy.EtoolsCopySession;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.resource.*;
import com.ibm.etools.gef.requests.CreateRequest;
import com.ibm.etools.logging.util.Level;

/**
 * This is a prototype factory that takes the URI to the prototype
 * and using the EditDomain to find the resourceset, will load it
 * an instance. It will not keep the resource set loaded, it will
 * load it each time and then after getting prototype out of the
 * set, it will unload it. That way we don't keep infrequent objects
 * loaded when not needed. The URI should be in a disposable XMI file,
 * i.e. not in one that the editor may need later. It should be entirely
 * self-contained and should have only one object in the extent, the prototype.
 * The prototype may contain other objects, these will be copied when the
 * prototype is copied.
 * @version 	1.0
 * @author
 */
public class EMFPrototypeCreationFactory implements CreateRequest.Factory, IDomainedFactory {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected URI prototypeURI;
	protected List prototypes;	// All of the prototypes from the extent
	protected RefObject prototype;	// The prototype to return.
	
	/**
	 * Constructor for EMFPrototypeCreationFactory.
	 * @param aPrototype
	 */
	public EMFPrototypeCreationFactory(String prototypeURIString) {
		setPrototypeURI(URIFactoryRegister.getFactory().makeURI(prototypeURIString));
	}
	
	public void setPrototypeURI(URI prototypeURI) {
		this.prototypeURI = prototypeURI;
	}
	
	

	/**
	 * @see IDomainedFactory#setEditDomain(EditDomain)
	 */
	public void setEditDomain(EditDomain domain) {
		// Now that we have a domain, find and set the actual prototype
		// We are being activated, so this is a good time to do it.
		
		prototype = null;
		prototypes = null;
		if (prototypeURI != null) {
			ResourceSet rset = EMFEditDomainHelper.getResourceSet(domain);
			try {
				prototype = rset.getObjectAndLoad(prototypeURI);
				if (prototype != null) {
					Extent extent = prototype.refExtent();
					if (extent == null) {
						prototype = null;
						System.out.println("Prototype from URI is not contained directly by the extent. URI="+prototypeURI); //$NON-NLS-1$
					} else {
						prototypes = new ArrayList(extent);	// Make of copy of the list of all of the prototypes.
						extent.clear();	// Now clear the extent so that unloading it won't unload the prototypes.
						extent.getResourceSet().remove(extent.getResource());	//Dispose of the resource.
					}
				}
			} catch (WrappedException e) {
				CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, e);
				// The resource had still been added and we need to remove it.
				Resource res = rset.getResource(prototypeURI);
				if (res != null)
					rset.remove(res);
			}
		}
	}

	/**
	 * @see Factory#getNewObject()
	 */
	public Object getNewObject() {
		if (prototype != null) {
			CopyGroup cg = new CopyGroup();
			cg.setPreserveIds(false);	// Don't want ids in the copies. That way they won't conflict with existing ones.
			for (int i=0; i<prototypes.size(); i++)
				cg.add((RefObject) prototypes.get(i));
				
			EtoolsCopyUtility cu = new EtoolsCopyUtility();
			cu.copy(cg);
			return cu.getCopy(prototype);	// Now return the copied prototype, anything it references from the cg will be dragged along (should only be annotations).
		} else
			return null;
	}

	/**
	 * @see Factory#getObjectType()
	 */
	public Object getObjectType() {
		return prototype != null ? prototype.refMetaObject() : null;		
	}

}
