package com.ibm.etools.cde.properties;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */



import java.util.*;
import org.eclipse.ui.views.properties.*;
import com.ibm.etools.cde.decorators.*;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.resource.*;
import org.eclipse.core.runtime.*;
import com.ibm.etools.cde.CDEPlugin;
import java.lang.reflect.Constructor;
import java.text.MessageFormat;

import com.ibm.etools.logging.util.Level;
import com.ibm.etools.propertysheet.INeedData;
import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.AnnotationPolicy;

public class PropertySourceAdapter extends AdapterImpl implements IPropertySource, INeedData {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EditDomain domain;
		
	public PropertySourceAdapter() {
	}
	
	public void setData(Object data) {
		domain = (EditDomain) data;
	}
	
	protected IPropertyDescriptor createPropertyDescriptorAdapter(RefStructuralFeature sFeature) {
		// Couldn't find the correct adapter factory from the sFeature, so use the one from the target instead.
		Resource r = ((RefObject) target).refResource();
		if (r != null) {
			ResourceSet rs = r.getResourceSet();
			if (rs != null) {
				Context c = rs.getContext();
				if (c != null) {
		  			return (IPropertyDescriptor) c.getAdapterFactory(AbstractPropertyDescriptorAdapter.IPROPERTYDESCRIPTOR_TYPE).adaptNew(sFeature, AbstractPropertyDescriptorAdapter.IPROPERTYDESCRIPTOR_TYPE);	
				}
			}
		}
		
		// No factory found, use default adapter.
		DecoratedPropertyDescriptorAdapter adapter = new DecoratedPropertyDescriptorAdapter();
		adapter.setTarget(sFeature);
		sFeature.addAdapter(adapter);
		return adapter;
	}
	
	public Object getEditableValue(){
		return target;
	}
	
	/**
	 * Helper to avoid casting all over the place.  Maybe should be cached later
	 */
	protected RefObject getRefTarget(){
		return (RefObject)target;
	}
	
	protected EDecorator getDecorator(EModelElement model, Class decoratorType) {
		Iterator decorItr = model.getEDecorators().iterator();
		while(decorItr.hasNext()) {
			EDecorator o = (EDecorator) decorItr.next();
			if (decoratorType.isInstance(o))
				return o;
		}
		return null;
	}
	
	/**
	 * See if this feature should be included. The default is
	 * to get the property descriptor decorator and see if it
	 * is not marked as hidden. Hidden features should not be included.
	 * Also if not hidden, then if it is explicitly NOT a designTimeProperty,
	 * then it should also not be included.
	 *
	 * Setting isDesignTimeProperty to explicit false means that the property
	 * is a runtime only property and is not settable at designtime. Setting it
	 * explicitly true means that it is only a designtime property and can't
	 * be set in the runtime (these are usually extra properties that result
	 * in some special code being generated and is not really a property of
	 * the object). If it is not explicitly set, then that means do the default,
	 * it is both a runtime and designtime property.
	 */
	protected boolean includeFeature(RefStructuralFeature sfeature) {
		PropertyDescriptorDecorator decorator = (PropertyDescriptorDecorator) getDecorator((EModelElement) sfeature, PropertyDescriptorDecorator.class);
		if (decorator != null)
			return !decorator.isHidden() &&
				(!decorator.isSetDesigntimeProperty() || decorator.isDesigntimeProperty());			
		return true;
	}
		
	public IPropertyDescriptor[] getPropertyDescriptors() {
		ArrayList descList = new ArrayList();
		RefObject mofTarget = (RefObject) target;
		RefObject mofClass = mofTarget.refMetaObject();
	
		ArrayList attrs = new ArrayList();
		attrs.addAll(mofClass.refAttributes());
		attrs.addAll(mofClass.refReferences());
		
		Iterator i = attrs.iterator();
		while (i.hasNext()) {
			RefStructuralFeature sfeature = (RefStructuralFeature) i.next();
			if (!sfeature.refIsMany() && includeFeature(sfeature)) {
				com.ibm.etools.cde.CDEHack.fixMe();	// Need a method of handling isMany too.
				// Not a many feature, and feature was marked as included.
				descList.add(getPropertyDescriptor(sfeature));
			}
		}
		
		if (domain != null)
			descList.addAll(AnnotationPolicy.getAnnotationPropertyDescriptors(getTarget(), domain));
			
		return (IPropertyDescriptor[]) descList.toArray(new IPropertyDescriptor[descList.size()]);
	}
		
	
	/**
	 * Get the property descriptor.
	 * This can be overridden if necessary to retrieve
	 * from a different location.
	 * 
	 * It first checks to see if there is property descriptor information that says
	 * there is specific descriptor that needs to know the source (i.e. not an adapter, which is for the
	 * feature no matter what the source is). If so, it created that. If not, it tries to adapt it.
	 */
	protected IPropertyDescriptor getPropertyDescriptor(RefStructuralFeature sfeature) {
		IPropertyDescriptor desc = null;
		desc = createPropertyDescriptorFromInfo(sfeature);	
		if (desc == null)			
			desc = (IPropertyDescriptor) sfeature.getAdapter(AbstractPropertyDescriptorAdapter.IPROPERTYDESCRIPTOR_TYPE);
		if (desc == null) {
			desc = createPropertyDescriptorAdapter(sfeature);
		}
		return desc;
	}

	/**
	 * Create a property descriptor of the class in the property descriptor info, if the info exists
	 * and says not adapter. It will try two possible constructors. If neither exists then it will
	 * return null. Target as RefObject is always sent on the constructor because if the source wasn't
	 * necessary to the descriptor, then a straight adapter would be used.
	 *
	 * isAdapter false means the descriptor is not an adapter, so it needs to be created each time
	 * it is requested. This is used in the case that the descriptor needs to know the source of the property for
	 * it to do its job.
	 */
	protected IPropertyDescriptor createPropertyDescriptorFromInfo(RefStructuralFeature sfeature) {
		PropertyDescriptorInformation descInfo = (PropertyDescriptorInformation) getDecorator((EModelElement) sfeature, PropertyDescriptorInformation.class);
		if (descInfo != null && !descInfo.isAdapter() && descInfo.isSetPropertyDescriptorClassname()) {
			String cname = descInfo.getPropertyDescriptorClassname();
			if (cname != null)
				try {
					Class c = CDEPlugin.getClassFromString(cname);
					if (c != null) {
						try {
							Constructor ctor = c.getConstructor(new Class[] {RefObject.class, RefStructuralFeature.class});
							IPropertyDescriptor pd = (IPropertyDescriptor) ctor.newInstance(new Object[] {(RefObject) target, sfeature});
							CDEPlugin.setInitializationData(pd, cname, null);
							return pd;
						} catch (NoSuchMethodException e) {
						}
						try {
							Constructor ctor = c.getConstructor(new Class[] {RefObject.class});
							IPropertyDescriptor pd = (IPropertyDescriptor) ctor.newInstance(new Object[] {(RefObject) target});
							CDEPlugin.setInitializationData(pd, cname, null);
							return pd;							
						} catch (NoSuchMethodException e) {
						}
					}
				} catch (Exception e) {
					String msg = MessageFormat.format(CDEMessages.getString(CDEMessages.OBJECT_NOINSTANTIATE), new Object[] {cname});
					CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, msg, e));
				}		
		}
		return null;
	}
	
	public Object getPropertyValue(Object feature) {
		try {
			Object value = ((RefObject) target).refValue((RefStructuralFeature)feature);
			if (value instanceof IPropertySource)
				return value;
			if (value instanceof RefObject) {
				Object result = ((RefObject)value).getAdapter(IPropertySource.class);
				return result != null ? result : value;	// If no adapter could be found, use the value.
			}
			return value;
		} catch ( Exception exc ) {
			CDEPlugin.getPlugin().getMsgLogger().write(Level.WARNING, new Status(IStatus.WARNING, CDEPlugin.getPlugin().getPluginID(), 0, "", exc)); //$NON-NLS-1$
			return null;	
		}
	}
	
	public boolean isAdapterForType(Object type) {
		return IPropertySource.class.equals(type);
	}
	
	public boolean isPropertySet(Object feature) {
		return ((RefObject) target).refIsSet((RefStructuralFeature) feature);
	}
	
	public void resetPropertyValue(Object feature) {
		((RefObject) target).refUnsetValue((RefStructuralFeature) feature);
	}
	
	public void setPropertyValue(Object feature, Object val) {
		((RefObject) target).refSetValue((RefStructuralFeature) feature, val);
	}
}
