package com.ibm.etools.jbcf;

/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
import java.text.MessageFormat;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import com.ibm.etools.beaninfo.PropertyDecorator;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.emf.ecore.EModelElement;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.proxy.ThrowableProxy;
 
/**
 * This interface is for error holder. It returns severities and errors.
 * @author richkulp
 */
public interface IErrorHolder {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		
	public final int ERROR_SEVERE = 2;
	public final int ERROR_WARNING = 1;
	public final int ERROR_NONE = 0;
	
	public final Class ERROR_HOLDER_TYPE = IErrorHolder.class;	// The type to use to ask for the adapter of type IErrorHolder, in those case where it is being used.	
	
	/**
	 * Return a severity of NONE, SEVERE or WARNING to indicate whether the object
	 * is in error.  
	 */
	public int getErrorStatus();
	/**
	 * Return a list of errors. They will be of type Error.
	 */
	public List getErrors();
	
	public static class ErrorType {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$

		protected String message;
		protected int severity;
		protected static Image fWarningErrorImage;
		protected static Image fSevereErrorImage;

		public ErrorType(String message, int severity) {
			this.message = message;
			this.severity = severity;
		}

		public String getMessage() {
			return message;
		}
		
		public int getSeverity() {
			return severity;
		}

		public Image getImage() {
			switch (severity) {
				case ERROR_SEVERE :
					return getSevereErrorImage();
				case ERROR_WARNING :
					return getWarningErrorImage();
				default :
					return null;
			}
		}

		public static Image getSevereErrorImage() {
			if (fSevereErrorImage == null) {
				fSevereErrorImage = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/full/cview16/error_obj.gif"); //$NON-NLS-1$
			}
			return fSevereErrorImage;
		}
		public static Image getWarningErrorImage() {
			if (fWarningErrorImage == null) {
				fWarningErrorImage = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/full/cview16/warning_obj.gif"); //$NON-NLS-1$
			}
			return fWarningErrorImage;
		}
	}


	public static class ExceptionError extends ErrorType {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
				
		public Throwable error;

		public ExceptionError(Throwable error, int severity) {
			super(null, severity);
			this.error = error;
		}
		public String getMessage() {
			if (error instanceof ThrowableProxy) {
				return MessageFormat.format(JBCFMessages.getString("BeanProxyException.msg"), new Object[] {((ThrowableProxy) error).getTypeProxy().getTypeName(), ((ThrowableProxy) error).getProxyLocalizedMessage()}); //$NON-NLS-1$
			} else {
				return MessageFormat.format(JBCFMessages.getString("BeanProxyException.msg"), new Object[] { error.getClass().getName(), error.getLocalizedMessage()}); //$NON-NLS-1$
			}
		}
	}
	

	/*
	 * Extension of PropertyError when the feature is a multi valued feature.
	 */
	public static class MultiPropertyError extends PropertyError {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected Object fObjectInError;
		public MultiPropertyError(Object errObject, Throwable error, RefStructuralFeature sf) {
			this(errObject, error, ERROR_WARNING, sf);
		}
		
		public MultiPropertyError(Object errObject, Throwable error, int severity, RefStructuralFeature sf) {
			super(error, severity, sf);
			fObjectInError = errObject;
		}
		
		
		public Object getErrorObject() {
			return fObjectInError;	
		}
	}


	
	

	/*
	 *  This is a specific error for when a property is applied
	 * and the target object throws an Throwable
	 */
	public static class PropertyError extends ExceptionError {
		public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected RefStructuralFeature fStructuralFeature;
		
		public PropertyError(Throwable error,RefStructuralFeature sf){	
			this(error, ERROR_WARNING, sf);
		}
		
		public PropertyError(Throwable error, int severity, RefStructuralFeature sf){	
			super(error, severity);
			fStructuralFeature = sf;
		}
	
		
		public String getMessage(){
			PropertyDecorator propertyDecorator = Utilities.getPropertyDecorator( (EModelElement) fStructuralFeature);
			// 230183 paw - We need to return the display name if it has one for NL purposes
			if (propertyDecorator != null)
					return propertyDecorator.getDisplayName() + " : " + super.getMessage(); //$NON-NLS-1$
			return fStructuralFeature.refName() + " : " + super.getMessage(); //$NON-NLS-1$
		}
		
		public RefStructuralFeature getFeature() {
			return fStructuralFeature;
		}
	}
	

				

}
