package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;
import java.net.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import com.ibm.etools.cde.CDEPlugin;
import com.ibm.etools.logging.util.IPluginHelper;
import com.ibm.etools.logging.util.PluginHelperImpl;

public class JBCFPlugin extends Plugin implements IPluginHelper{
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final String PI_JBCF = "com.ibm.etools.jbcf"; // Plugin ID, used for QualifiedName. //$NON-NLS-1$
	public static final String PI_JBCF_REGISTRATIONS = "registrations"; //$NON-NLS-1$
	// ID of the registrations extension point.

	public static final String PI_VARIABLE = "variable"; // <variable> in extension point. //$NON-NLS-1$
	public static final String PI_PATH = "path"; // <path="..."> in extension point.	 //$NON-NLS-1$
	public static final String PI_CONTRIBUTOR = "contributor"; //$NON-NLS-1$
	public static final String PI_PALETTECATS = "palettecats";	//$NON-NLS-1$
	public static final String PI_RUNTIME = "runtime";	//$NON-NLS-1$
	public static final String PI_SOURCE = "source";	//$NON-NLS-1$	
	public static final String PI_SOURCEPREFIX = "prefix"; //$NON-NLS-1$
	public static final String PI_EXTEND = "extend";	//$NON-NLS-1$			
	// <contributor ...> or contributor="..." in extension point

	private static JBCFPlugin PLUGIN;
	private static JBCFMsgLogger msgLogger = null;
    public static final String DEBUG_IOCONSOLE_OPTION = "/debug/ioconsole" ;  //$NON-NLS-1$
	private static int defaultLogLevel = JBCFMsgLogger.LOG_FINEST ;
    public boolean LOG_ECHO_CONSOLE = false ;
    public static Image fJavaBeanImage;
    public static Image fAppletImage;
    public static Image CHECK_IMAGE;

	// Map of registered variable contributors, mapped key is path, value is IConfigurationElement[].
	// It is allowed to have more than one. They will be concatenated together when used.
	private HashMap variableContributors = new HashMap();

	private static ImageDescriptor WIZARD_TITLE_DESC;

	public JBCFPlugin(IPluginDescriptor pd) {
		super(pd);
		PLUGIN = this;
        LOG_ECHO_CONSOLE = "true".equalsIgnoreCase(Platform.getDebugOption( //$NON-NLS-1$
                                     getDescriptor().getUniqueIdentifier()+DEBUG_IOCONSOLE_OPTION
                                     )) ;                                                                                    
	}

	public static JBCFPlugin getPlugin() {
		return PLUGIN;
	}
	

	/*
	 * @see Plugin#startup()
	 */
	public void startup() throws CoreException {
		super.startup();

		processRegistrationExtensionPoint();
	}

	protected void processRegistrationExtensionPoint() {
		// Read in the registration information from the extensions.
		// We'll first gather together in Lists, and then send as arrays at one time to register them.
		HashMap registrations = new HashMap();
		IConfigurationElement[] configs =
			getDescriptor().getExtensionPoint(PI_JBCF_REGISTRATIONS).getConfigurationElements();
		for (int i = 0; i < configs.length; i++) {
			IConfigurationElement iConfigurationElement = configs[i];
			if (PI_VARIABLE.equals(iConfigurationElement.getName())) {
				if (iConfigurationElement.getAttributeAsIs(PI_CONTRIBUTOR) != null
					|| iConfigurationElement.getChildren(PI_CONTRIBUTOR).length > 0
					|| iConfigurationElement.getAttributeAsIs(PI_PALETTECATS) != null) {
					String varpathstr = iConfigurationElement.getAttributeAsIs(PI_PATH);
					if (varpathstr == null)
						continue; // Not proper format.
					IPath varpath = new Path(varpathstr);
					List varentry = (List) registrations.get(varpath);
					if (varentry == null) {
						varentry = new ArrayList(1);
						registrations.put(varpath, varentry);
					}
					varentry.add(iConfigurationElement);
				}
			}
		}

		// Now we've processed all of the extensions.
		Iterator regItr = registrations.entrySet().iterator();
		while (regItr.hasNext()) {
			Map.Entry entry = (Map.Entry) regItr.next();
			List registrationsList = (List) entry.getValue();
			registerRegistration(
				(IPath) entry.getKey(),
				(IConfigurationElement[]) registrationsList.toArray(
					new IConfigurationElement[registrationsList.size()]));
		}
	}

	/**
	 * Register one registration for the path.
	 * The path must be a classpath variable for the first segment. It won't be looked for otherwise.
	 * If it is only one segment long, then it is for the variable itself, and it will be used
	 * for all paths that start with that variable. This allows several different jars within 
	 * the variable's path to share the same registration information.
	 */
	public void registerRegistration(IPath path, IConfigurationElement registration) {
		IConfigurationElement[] registered = (IConfigurationElement[]) variableContributors.get(path);
		if (registered == null)
			registered = new IConfigurationElement[] { registration };
		else {
			IConfigurationElement[] old = registered;
			registered = new IConfigurationElement[old.length + 1];
			System.arraycopy(old, 0, registered, 0, old.length);
			registered[old.length] = registration;
		}

		variableContributors.put(path, registered);
	}

	/**
	 * Register multiple registrations for the path.
	 * The path must be a classpath variable for the first segment. It won't be looked for otherwise.
	 * If it is only one segment long, then it is for the variable itself, and it will be used
	 * for all paths that start with that variable. This allows several different jars within 
	 * the variable's path to share the same beaninfo registration information.
	 */
	public void registerRegistration(IPath path, IConfigurationElement[] registrations) {
		IConfigurationElement[] registered = (IConfigurationElement[]) variableContributors.get(path);
		if (registered == null) {
			registered = new IConfigurationElement[registrations.length];
			System.arraycopy(registrations, 0, registered, 0, registrations.length);
		} else {
			IConfigurationElement[] old = registered;
			registered = new IConfigurationElement[old.length + registrations.length];
			System.arraycopy(old, 0, registered, 0, old.length);
			System.arraycopy(registrations, 0, registered, old.length, registrations.length);
		}

		variableContributors.put(path, registered);
	}
	
	/**
	 * Return the registrations for a specified path. Return null if not registered.
	 */
	public IConfigurationElement[] getRegistrations(IPath path) {
		return (IConfigurationElement[]) variableContributors.get(path);
	}
	
	
	public JBCFMsgLogger getMsgLogger(){
		if(msgLogger == null)
			msgLogger = new JBCFMsgLogger(this);
		return msgLogger;
	}
	/*
	 * @see IPluginHelper#getMsgLoggerConfig(Plugin)
	 */
	public Hashtable getMsgLoggerConfig(Plugin plugin) {
		return new PluginHelperImpl().getMsgLoggerConfig(this);
	}

	/*
	 * @see IPluginHelper#setMsgLoggerConfig(Hashtable)
	 */
	public void setMsgLoggerConfig(Hashtable msgLoggerConfig) {
		getMsgLogger().setMsgLoggerConfig(msgLoggerConfig);
	}

	public static void log(String msg){
		getPlugin().getMsgLogger().log(msg, defaultLogLevel);
	}

	public static void log(String msg, int level){
		getPlugin().getMsgLogger().log(msg, level);
	}
	
	public static void log(Throwable e){
		getPlugin().getMsgLogger().log(e, JBCFMsgLogger.LOG_SEVERE);
	}
	
	public static void log(Throwable e, int level){
		getPlugin().getMsgLogger().log(e, level);
	}
	public static Image getJavaBeanImage(){
		if ( fJavaBeanImage == null ){
			fJavaBeanImage = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/full/cview16/javabean.gif"); //$NON-NLS-1$
		}
		return fJavaBeanImage;
	}
	public static Image getAppletImage(){
		if ( fAppletImage == null ){
			fAppletImage = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/full/cview16/applet.gif"); //$NON-NLS-1$
		}
		return fAppletImage;
	}
	
	public static ImageDescriptor getWizardTitleImageDescriptor() {
		if ( WIZARD_TITLE_DESC == null ) {
			try { 
				WIZARD_TITLE_DESC = ImageDescriptor.createFromURL(new URL("platform:/plugin/com.ibm.etools.jbcf/icons/full/wizban/javabean_wiz.gif")); //$NON-NLS-1$
			} catch ( MalformedURLException exc ) {
			}
		}		
		return WIZARD_TITLE_DESC;
	}

	public static Image getCheckImage() {
		if ( CHECK_IMAGE == null ) {
			CHECK_IMAGE = CDEPlugin.getImageFromURLString("platform:/plugin/com.ibm.etools.jbcf/icons/booleantrue.gif"); //$NON-NLS-1$
		}		
		return CHECK_IMAGE;
	}

}