package com.ibm.etools.jbcf;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.jface.util.ListenerList;

import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.java.instantiation.base.IJavaDataTypeInstance;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.IErrorNotifier.ErrorListener;
import com.ibm.etools.proxy.*;

/**
 * Proxy Host for Primitives (e.g. int, char, etc.).
 * Lightweight proxy host.
 * 
 * @version 	1.0
 * @author
 */
public class PrimitiveProxyAdapter extends AdapterImpl implements IBeanProxyHost {

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private IBeanProxy beanProxy; // It should be accessed only through accessors, even subclasses.
	private boolean ownsProxy;

	// The domain must be given to us so we can create instances
	protected IBeanProxyDomain domain;
	
	// Hold any exception that occurs when we try to create the BeanProxy
	protected Throwable instantiationError;
	protected ListenerList errorListeners;		

	public PrimitiveProxyAdapter(IBeanProxyDomain domain) {
		super();
		this.domain = domain;
	}
	/*
	 * @see IBeanProxyHost#releaseBeanProxy()
	 */
	public void releaseBeanProxy() {
		if (getBeanProxy() != null && getBeanProxy().isValid()) {
			getBeanProxy().getProxyFactoryRegistry().releaseProxy(getBeanProxy());
			// Give it a chance to clean up
		}
		beanProxy = null; // Now throw it away			
	}

	public void reinstantiateChild(IBeanProxyHost aChildHost) {
	}
	/*
	 * @see IBeanProxyHost#getBeanPropertyValue(RefStructuralFeature)
	 */
	public IJavaInstance getBeanPropertyValue(RefStructuralFeature aBeanPropertyFeature) {
		// Primitives have no properties.
		return null;
	}

	/*
	 * @see IBeanProxyHost#getBeanProxy()
	 */
	public IBeanProxy getBeanProxy() {
		return beanProxy;
	}

	/*
	 * @see IBeanProxyHost#getOriginalSettingsTable()
	 */
	public Map getOriginalSettingsTable() {
		return Collections.EMPTY_MAP;
	}

	/*
	 * @see IBeanProxyHost#instantiateBeanProxy()
	 */
	public IBeanProxy instantiateBeanProxy() {
		if (beanProxy == null) {
			IJavaDataTypeInstance jTarget = (IJavaDataTypeInstance) target;
			String qualifiedClassName = jTarget.getJavaType().getQualifiedName();	// I'm primitive, so I know I don't need to do for reflection get name.
			IBeanTypeProxy targetClass = domain.getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy(qualifiedClassName);

			if (jTarget.isSetInitializationString()) {
				String initializationString = jTarget.getInitializationString();
				try {
					beanProxy = targetClass.newInstance(initializationString);
					ownsProxy = true;
				} catch (ThrowableProxy exc) {
					processInstantiationError(exc);					
					JBCFPlugin.log("Could not instantiate " //$NON-NLS-1$
					+qualifiedClassName + " with initialization string=" //$NON-NLS-1$
					+initializationString, JBCFMsgLogger.LOG_WARNING);
					JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
				} catch (InstantiationException exc) {
					processInstantiationError(exc);					
					JBCFPlugin.log("Could not instantiate " //$NON-NLS-1$
					+qualifiedClassName + " with initialization string=" //$NON-NLS-1$
					+initializationString, JBCFMsgLogger.LOG_WARNING);
					JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
				}
			} else {
				try {
					beanProxy = targetClass.newInstance();
					ownsProxy = true;
				} catch (ThrowableProxy exc) {
					processInstantiationError(exc);					
					JBCFPlugin.log("Could not instantiate " + qualifiedClassName, JBCFMsgLogger.LOG_WARNING); //$NON-NLS-1$
					JBCFPlugin.log(exc, JBCFMsgLogger.LOG_WARNING);
				}
			}
		}

		return beanProxy;
	}
	
	protected void processInstantiationError(Throwable exc){
		instantiationError = exc;
		fireSeverityError(IBeanProxyHost.ERROR_SEVERE);
	}
	
	protected void fireSeverityError(int severity){
		if ( errorListeners != null ) {
			Object[] listeners = errorListeners.getListeners();
			for (int i = 0; i < listeners.length; i++) {
				((IBeanProxyHost.ErrorListener)listeners[i]).errorStatus(severity);
			}
		}
	}		

	/*
	 * @see IBeanProxyHost#instantiateBeanProxy(IBeanProxy)
	 */
	public IBeanProxy instantiateBeanProxy(IBeanProxy proxy) {
		ownsProxy = false;
		beanProxy = proxy;
		return beanProxy;
	}

	/*
	 * @see IBeanProxyHost#isBeanProxyInstantiated()
	 */
	public boolean isBeanProxyInstantiated() {
		return beanProxy != null;
	}

	/*
	 * @see IBeanProxyHost#revalidateBeanProxy()
	 */
	public void revalidateBeanProxy() {
	}

	/*
	 * @see IBeanProxyHost#setBeanProxy(IBeanProxy)
	 */
	public void setBeanProxy(IBeanProxy beanProxy) {
		this.beanProxy = beanProxy;
		ownsProxy = false;
	}

	/*
	 * @see IBeanProxyHost#setOwnsProxy(boolean)
	 */
	public void setOwnsProxy(boolean ownsProxy) {
		this.ownsProxy = ownsProxy;
	}

	/*
	 * @see IBeanProxyHost#getBeanProxyDomain()
	 */
	public IBeanProxyDomain getBeanProxyDomain() {
		return domain;
	}

	/*
	 * @see Adapter#isAdapterForType(Object)
	 */
	public boolean isAdapterForType(Object type) {
		return BEAN_PROXY_TYPE == type || ERROR_NOTIFIER_TYPE == type || ERROR_HOLDER_TYPE == type;
	}
	
	public int getErrorStatus(){
		
		if (instantiationError != null)
			return ERROR_SEVERE;
		else
			return ERROR_NONE;
	}
	
	public List getErrors() {
		if (instantiationError != null)
			return Collections.nCopies(1, new IErrorNotifier.ExceptionError(instantiationError,ERROR_SEVERE));
		else
			return Collections.EMPTY_LIST;
	}

	public void addErrorListener(ErrorListener aListener) {
		if(errorListeners == null) errorListeners = new ListenerList(2);
		errorListeners.add(aListener);
	}
	
	public void removeErrorListener(ErrorListener aListener) {
		if (errorListeners != null)
			errorListeners.remove(aListener);
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#invalidateBeanProxy()
	 */
	public void invalidateBeanProxy() {
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#validateBeanProxy()
	 */
	public void validateBeanProxy() {
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#getBeanProxyValue(RefStructuralFeature)
	 */
	public IBeanProxy getBeanPropertyProxyValue(RefStructuralFeature aBeanPropertyFeature) {
		// There are no properties of a primitive.
		return null;
	}

	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#applyBeanPropertyProxyValue(RefStructuralFeature, IBeanProxy)
	 */
	public void applyBeanPropertyProxyValue(RefStructuralFeature aBeanPropertyFeature, IBeanProxy aproxy) {
		// There are no properties of a primitive.		
	}

}