package com.ibm.etools.jbcf.codegen.util;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.JBCFMsgLogger;
import com.ibm.etools.jbcf.JBCFPlugin;

import org.eclipse.swt.widgets.Display ;
import org.eclipse.jface.text.IDocumentListener ;


/**
 * The Worker class is responsible for running and controlling connections. <br>
 * A worker runs in its own thread and waits for connections that need to be performed. <br>
 * A WorkerPool object owning a reference to a Worker object can assign a connection to it,
 * and also shut it down in an orderly manner.
 */
public class StrategyWorker implements Runnable{
	

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
  /**
   * the worker pool owning this worker
   */
  private WorkerPool              fPool;
  
  private IBackGroundWorkStrategy fStrategy = null ;
  private ICancelMonitor          fMonitor = null ;
  private Object[]                fworkItems = null ;
  private Display                 fdisplay = null ;
  private IDocumentListener       fdocListener = null ;


  /**
   * the thread running this object
   */
  private Thread           workerThread = null;

  /**
   * loop continuation flag
   */
  private volatile boolean cont = true;
  //
  // For graceful cancelling
  //
  boolean fCancel = false;

  /**
   * Creates a worker owned to a specified worker pool.
   */
  public StrategyWorker (WorkerPool pool) {
	fPool = pool;
  }  
  /**
   * Assigns a connection to this worker and signals it to perform it.
   * @param connection the connection to be performed.
   */
  public synchronized void assignStrategy(IBackGroundWorkStrategy strat, Object[] items, IDocumentListener docListener, Display disp, ICancelMonitor monitor) {
  	fStrategy = strat ;
  	fMonitor = monitor ;
  	fworkItems = items ;
  	fdisplay = disp ;
  	fdocListener = docListener ;
	notify();
  }  
  void cancel() {
  	fCancel = true;
  }  
  /**
   * Shuts down this worker. <br>
   * If this worker is currently running a conneection, it's notified that it should finish as soon as possible.
   */
  public void finish() {
	cont = false;	
	if (workerThread != null) workerThread.interrupt();
  }  
  boolean isCancel() {return fCancel;}  
  /**
   * Runs the worker thread. <br>
   * Initially, the worker is in an idle mode, and waits until a connection is assigned to it. <br>
   * When a connection has been assigned to it, the worker invokes its <code>perform</code> method. <br>
   * When the connection is done, the worker returns itself to the worker pool and goes back to wait
   * for a new connection.
   */
public void run () {
  	
  workerThread = Thread.currentThread();	
  mainloop: while (cont) {

    try {
  	
  	  waitForStrategy() ;
	  /* save a reference to this thread */
	

  	  JBCFPlugin.log ("StrategyWorker running: "+fStrategy.getClass().getName(), JBCFMsgLogger.LOG_FINE) ; //$NON-NLS-1$
  	  if (fMonitor == null || !fMonitor.isCanceled()) {
  	     fStrategy.run(fdisplay,fdocListener, fworkItems,fMonitor) ;  	     	  	   
  	  }
  	  
    } 
    catch (InterruptedException e) {}
    catch (Throwable t) {
    	try {
    	 JBCFPlugin.log(t, JBCFMsgLogger.LOG_WARNING) ;
    	}
    	catch (Throwable tt) {}
    }
    finally {
    	fStrategy = null ;  	
  	fworkItems = null;     	
    	if (fMonitor != null)
    	   fMonitor.setCompleted() ;
    	fMonitor = null ;
    	fPool.returnWorker(this);	
    }
  }		
}  
  
  private synchronized void waitForStrategy() throws InterruptedException {
	while(fStrategy == null)
	  wait(); 
  }  
                        	
	
}