package com.ibm.etools.jbcf.vce.javabeanviewer;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


import java.util.*;
import com.ibm.etools.draw2d.*;
import com.ibm.etools.draw2d.geometry.*;
/**
 * This class implements the <pre>ILayoutManager</pre> interface 
 * using the XY Layout algorithm. This lays out the components 
 * using the layout constraints as defined by each component.
 */

public class SizeLayout
	extends AbstractLayout
{
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
protected Map constraints = new HashMap();

/**
 * Sets the layout constraint of the given figure.
 * The constraints can only be of type Rectangle.
 *
 * @param figure  Figure for which the constarint is being set.
 * @param newConstraint  Constraint for the input figure.
 * @see  #getConstraint(IFigure)
 */
public void setConstraint(IFigure figure, Object newConstraint) {
	super.setConstraint(figure, newConstraint);
	if (newConstraint != null)
		constraints.put(figure, newConstraint);
}

/**
 * Calculates and returns the preferred size of the input figure.
 * Since in XYLayout the location of the child should be preserved,
 * the preferred size would be a region which would hold all the
 * children of the input figure. If no constraint is set, that
 * child is ignored for calculation. If width and height are not
 * positive, the preferred dimensions of the child are taken.
 * 
 * @param f  Figure for which the preferred size is required.
 * @return  Preferred size of the input figure.
 */
protected Dimension calculatePreferredSize(IFigure f){
	Rectangle rect = new Rectangle();
	ListIterator children = f.getChildren().listIterator();
	while (children.hasNext()){
		IFigure child = (IFigure)children.next();
		Rectangle r = (Rectangle)constraints.get(child);
		if (r == null)
			continue;
		if(r.width==-1 && r.height==-1)
			r = r.getResized(child.getPreferredSize());
		rect.union(r);
	}
	Dimension d = rect.getSize();
	Insets insets = f.getInsets();
	return new Dimension(d.width + insets.getWidth(), d.height + insets.getHeight()).
		union(getBorderPreferredSize(f));
}

/**
 * Implements the algorithm to layout the components of the given container figure.
 * Each component is laid out using it's own layout constraint specifying it's size
 * and position.
 *
 * @param parent  Parent whose children are to be arranged.
 */
public void layout(IFigure parent) {
	Iterator children = parent.getChildren().iterator();
	Point offset = parent.getClientArea().getLocation();
	IFigure f;
	int x=2;
	int y=2;
	while (children.hasNext()) {
		f = (IFigure)children.next();
		Rectangle bounds = (Rectangle) getConstraint(f);
		if (bounds == null) continue;
		bounds = bounds.getTranslated(offset);
		bounds.x = x;
		bounds.y = y;
		if (bounds.width == -1 || bounds.height == -1) {
			Dimension preferredSize = f.getPreferredSize();
			bounds = bounds.getCopy();
			if (bounds.width == -1)
				bounds.width = preferredSize.width;
			if (bounds.height == -1)
				bounds.height = preferredSize.height;
		}
		// Layout the components from left to right with a space of five
		x = x + bounds.width + 5;
		f.setBounds(bounds);
	}
}

/**
 * Returns the layout constraint for the given figure.
 * If no constraint has been set for the receiver so far,
 * set the default constraints based on the figure.
 *
 * @param figure   Figure whose constraint is desired.
 * @see  #setConstraint(IFigure, Object)
 */
public Object getConstraint(IFigure figure) {
	return constraints.get(figure);
}

/**
 * Removes the input figure from the list of constraints.
 *
 * @param figure  Figure which should be removed from the list. 
 */
public void remove(IFigure figure){
	super.remove(figure);
	constraints.remove(figure);
}

}


