package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.common.command.*;
import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cde.emf.EMFEditDomainHelper;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.draw2d.geometry.Rectangle;
import com.ibm.etools.jbcf.BeanUtilities;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.awt.IRectangleBeanProxy;
/**
 * Command to apply the Null Layout constraint(s). This is
 * a command because it is complicated because of the
 * possibility of storing in bounds, size, and/or location.
 */
public class ApplyNullLayoutConstraintCommand extends CommandWrapper {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IJavaObjectInstance target;
	protected ResourceSet rset;
	protected Rectangle constraint;	// Could be a Rectangle, Point, or Dimension.
	protected byte changed = NO_CHANGE;
	protected final static byte
		NO_CHANGE = 0,
		MOVED = 0x1,
		RESIZED = 0x2;
	
	public ApplyNullLayoutConstraintCommand(String label) {
		super(label);
	}

	public ApplyNullLayoutConstraintCommand() {
	}


	public void setTarget(IJavaObjectInstance target) {
		this.target = target;
	}

	public void setDomain(EditDomain domain) {
		rset = EMFEditDomainHelper.getResourceSet(domain);
	}
			
	public void setConstraint(Rectangle constraint, boolean moved, boolean resized) {
		if (moved)
			changed |= MOVED;
		if (resized)
			changed |= RESIZED;
		this.constraint = constraint;
	}
	
	protected boolean prepare() {
		// Need to override prepare because prepare expects to have a command
		// create, and at the time of prepare being called, we don't have a command yet.
		return target != null && rset != null && constraint != null;
	}
	
	public void execute() {
		// It will determine what to set. If bounds are set, it will set that, otherwise it will try size/location. If neither
		// is set, then it will set a bounds.
		CommandBuilder cb = new CommandBuilder(null);
		EStructuralFeature sfComponentBounds = JavaInstantiation.getSFeature(target, JBCFConstants.SF_COMPONENT_BOUNDS);					
		if (target.refIsSet(sfComponentBounds)) {
			// We have a bounds.
			IRectangleBeanProxy oldBounds = (IRectangleBeanProxy) BeanProxyUtilities.getBeanProxy((IJavaObjectInstance) target.refValue(sfComponentBounds), rset);
			int x, y, width, height;
			if ((changed & MOVED) != 0) {
				x = constraint.x;
				y = constraint.y;
			} else {
				x = oldBounds.getX();
				y = oldBounds.getY();
			}
			if ((changed & RESIZED) != 0) {
				width = constraint.width;
				height = constraint.height;
			} else {
				width = oldBounds.getWidth();
				height = oldBounds.getHeight();
			}
			Object bounds = BeanUtilities.createJavaObject("java.awt.Rectangle", rset, //$NON-NLS-1$
				RectangleJavaClassCellEditor.getJavaInitializationString(x, y, width, height));
			cb.applyAttributeSetting(target, sfComponentBounds, bounds);
		} else {
			EStructuralFeature sfComponentSize = JavaInstantiation.getSFeature(target, JBCFConstants.SF_COMPONENT_SIZE);			
			if ((changed & RESIZED) != 0 && (changed & MOVED) == 0) {
				// We want size only. In that case, we will set the size. This is usually on the freeform.
				Object size = BeanUtilities.createJavaObject(
					"java.awt.Dimension", //$NON-NLS-1$
					rset,
					DimensionJavaClassCellEditor.getJavaInitializationString(constraint.width, constraint.height));
				cb.applyAttributeSetting(target, sfComponentSize, size);
			} else {
				EStructuralFeature sfComponentLocation = JavaInstantiation.getSFeature(target, JBCFConstants.SF_COMPONENT_LOCATION);
				if (target.refIsSet(sfComponentSize) || target.refIsSet(sfComponentLocation)) {
					// One of them is set, so we are not using bounds. If neither was set, we would drop to bounds.
					if ((changed & RESIZED) != 0) {
						Object size = BeanUtilities.createJavaObject(
							"java.awt.Dimension", //$NON-NLS-1$
							rset,
							DimensionJavaClassCellEditor.getJavaInitializationString(constraint.width, constraint.height));
						cb.applyAttributeSetting(target, sfComponentSize, size);
					}
					
					if ((changed & MOVED) != 0) {
						Object loc = BeanUtilities.createJavaObject(
							"java.awt.Point", //$NON-NLS-1$
							rset,
							PointJavaClassCellEditor.getJavaInitializationString(constraint.x, constraint.y));
						cb.applyAttributeSetting(target, sfComponentLocation, loc);
					}
				} else {
					// If we got this far, then we are applying bounds because we had no bounds, size, or location set.
					Object bounds = BeanUtilities.createJavaObject(
						"java.awt.Rectangle", //$NON-NLS-1$
						rset,
						RectangleJavaClassCellEditor.getJavaInitializationString(constraint));
					cb.applyAttributeSetting(target, sfComponentBounds, bounds);
				}
			}
		}
		command = cb.getCommand();
		command.execute();
	}
}