package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import com.ibm.etools.gef.editparts.AbstractGraphicalEditPart;
import com.ibm.etools.cde.ImageFigure;
import com.ibm.etools.cde.ImageFigureController;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.draw2d.Label;
import com.ibm.etools.draw2d.geometry.Point;
import com.ibm.etools.draw2d.geometry.Rectangle;
import com.ibm.etools.cde.OutlineBorder;
import com.ibm.etools.gef.EditPolicy;
import com.ibm.etools.cde.DefaultComponentEditPolicy;
import com.ibm.etools.cde.IVisualComponent;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.jbcf.ErrorFigure;
import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.jbcf.BeanProxyUtilities;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.ui.views.properties.IPropertySource;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.cde.IConstraintHandler;
/**
 * EditPart for a java.awt.Component.
 * The parent editpart is responsible to set transparent.
 * If transparent, then there won't be any image capture.
 *
 * Note: Bit of a kludge, but because a component could be
 * either on the freeform or within another component, we
 * need to differentiate for propertySource between the
 * component and the componentConstraint (or whatever).
 * This is so that when on the freeform, the component property source
 * is it, but when on another component, we want to merge the
 * constraint info with the component features. This is done by
 * setting into the editpart who should be used as the model to find the propertySource.
 * The default is the model of the editpart, while if set, should use the propertySourceModel.
 * The component itself is considered to be the model. This is so that when moving a component
 * between two different parents, it will see the component and not the constraint as to what
 * is being added. The policy will create a new constraint for where it is going.
 * 
 * The initialization data is used to configure whether this edit part should show borders on
 * the figure or not. The default is false. The initialization data can be "true" or "false",
 * case-insensitive. This init data can be placed in the xmi to determine what it should be.
 */
public class ComponentGraphicalEditPart extends AbstractGraphicalEditPart implements IExecutableExtension {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected boolean transparent = false;	// Whether there should be an image or not.
	protected ImageFigureController imageFigureController;
	protected IPropertySource propertySource;	// This is the property source. 
	protected ComponentModelAdapter constraintHandler;	// A subclass of this is for the editpart constraint handler.\
	protected IBeanProxyHost.ErrorListener fBeanProxyErrorListener;
	protected boolean border = false;	// Whether there should be a border or not around the figure.

public ComponentGraphicalEditPart(Object model) {
	setModel(model);
}

	/**
	 * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(IConfigurationElement, String, Object)
	 */
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		if (data instanceof String)
			border = Boolean.valueOf((String) data).booleanValue();
	}
	
protected IFigure createFigure() {
	ImageFigure fig = new ImageFigure();
	if (border)
		fig.setBorder(new OutlineBorder());
	fig.setOpaque(!transparent);
	if (!transparent) {
		imageFigureController = new ImageFigureController();
		imageFigureController.setImageFigure(fig);
	}
	return fig;
}

public Object getAdapter(Class type) {
	if (type == IVisualComponent.class)
		return getVisualComponent();
	else if (type == IPropertySource.class)
		if (propertySource != null)
			return propertySource;
		else
			return ((IJavaInstance) getModel()).getAdapter(IPropertySource.class);
	else if (type == IConstraintHandler.class) {
		if (constraintHandler == null) {
			// It can be used as is because only change size needs to be handled.
			// The figure constraint will be -1,-1.
			constraintHandler = new ComponentModelAdapter(getModel());
		}
		return constraintHandler;
	}
	Object result = super.getAdapter(type);
	if ( result != null ) {
		return result;
	} else {
		// See if any of the MOF adapters on our target can return a value for the request
		Iterator mofAdapters = ((IJavaInstance)getModel()).getAdapters().iterator();
		while(mofAdapters.hasNext()){
			Object mofAdapter = mofAdapters.next();
			if ( mofAdapter instanceof IAdaptable ) {
				Object mofAdapterAdapter = ((IAdaptable)mofAdapter).getAdapter(type);
				if ( mofAdapterAdapter != null ) {
					return mofAdapterAdapter;
				}
			}
		}
	}
	return null;
}	
public void setPropertySource(IPropertySource source) {
	propertySource = source;
}


protected ErrorFigure fErrorIndicator;

/**
 * If we are transparent, then have the proxy set to don't apply visible. If on the
 * freeform (i.e. not transparent) we want it to always be visible.
 */
public void activate() {
	super.activate();
		
	if (!transparent) {
		imageFigureController.setImageNotifier(getVisualComponent());
	}
	
	if ( fErrorIndicator == null){
		fErrorIndicator = new ErrorFigure(IBeanProxyHost.ERROR_NONE);
		getFigure().add(fErrorIndicator);
	}
	
	// Listen to the IBeanProxyHost so it tells us when errors occur
	fBeanProxyErrorListener = new IErrorNotifier.ErrorListenerAdapter(){
		public void errorStatus(int severity){
			setSeverity(severity);
		}
	};
	
	setSeverity(getComponentProxy().getErrorStatus());	// Set the initial status
	
	BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel()).addErrorListener(fBeanProxyErrorListener);
	
}

protected void setSeverity(int severity) {
	fErrorIndicator.sevSeverity(severity);
	getFigure().setVisible(!(severity == IBeanProxyHost.ERROR_SEVERE));
}

public void deactivate() {
	if (imageFigureController != null)
		imageFigureController.deactivate();
	if ( fBeanProxyErrorListener != null){
		IBeanProxyHost beanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance)getModel());
		beanProxyHost.removeErrorListener(fBeanProxyErrorListener);
	}
	super.deactivate();
}

/**
 * createInputPolicies method comment.
 */
protected void createEditPolicies() {
	installEditPolicy(EditPolicy.COMPONENT_ROLE, new DefaultComponentEditPolicy());
}

/**
 * Called when the context menu is about to open.
 * Override to add your own context dependent menu contributions.
 */
//public void fillContextMenu(com.ibm.jface.ContextMenuManager menu, com.ibm.jface.IViewer viewer) {
//	super.fillContextMenu(menu, viewer);
//	com.ibm.jface.util.HackFinder.revisit(); 	// Needs a more general place for this
//	if (CustomizerAction.hasCustomizer(objectModel)) {
//		menu.add(new CustomizerAction(VCENls.RESBUNDLE, "CustomizeAction.", objectModel, (String) ((VCEElementAdaptor) getModel()).getElementProperty(com.ibm.jface.elements.IBasicProperties.P_LABEL)));//$NON-NLS-1$
//	}
//	// The layout managers can contribute to a menu called "layout"
//	MenuManager layoutMenu = new MenuManager("layout");
//	// See whether any of our input policies want to contribute to the menu
//	InputPolicyIterator i = getInputPolicyIterator();
//	while ( i.hasNext() ) {
//		IInputPolicy ip = i.next();
//		if ( ip instanceof IVCELayoutPolicy ) {
//			((IVCELayoutPolicy)ip).fillContextMenu(layoutMenu , viewer);
//		}
//	}
	// See if our parent wants to contribute to the menu.
	// In the case where we are on the free form ( like a top level frame ) out parent will
	// not be a ContainerGraphViewObject so we need to do the instaneof check
//	if ( getParent() instanceof ContainerGraphViewObject ) {
//		Enumeration enum = ((ContainerGraphViewObject)getParent()).getInputPolicies();
//		while ( enum.hasMoreElements() ) {
//			Object ip = enum.nextElement();
//			if ( ip instanceof IVCELayoutPolicy ) {
//				((IVCELayoutPolicy)ip).fillChildContextMenu(layoutMenu , viewer , this);
//			}
//		}
//	}
	// If any contributions were made to the layout menu add it to the pop-up menu
	// This way we don't have a layout menu with no options on it
//	Enumeration enum = layoutMenu.elements();
//	if ( layoutMenu.elements().hasMoreElements() ) {
//		menu.add(layoutMenu);
//	}
//	return;
//}
/**
 * Return the IBean
 */
public IJavaInstance getBean() {
	return (IJavaInstance) getModel();
}
/**
 * Return the ComponentProxy.
 * Creation date: (3/15/00 12:33:41 PM)
 * @return com.ibm.vce.figures.IVisualComponent
 */
protected ComponentProxyAdapter getComponentProxy() {
	IBeanProxyHost beanProxy = BeanProxyUtilities.getBeanProxyHost(getBean());
	return (ComponentProxyAdapter) beanProxy;
}

/**
 * Parents of this will call this and pass in true
 * if the image should be retrieved. It is important that
 * this is called by parent editpart before the
 * editpart is activated (this is best done in 
 * the createEditPart method of the parent edit part).
 */
public void setTransparent(boolean transparent) {
	this.transparent = transparent;
}

/**
 * Return the visual component.
 * Creation date: (3/15/00 12:33:41 PM)
 * @return com.ibm.vce.figures.IVisualComponent
 */
protected IVisualComponent getVisualComponent() {
	return (IVisualComponent) getComponentProxy(); // For AWT, the component proxy is the visual component.
}
}
