package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.properties.PropertySourceAdapter;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.propertysheet.ISourcedPropertyDescriptor;
import com.ibm.etools.propertysheet.ICommandPropertyDescriptor;
import com.ibm.etools.propertysheet.WrapperedPropertyDescriptor;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.proxy.IBeanProxy;
import com.ibm.etools.jbcf.IBeanProxyDomain;
import com.ibm.etools.jbcf.IBeanProxyHost;
/**
 * Default PropertySourceAdapter for ConstraintComponent sources and subclasses.
 *
 * By default it will merge in the properties from the component with the constraint
 * property. If layout manager is null, or the layout manager is not
 * an instance of LayoutManager2, then the constraint property will not be
 * returned. If the layout manager is not null, then bounds/size/location are
 * hidden and not returned. When applying/retrieving properties, if they are for the "component" they
 * will forwarded over to the component.
 */

public class ConstraintComponentPropertySourceAdapter extends PropertySourceAdapter {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IPropertySource componentPS;
	protected IPropertyDescriptor constraintDescriptor;
	protected EStructuralFeature sfConstraintConstraint;
	
	
	/*
	 * @see Adapter#setTarget(Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		super.setTarget(newTarget);
		
		sfConstraintConstraint = JavaInstantiation.getSFeature(((RefObject) newTarget).getResourceSet(), JBCFConstants.SF_CONSTRAINT_CONSTRAINT);
	}
	
	/*
	 * Need to merge in the properties of the component.
	 * We will have to wrapper any ISourced or ICommand descriptors because
	 * those will have the wrong source passed into them. We need to intercept
	 * and have the correct source (the component) passed into them.
	 *
	 * 
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		
		IBeanProxyHost containerProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) ((RefObject) getRefTarget().refContainer()));
		boolean containerBad = containerProxyHost.getErrorStatus() == IBeanProxyHost.ERROR_SEVERE; 
		IBeanProxy layoutManagerProxy =  !containerBad ? BeanAwtUtilities.invoke_getLayout(containerProxyHost.getBeanProxy()) : null;
		
		RefObject component = (RefObject) getRefTarget().refValue(JavaInstantiation.getSFeature(getRefTarget().getResourceSet(), JBCFConstants.SF_CONSTRAINT_COMPONENT));
		IPropertyDescriptor[] theirs = null;
		IPropertyDescriptor[] wrappedTheirs = null;
		int wi = 0;		
		if (component != null) {
			componentPS = (IPropertySource) component.getAdapter(IPropertySource.class);
			theirs = componentPS.getPropertyDescriptors();
			
			wrappedTheirs = new IPropertyDescriptor[theirs.length];
			for (int i = 0; i<theirs.length; i++) {
				IPropertyDescriptor pd = theirs[i];
				if ((containerBad || layoutManagerProxy != null) && pd.getId() instanceof RefStructuralFeature) {
					// exclude bounds/size/location because we have a layout manager, or the container is bad.
					String fn = ((RefStructuralFeature) pd.getId()).refName();
					if ("bounds".equals(fn) || "size".equals(fn) || "location".equals(fn)) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						continue;
				}
				if (pd instanceof ISourcedPropertyDescriptor || pd instanceof ICommandPropertyDescriptor)
					wrappedTheirs[wi++] = new WrapperedPropertyDescriptor(componentPS, pd);
				else
					wrappedTheirs[wi++] = pd;
			}
		} else {
			theirs = new IPropertyDescriptor[0];
			wrappedTheirs = new IPropertyDescriptor[0];
		}
		
		// Now get our constraint property descriptor, if any.
		EditDomain domain = containerProxyHost.getBeanProxyDomain().getEditDomain();
		ILayoutPolicyFactory fact = !containerBad ? BeanAwtUtilities.getLayoutPolicyFactoryFromLayoutManger(layoutManagerProxy, domain) : new UnknownLayout2PolicyFactory();
		constraintDescriptor = fact.getConstraintPropertyDescriptor(sfConstraintConstraint);
		
		// Finally build the complete list.
		IPropertyDescriptor[] finalList = null;
		if (constraintDescriptor != null) {
			finalList = new IPropertyDescriptor[1 + wi];
			finalList[0] = constraintDescriptor;
			System.arraycopy(wrappedTheirs, 0, finalList, 1, wi);
		} else {
			if (wi != theirs.length) {
				// we've removed some, so make a smaller array
				finalList = new IPropertyDescriptor[wi];
				System.arraycopy(wrappedTheirs, 0, finalList, 0, wi);
			} else
				finalList = wrappedTheirs;
		}
		return finalList;
	}
		
	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public Object getPropertyValue(Object feature) {
		if (feature == sfConstraintConstraint)
			return super.getPropertyValue(feature);
			
		return componentPS != null ? componentPS.getPropertyValue(feature) : null;
	}
	
	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public boolean isPropertySet(Object feature) {
		if (feature == sfConstraintConstraint)
			return super.isPropertySet(feature);
			
		return componentPS != null ? componentPS.isPropertySet(feature) : false;
	}	

	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public void resetPropertyValue(Object feature) {
		if (feature == sfConstraintConstraint)
			super.resetPropertyValue(feature);
		else if (componentPS != null)
			componentPS.resetPropertyValue(feature);
	}	
	
	/*
	 * If one of ours, send it on up, else send it to the component.
	 */
	public void setPropertyValue(Object feature, Object val)  {
		if (feature == sfConstraintConstraint)
			super.setPropertyValue(feature, val);
		else if (componentPS != null)
			componentPS.setPropertyValue(feature, val);
	}	


}