package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.cde.*;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cde.emf.EMFContainerPolicy;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.requests.CreateRequest;
import com.ibm.etools.gef.requests.GroupRequest;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;
/**
 * AWT Container policy for handling children in an AWT container.
 * This is special in that none of the add/create commands
 * come through the standard methods. This is because they are
 * special in Containers. The children are ConstraintComponents but
 * the model object of the child editpart is awt.Component. So the
 * requests wouldn't have the ConstraintComponent in them.
 * So the layout and tree editpolicies will cast directly to
 * ContainerEditPolicy and use the special methods listed here.
 *
 * The delete, move, and orphan commands can come through the
 * standard way because no pre-processing is needed in the
 * layout/tree policies to strip out the components and constraints.
 */
public class ContainerPolicy extends EMFContainerPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfConstraintComponent;
	protected EClass 
		classConstraintComponent,
		classComponent;
	
	public ContainerPolicy(EditDomain domain) {
		super(JavaInstantiation.getSFeature(JBCFEditDomainHelper.getResourceSet(domain), JBCFConstants.SF_CONTAINER_COMPONENTS), domain);
		
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain);
		sfConstraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		classConstraintComponent = (EClass) rset.getObject(JBCFConstants.CLASS_CONTAINER_CONSTRAINTCOMPONENT);
		classComponent = (EClass) sfConstraintComponent.getETypeClassifier();
	}
	
	protected boolean isValidChild(Object child) {
		return classConstraintComponent.isInstance(child);
	}
	
	protected boolean isValidComponent(Object component) {
		// Also need to verify that the component is a valid component
		return classComponent.isInstance(component);
	}
	
	public Command getCreateCommand(CreateRequest request, Object positionBeforeChild) {	
		return UnexecutableCommand.INSTANCE;	// This should never be called. It should go through the getCreateCommand(constraint, component, position);
	}
	
	/**
	 * Get the create command for the child constraint listed here at the position.
	 */
	public Command getCreateCommand(Object childConstraintComponent, Object childComponent, Object positionBeforeChild) {
		// Verify that the component and the child are valid.
		if (!isValidChild(childConstraintComponent) || !isValidComponent(childComponent) || !isParentAcceptable(childComponent))
			return UnexecutableCommand.INSTANCE;
			
		// Add the child to the component.
		CommandBuilder cb = new CommandBuilder(""); //$NON-NLS-1$
		cb.applyAttributeSetting((RefObject) childConstraintComponent, sfConstraintComponent, childComponent);
		// The annotation on the childComponent won't be picked up in the primCreateCommand because the childComponent is
		// not the child of the container, and it is not yet a child of the constraintComponent. The constraintComponent
		// is the child of the container, and since the childComponent is not yet in the constraintComponent, its annotations
		// won't be found. So we have to do it explicitly.
		List annotations = AnnotationPolicy.getAllAnnotations(new ArrayList(), childComponent, domain.getAnnotationLinkagePolicy());
		cb.append(AnnotationPolicy.getDefaultAddAnnotationsCommand(annotations, domain));
		cb.append(primCreateCommand(childConstraintComponent, positionBeforeChild != null ? ((RefObject) positionBeforeChild).refContainer() : null, containmentSF));
		return cb.getCommand();
	}
	
	
	public Command getAddCommand(GroupRequest request, Object position) {
		return UnexecutableCommand.INSTANCE;	// This should never be called. It should go through the getAddCommand(constraints, components, position);
	}

	/**
	 * Get the add constraints and components command for the list and position.
	 */
	public Command getAddCommand(List constraints, List children, Object positionBeforeChild) {
		// We need to create the add children to constraints command first.
		// This is because we can't set the children into the constraints except
		// at command execution time. If we do it now it would immediately
		// make the change and this would strip the child out even if the commands
		// were never executed.
		//
		// It is assumed that both lists are the same length.

		CommandBuilder cb = new CommandBuilder(""); //$NON-NLS-1$
		Iterator cons = constraints.iterator();
		Iterator chlds = children.iterator();
		while(cons.hasNext()) {
			Object childConstraintComponent = cons.next();
			Object childComponent = chlds.next();
			if (!isValidChild(childConstraintComponent) || !isValidComponent(childComponent) || !isParentAcceptable(childComponent))
				return UnexecutableCommand.INSTANCE;
			cb.applyAttributeSetting((RefObject) childConstraintComponent, sfConstraintComponent, childComponent);
		}
		
		cb.append(primAddCommand(constraints, positionBeforeChild != null ? ((RefObject) positionBeforeChild).refContainer() : null, containmentSF));
		return cb.getCommand();
	}
	
	/**
	 * Delete the dependent. The child is the component, not the constraintComponent.
	 */
	public Command getDeleteDependentCommand(Object child) {
		return super.getDeleteDependentCommand(((RefObject) child).refContainer());
	}
	
	
	/**
	 * Get the move children command for the list. The children
	 * are the components, not the constraintComponents.
	 */	
	public Command getMoveChildrenCommand(List children, Object positionBeforeChild) {
		// We need to convert to the constraintComponents, not the components.
		List constraints = new ArrayList(children.size());
		Iterator itr = children.iterator();
		while(itr.hasNext()) {
			constraints.add(((RefObject) itr.next()).refContainer());
		}
		return super.getMoveChildrenCommand(constraints, positionBeforeChild != null ? ((RefObject) positionBeforeChild).refContainer() : null);
	}
		

	/**
	 * Get the orphan command for the list. The children
	 * are the components, not the constraintComponents.
	 */
	public Command getOrphanChildrenCommand(List children) {
		// We need to unset the components from the constraints after
		// orphaning the constraints so that they are free of any
		// containment when they are added to their new parent. If we
		// didn't unset the components, then upon undo the component
		// would not be in the constraint when it is added back in.

		List constraints = new ArrayList(children.size());
		Iterator itr = children.iterator();
		while(itr.hasNext()) {
			constraints.add(((RefObject) itr.next()).refContainer());
		}
		
		CommandBuilder cb = new CommandBuilder(null);
		cb.append(super.getOrphanChildrenCommand(constraints));
		cb.cancelGroupAttributeSetting(constraints, sfConstraintComponent);	// Cancel out all of the component settings.		
		return cb.getCommand();
	}

}