package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.AnnotationPolicy;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cde.emf.AbstractEMFContainerPolicy;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.common.command.UnexecutableCommand;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.IIntegerBeanProxy;

/**
 * Container policy for JSplitPane
 *
 * General information about JSplitPane:
 * JSplitPane's children are really set as the left or top component and the right or bottom component. 
 * If the JSplitPane's orientation is HORIZONTAL_SPLIT, the components will be set left and right.
 * If the JSplitPane's orientation is VERTICAL_SPLIT, the components will be set top and bottom.
 * We'll check for both since we don't know the orientation... and even if we did, Sun lets you set
 * the components regardless of what the orientation is set to. In other words, you can set the left
 * and right components even though the orientation is vertical... and the opposite is true as well. 
 */
public class JSplitPaneContainerPolicy extends AbstractEMFContainerPolicy {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	public static final int VERTICAL_SPLIT = 0;
	public static final int HORIZONTAL_SPLIT = 1;

	protected JavaClass classComponent;
	protected EStructuralFeature sfLeftComponent,
		sfRightComponent,
		sfTopComponent,
		sfBottomComponent,
		sf_constraintComponent,
		sf_constraintConstraint,
		sf_containerComponents;
	protected RefObject leftComp,
		rightComp,
		topComp,
		bottomComp;

	public JSplitPaneContainerPolicy(EditDomain domain) {
		super(domain);

		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain);
		sfLeftComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_LEFTCOMPONENT);
		sfRightComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_RIGHTCOMPONENT);
		sfBottomComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_BOTTOMCOMPONENT);
		sfTopComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JSPLITPANE_TOPCOMPONENT);
		sf_constraintComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_COMPONENT);
		sf_constraintConstraint = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONSTRAINT_CONSTRAINT);
		sf_containerComponents = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_CONTAINER_COMPONENTS);
		classComponent = (JavaClass) Utilities.getJavaClass("java.awt.Component", rset); //$NON-NLS-1$
	}
	/**
	 * Determine which components have been set in the Splitpane
	 */
	protected void determineSplitpaneOccupants() {
		leftComp = (RefObject) ((RefObject) container).refValue(sfLeftComponent);
		topComp = (RefObject) ((RefObject) container).refValue(sfTopComponent);
		rightComp = (RefObject) ((RefObject) container).refValue(sfRightComponent);
		bottomComp = (RefObject) ((RefObject) container).refValue(sfBottomComponent);

		/*
		 * Since Sun allows JSplitPane to contain children that have been added with constraints (i.e. add(aComponent, "left")),
		 * we must look for any constraint components and include them as part of the possibly settings.
		 */
		List constraintChildren = (List) ((RefObject) container).refValue(sf_containerComponents);
		Iterator itr = constraintChildren.iterator();
		while (itr.hasNext()) {
			RefObject constraintComponent = (RefObject) itr.next();
			RefObject component = (RefObject) constraintComponent.refValue(sf_constraintComponent);
			// See whether the component is in severe error.  If so then don't include it here
			if (BeanProxyUtilities.getBeanProxyHost((IJavaInstance)component).getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
				IJavaObjectInstance constraintString = (IJavaObjectInstance) constraintComponent.refValue(sf_constraintConstraint);
				// We know the constraints value should be a bean so we can use its toString to get the string value
				String constraint = BeanProxyUtilities.getBeanProxy(constraintString).toBeanString();
				if (constraint != null) {
					if (constraint.equals("left"))  //$NON-NLS-1$
						leftComp = component;
					else if (constraint.equals("top"))  //$NON-NLS-1$
						topComp = component;
					else if (constraint.equals("right"))  //$NON-NLS-1$
						rightComp = component;
					else if (constraint.equals("bottom"))  //$NON-NLS-1$
						bottomComp = component;
				}
			}
		}
	}
	public Command getAddCommand(List children, Object positionBeforeChild) {
		// The code logic can only handle setting one component at a time.
		if (children == null || children.isEmpty() || children.size() > 1)
			return UnexecutableCommand.INSTANCE;
		Object child = children.get(0);
		// Determine the structural feature to use for this child based on what is already set in the JSplitPane 
		// and the positionBeforeChild component (it could be null). Also add commands to a command builder
		// for any repositioning of the current component. 
		// If null is returned, no SF was return, just return the CommandBuilder
		CommandBuilder cBldr = new CommandBuilder(""); //$NON-NLS-1$
		RefStructuralFeature containmentSF = getStructuralFeature(child, positionBeforeChild, cBldr);
		if (containmentSF != null)
			cBldr.append(primAddCommand(child, positionBeforeChild, containmentSF));
		return cBldr.getCommand();
	}
	/**
	 * Determine whether the component should be added as the left or right component
	 */
	public Command getCreateCommand(Object child, Object positionBeforeChild) {
		// Determine the structural feature to use for this child based on what is already set in the JSplitPane 
		// and the positionBeforeChild component (it could be null). Also add commands to a command builder
		// for any repositioning of the current component. 
		// If null is returned, no SF was return, just return the CommandBuilder
		CommandBuilder cBldr = new CommandBuilder(""); //$NON-NLS-1$
		RefStructuralFeature containmentSF = getStructuralFeature(child, positionBeforeChild, cBldr);
		if (containmentSF != null)
			cBldr.append(primCreateCommand(child, positionBeforeChild, containmentSF));
		return cBldr.getCommand();
	}
	/**
	 * Return a structural feature to use to add or create a child and based on the positionBeforeChild, 
	 * and create any other commands that are needed to reposition the current left or right component.
	 *
	 * Determine the structural feature to use for this child based on what is already set in the JSplitPane 
	 * and the positionBeforeChild component (it could be null). Also add commands to a command builder
	 * for any repositioning of the current component. For example, if the right component is set and the positionBeforeChild
	 * is null, this would mean the new child component wants to be at the end (set in the right component position).
	 * Therefore we have to issue a cancel and set to left component for the current child , and return the SF for
	 * setting the new child to the right component which is done by the caller.
	 */
	protected RefStructuralFeature getStructuralFeature(Object child, Object positionBeforeChild, CommandBuilder cmdbldr) {
		if (!isValidChild(child) || !isParentAcceptable(child)) {
			cmdbldr.append(UnexecutableCommand.INSTANCE);
			return null;
		}
		determineSplitpaneOccupants();

		// If both components are occupied, we can't add any more.
		if ((leftComp != null || topComp != null) && (rightComp != null || bottomComp != null)) {
			cmdbldr.append(UnexecutableCommand.INSTANCE);
			return null;
		}
		// Get the orientation from the live JSplitPane and set the SF's based on this setting.
		IBeanProxyHost proxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) container);
		IIntegerBeanProxy intProxy = (IIntegerBeanProxy) BeanProxyUtilities.invoke(proxyHost.getBeanProxy(), "getOrientation"); //$NON-NLS-1$
		int orientation = intProxy.intValue();
		RefStructuralFeature leftTopSF, rightBottomSF;
		if (orientation == HORIZONTAL_SPLIT) {
			leftTopSF = sfLeftComponent;
			rightBottomSF = sfRightComponent;
		} else {
			leftTopSF = sfTopComponent;
			rightBottomSF = sfBottomComponent;
		}
		// If nothing is occupied, set the left/top component.
		if (leftComp == null && topComp == null && rightComp == null && bottomComp == null) {
			return leftTopSF;
		}
		// Handle the cases where the positionBeforeChild is null
		RefStructuralFeature containmentSF;
		if (positionBeforeChild == null) {
			// If the left/top is already occupied and the positionBeforeChild is null, set the right/bottom component.
			if (leftComp != null || topComp != null) {
				return rightBottomSF;
			}
			// If the right/bottom is occupied, this R/B component needs to be moved to the left/top
			// and this new child needs to be set as the R/B. This requires a cancel & add for the existing 
			// R/B component, and an add for the new child.
			if (leftComp == null && topComp == null && (rightComp != null || bottomComp != null)) {
				RefObject component = null;
				if (rightComp != null) {
					containmentSF = sfRightComponent;
					component = rightComp;
				} else {
					containmentSF = sfBottomComponent;
					component = bottomComp;
				}
				// Need to handle the case where the Component has a constraint which means it is 
				// contained in a ConstraintComponent.
				if (component.refContainerSF() == sf_constraintComponent) 
					cmdbldr.append(super.getDeleteDependentCommand(component.refContainer(), sf_containerComponents));
				else
					cmdbldr.cancelAttributeSetting((RefObject) container, containmentSF, component);
				cmdbldr.applyAttributeSetting((RefObject) container, leftTopSF, component);
				return rightBottomSF;
			}
		} else {
			// Handle the cases where positionBeforeChild is not null

			// If the right/bottom is occupied, just set the left component
			if (leftComp == null && topComp == null && (rightComp != null || bottomComp != null)) {
				return leftTopSF;
			}
			// If the left/top is occupied, this L/T component needs to be moved to the bottom/right
			// and this new child needs to be set as the L/T. This requires a cancel & add for the existing 
			// L/T component, and an add for the new child.
			if (rightComp == null && bottomComp == null && (leftComp != null || topComp != null)) {
				RefObject component = null;
				if (leftComp != null) {
					containmentSF = sfLeftComponent;
					component = leftComp;
				} else {
					containmentSF = sfTopComponent;
					component = topComp;
				}
				// Need to handle the case where the Component has a constraint which means it is 
				// contained in a ConstraintComponent.
				if (component.refContainerSF() == sf_constraintComponent) 
					cmdbldr.append(super.getDeleteDependentCommand(component.refContainer(), sf_containerComponents));
				else
					cmdbldr.cancelAttributeSetting((RefObject) container, containmentSF, component);
				cmdbldr.applyAttributeSetting((RefObject) container, rightBottomSF, component);
				return leftTopSF;
			}
		}
		return null;
	}
	/**
	 * Delete the dependent. The child could be the component or the constraintComponent.
	 */
	public Command getDeleteDependentCommand(Object child) {
		RefStructuralFeature sf = ((RefObject) child).refContainerSF();
		// Need to handle the possibility that the child was added with a constraint (i.e. add(thisComponent, "left"))
		if (sf == sf_constraintComponent)
			return super.getDeleteDependentCommand(((RefObject) child).refContainer(), sf_containerComponents);

		return super.getDeleteDependentCommand(child, sf);
	}
	/**
	 * Get the move children command for the list. The children
	 * are the components, not the constraintComponents.
	 */
	public Command getMoveChildrenCommand(List children, Object positionBeforeChild) {
		if (children.size() > 1)
			return UnexecutableCommand.INSTANCE;
		Object child = children.get(0);
		// If we trying to move a component before itself do nothing.
		if (positionBeforeChild != null && positionBeforeChild == child)
			return UnexecutableCommand.INSTANCE;
		determineSplitpaneOccupants();
		// Get the orientation from the live JSplitPane and set the SF's based on this setting.
		IBeanProxyHost proxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) container);
		IIntegerBeanProxy intProxy = (IIntegerBeanProxy) BeanProxyUtilities.invoke(proxyHost.getBeanProxy(), "getOrientation");	//$NON-NLS-1$
		int orientation = intProxy.intValue();
		RefStructuralFeature leftTopSF, rightBottomSF;
		if (orientation == HORIZONTAL_SPLIT) {
			leftTopSF = sfLeftComponent;
			rightBottomSF = sfRightComponent;
		} else {
			leftTopSF = sfTopComponent;
			rightBottomSF = sfBottomComponent;
		}
		// If we trying to move the right/bottom component to the last position (which is already the rightbottom component), 
		// do nothing.
		if (positionBeforeChild == null && (rightComp == child || bottomComp == child))
			return UnexecutableCommand.INSTANCE;
		// If we trying to move the right/bottom component to the first position (which is the left/top component), 
		// or move the left/top component to the last position ((which is the rightbottom component), switch the components.
		if ((leftComp != null || topComp != null) && (rightComp != null || bottomComp != null)) {
			CommandBuilder cmdbldr = new CommandBuilder("Move children"); //$NON-NLS-1$
			RefObject leftTopComp, rightBottomComp = null;
			RefStructuralFeature containmentSF;
			
			// First unset the structural feature for both components
			if (leftComp != null) {
				containmentSF = sfLeftComponent;
				leftTopComp = leftComp;
			} else {
				containmentSF = sfTopComponent;
				leftTopComp = topComp;
			}
			if (leftTopComp.refContainerSF() == sf_constraintComponent) 
				cmdbldr.append(super.getDeleteDependentCommand(leftTopComp.refContainer(), sf_containerComponents));
			else
				cmdbldr.cancelAttributeSetting((RefObject) container, containmentSF, leftTopComp);
				
			if (rightComp != null) {
				containmentSF = sfRightComponent;
				rightBottomComp = rightComp;
			} else {
				containmentSF = sfBottomComponent;
				rightBottomComp = bottomComp;
			}
			if (rightBottomComp.refContainerSF() == sf_constraintComponent) 
				cmdbldr.append(super.getDeleteDependentCommand(rightBottomComp.refContainer(), sf_containerComponents));
			else
				cmdbldr.cancelAttributeSetting((RefObject) container, containmentSF, rightBottomComp);
			cmdbldr.cancelAttributeSetting((RefObject) container, containmentSF, rightBottomComp);
			
			// Then set the components in the reverse positions
			cmdbldr.applyAttributeSetting((RefObject) container, leftTopSF, rightBottomComp);
			cmdbldr.applyAttributeSetting((RefObject) container, rightBottomSF, leftTopComp);
			return cmdbldr.getCommand();
		}
		return UnexecutableCommand.INSTANCE;
	}
	/**
	 * Get the orphan command for the list.
	 * The children could be a mix which may have constraints (i.e. add(aComponent, "left")
	 * or may have been added using JSplitPane setters (i.e. setLeftComponent(Component)).
	 */
	public Command getOrphanChildrenCommand(List children) {
		List constraintComponents = new ArrayList(children.size());
		List components = new ArrayList(children.size());
		Iterator itr = children.iterator();
		while (itr.hasNext()) {
			RefObject child = (RefObject) itr.next();
			// Check if the child has a constraint which means it's contained in a ConstraintComponent
			if (child.refContainerSF() == sf_constraintComponent)
				constraintComponents.add(child.refContainer());
			else
				components.add(child);
		}

		CommandBuilder cb = new CommandBuilder(null);
		itr = components.iterator();
		while (itr.hasNext()) {
			RefObject component = (RefObject) itr.next();
			cb.cancelAttributeSetting((RefObject) container, component.refContainerSF(), component);
		}
		// We need to unset the components from the constraint components after
		// orphaning the constraints so that they are free of any
		// containment when they are added to their new parent. If we
		// didn't unset the components, then upon undo the component
		// would not be in the constraint when it is added back in.
		if (!constraintComponents.isEmpty()) {
			cb.append(super.getOrphanChildrenCommand(constraintComponents, sf_containerComponents));
			cb.cancelGroupAttributeSetting(constraintComponents, sf_constraintComponent);
		}
		return cb.getCommand();
	}
	protected boolean isValidChild(Object child) {
		return (
			classComponent.isInstance(child)
				&& ((!((RefObject) getContainer()).refIsSet(sfLeftComponent) && !((RefObject) getContainer()).refIsSet(sfTopComponent))
					|| (!((RefObject) getContainer()).refIsSet(sfRightComponent)
						&& !((RefObject) getContainer()).refIsSet(sfBottomComponent))));
	}
	/**
	 * Need this for the JSplitPaneLayoutEditPolicy to handle the special case in which
	 * the right/bottom position is occupied and the only component.
	 */
	public boolean isRightComponentOccupiedAndOnlyComponent() {
		determineSplitpaneOccupants();
		if (topComp == null && leftComp == null && (bottomComp != null || rightComp != null))
			return true;
		return false;
	}
	protected Command primAddCommand(Object child, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		CommandBuilder cBld = new CommandBuilder(""); //$NON-NLS-1$
		cBld.applyAttributeSetting((RefObject) container, containmentSF, child, positionBeforeChild);
		return cBld.getCommand();
	}
	protected Command primCreateCommand(Object child, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		CommandBuilder cBld = new CommandBuilder(""); //$NON-NLS-1$
		cBld.applyAttributeSetting((RefObject) container, containmentSF, child, positionBeforeChild);
		List annotations = AnnotationPolicy.getAllAnnotations(new ArrayList(), child, domain.getAnnotationLinkagePolicy());
		return AnnotationPolicy.getCreateRequestCommand(annotations, cBld.getCommand(), domain);
	}
}