package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.cde.AnnotationPolicy;
import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.cde.commands.CommandBuilder;
import com.ibm.etools.cde.emf.EMFContainerPolicy;
import com.ibm.etools.common.command.Command;
import com.ibm.etools.emf.ecore.*;
import com.ibm.etools.emf.ecore.EClass;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.JBCFEditDomainHelper;

/**
 * Container Edit Policy for Swing JTabbedPanes.
 */
public class JTabbedPaneContainerPolicy extends EMFContainerPolicy {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EClass classJComponent;
	protected EClass classJTabComponent;
	protected EStructuralFeature sfComponent;
	protected EFactory visualsFact;

	public JTabbedPaneContainerPolicy(EditDomain domain) {
		super(JavaInstantiation.getSFeature(JBCFEditDomainHelper.getResourceSet(domain), JBCFConstants.SF_JTABBEDPANE_TABS), domain);

		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain);
		sfComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_COMPONENT);
		classJTabComponent = (EClass) rset.getObject(JBCFConstants.CLASS_JTABBEDPANE_JTABCOMPONENT);
		classJComponent = (EClass) sfComponent.getETypeClassifier();
		visualsFact = JBCFConstants.getFactory(classJTabComponent);
	}

	/*
	 * The child in this case is the JComponent and not the JTabComponent. The isValidChild
	 * in this case is called by super classes and they are passed the JComponent at that
	 * point in time. We will later wrapper it into a JTabComponent after it has gone through
	 * this test.
	 */
	protected boolean isValidChild(Object child) {
		return classJComponent.isInstance(child);
	}

	/**
	 * Delete the dependent. The child is the component, not the JTabComponent.
	 */
	public Command getDeleteDependentCommand(Object child) {
		return super.getDeleteDependentCommand(((RefObject) child).refContainer());
	}
	/**
	 * Get the move children command for the list. The children
	 * are the components, not the JTabComponents.
	 */	
	public Command getMoveChildrenCommand(List children, Object positionBeforeChild) {
		// We need to convert to the JTabComponents, not the components.
		List jtabComponents = new ArrayList(children.size());
		Iterator itr = children.iterator();
		while(itr.hasNext()) {
			jtabComponents.add(((RefObject) itr.next()).refContainer());
		}
		return super.getMoveChildrenCommand(jtabComponents, positionBeforeChild != null ? ((RefObject) positionBeforeChild).refContainer() : null);
	}
	/**
	 * Get the orphan command for the list. The children
	 * are the components, not the JTabComponents.
	 */
	public Command getOrphanChildrenCommand(List children) {
		// We need to unset the components from the JTabComponents after
		// orphaning the JTabComponents so that they are free of any
		// containment when they are added to their new parent. If we
		// didn't unset the components, then upon undo the component
		// would not be in the JTabComponent when it is added back in.

		List jtabComponents = new ArrayList(children.size());
		Iterator itr = children.iterator();
		while(itr.hasNext()) {
			jtabComponents.add(((RefObject) itr.next()).refContainer());
		}
		
		CommandBuilder cb = new CommandBuilder(null);
		cb.append(super.getOrphanChildrenCommand(jtabComponents));
		cb.cancelGroupAttributeSetting(jtabComponents, sfComponent);	// Cancel out all of the component settings.		
		return cb.getCommand();
	}
	/**
	 * Add children to the JTabbedPane. Each component will part of a JTabComponent.
	 */
	protected Command primAddCommand(List children, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		ArrayList jtabComponents = new ArrayList(children.size());
		CommandBuilder cb = new CommandBuilder(""); //$NON-NLS-1$
		Iterator itr = children.iterator();
		while (itr.hasNext()) {
			Object child = itr.next();
			RefObject jtabComponent = (RefObject) visualsFact.create(classJTabComponent);
			// Add the child to the component.
			cb.applyAttributeSetting((RefObject) jtabComponent, sfComponent, child);
			jtabComponents.add(jtabComponent);
		}
		cb.append(super.primAddCommand(jtabComponents, positionBeforeChild != null ? ((RefObject) positionBeforeChild).refContainer() : null, containmentSF));
		return cb.getCommand();
	}
	/**
	 * Create a new child which in this case is part of a JTabComponent.
	 */
	protected Command primCreateCommand(Object child, Object positionBeforeChild, RefStructuralFeature containmentSF) {
		RefObject jtabComponent = (RefObject) visualsFact.create(classJTabComponent);
		// Add the child to the component.
		CommandBuilder cb = new CommandBuilder(""); //$NON-NLS-1$
		cb.applyAttributeSetting((RefObject) jtabComponent, sfComponent, child);
		// The annotation on the child won't be picked up in the primCreateCommand because the child is
		// not the child of the tabpane, and it is not yet a child of the jTabComponent. The jTabComponent
		// is the child of the container, and since the child is not yet in the jTabComponent, its annotations
		// won't be found. So we have to do it explicitly.
		List annotations = AnnotationPolicy.getAllAnnotations(new ArrayList(), child, domain.getAnnotationLinkagePolicy());
		cb.append(AnnotationPolicy.getDefaultAddAnnotationsCommand(annotations, domain));		
		cb.append(super.primCreateCommand(jtabComponent, positionBeforeChild != null ? ((RefObject) positionBeforeChild).refContainer() : null, containmentSF));
		return cb.getCommand();
	}
}