package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import com.ibm.etools.cde.EditDomain;
import com.ibm.etools.draw2d.IFigure;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.Notifier;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.gef.*;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.java.instantiation.base.JavaInstantiation;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.IBeanProxyHost;
import com.ibm.etools.proxy.IIntegerBeanProxy;

/**
 * Graphical edit part for handling JTabbedPane in the Graph viewer
 * 
 * Since we can't really select the individual tabs on the JTabbedPane,
 * we depend on the tree editpart in the outline viewer for selecting the
 * pages or components on those pages. 
 * 
 * When the tree editpart is selected, this editpart is also selected
 * and the JTabbedPaneProxyAdapter is notified so the page can be brought forward. 
 * When this occurs, the previously viewed page figures are hidden and the 
 * currently selected page is made visible. If we didn't hide the editparts 
 * from the other pages, the figure rectangle outlines will "bleed" through 
 * and show on the top page.
 */
public class JTabbedPaneGraphicalEditPart extends ContainerGraphicalEditPart {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	private EditPartListener pageListener;
	protected IJavaObjectInstance fSelectedPage;
	protected boolean fSelectingPage = false;
	protected JTabbedPaneProxyAdapter tabbedpaneAdapter;
	
	protected EStructuralFeature
		sfTabs,
		sfComponent;

	public JTabbedPaneGraphicalEditPart(Object model) {
		super(model);
	}
	public void activate() {
		setListener(createPageListener());
		((RefObject) getModel()).addAdapter(getModelAdapter());
		super.activate();
		List children = getChildren();
		for (int i = 0; i < children.size(); i++) {
			EditPart page = (EditPart) children.get(i);
			addPageListenerToChildren(page);
			if (i == 0) {
				setPageVisible(page, true);
				selectPage((EditPart)getChildren().get(0));
			} else {
				setPageVisible(page, false);
			}
		}
	}
	protected void addPageListenerToChildren(EditPart ep) {
		ep.addEditPartListener(pageListener);
		Iterator childen = ep.getChildren().iterator();
		while (childen.hasNext())
			 addPageListenerToChildren((EditPart) childen.next());
	}
	public void deactivate() {
		Iterator children = getChildren().iterator();
		while (children.hasNext())
			 removePageListenerFromChildren((EditPart) children.next());
		((RefObject) getModel()).removeAdapter(getModelAdapter());
		setListener(null);
		super.deactivate();
	}
	protected void removePageListenerFromChildren(EditPart ep) {
		ep.removeEditPartListener(pageListener);
		Iterator childen = ep.getChildren().iterator();
		while (childen.hasNext())
			 removePageListenerFromChildren((EditPart) childen.next());
	}
	protected void createLayoutEditPolicy() {
		installEditPolicy(
			EditPolicy.LAYOUT_ROLE,
			new JTabbedPaneLayoutEditPolicy(EditDomain.getEditDomain(this)));
	}
	/*
	 * Create an EditPartListener for itself and its pages so it knows 
	 * when a page has been added, removed, or selected
	 */
	protected EditPartListener createPageListener() {
		return new EditPartListener.Stub() {
			public void childAdded(EditPart editpart, int index) {
				addPageListenerToChildren(editpart);
			}
			public void removingChild(EditPart editpart, int index) {
				removePageListenerFromChildren(editpart);
			}
			public void selectedStateChanged(EditPart editpart) {
				if (editpart == null || editpart == JTabbedPaneGraphicalEditPart.this)
					return;
				// Find the page where this editpart resides and bring the page to the
				// front if isn't already.
				if ((editpart != null)
					&& (editpart.getSelected() == EditPart.SELECTED
						|| editpart.getSelected() == EditPart.SELECTED_PRIMARY)) {
					EditPart page = getPageOfSelectedEditpart(editpart);
					if (page != null && page.getModel() != fSelectedPage){
						// First hide the previously selected page
						setPageVisible(getEditPartFromModel(fSelectedPage), false);
						// Then show the newly selected page
						setPageVisible(page, true);
						selectPage(page);
					}
				}
			}
		};
	}
	/*
	 * Search through the JTabbedPane's pages (children) to find the page
	 * that matches the page model that is selected.
	 */
	protected EditPart getEditPartFromModel(IJavaObjectInstance pageModel) {
		Iterator children = getChildren().iterator();
		while (children.hasNext()) {
			EditPart page = (EditPart) children.next();
			if ((IJavaObjectInstance) page.getModel() == pageModel)
				return page;
		}
		return null;
	}
	/*
	 * Return the proxy adapter associated with this JTabbedPane.
	 */
	protected JTabbedPaneProxyAdapter getJTabbedPaneProxyAdapter() {
		if (tabbedpaneAdapter == null) {
			IBeanProxyHost tabbedpaneProxyHost =
				BeanProxyUtilities.getBeanProxyHost((IJavaObjectInstance) getModel());
			tabbedpaneAdapter = (JTabbedPaneProxyAdapter) tabbedpaneProxyHost;
		}
		return tabbedpaneAdapter;
	}
	/*	
	 * Model children is the tabs feature.
	 * However, this returns the JTabComponents, but we want to return instead
	 * the components themselves. They are the "model" that gets sent to the createChild and
	 * component edit part.
	 */
	protected List getModelChildren() {
		List jtabComponents =
			(List) ((RefObject) getModel()).refValue(sfTabs);
		ArrayList children = new ArrayList(jtabComponents.size());
		Iterator itr = jtabComponents.iterator();
		while (itr.hasNext()) {
			RefObject component = (RefObject) itr.next();
			// Get the component out of the JTabComponent
			children.add(component.refValue(sfComponent));
		}
		return children;
	}

	/*
	 * If the parent of this editpart is the JTabbedPane, we're on the page.
	 * If not recursely call up through the parent chain until we find the 
	 * editpart (page) that the original editpart was found in.
	 */
	protected EditPart getPageOfSelectedEditpart(EditPart ep) {
		if (ep == null || ep.getParent() == this)
			return ep;
		return getPageOfSelectedEditpart(ep.getParent());
	}
	public void notifyChanged(Notifier notifier, int eventType, RefObject sf, Object oldValue, Object newValue, int pos) {

		if (sf == sfTabs) {
			refreshChildren();
			// With changes to the editpart structure (deletes and moves), its hard to determine
			// which page should be hidden and which should be shown so we'll just query the live
			// JTabbedPane and find out which page is selected...then show it's corresponding
			// editpart. Then hide the rest if they are visible.
			int selectedIndex = getJTabbedPaneProxyAdapter().getSelectedIndex();
			if (selectedIndex != -1) {
				if (selectedIndex < getChildren().size()
					&& ((IJavaObjectInstance) ((EditPart) getChildren().get(selectedIndex)).getModel()) != fSelectedPage) {
					List childen = getChildren();
					for (int i = 0; i < children.size(); i++) {
						GraphicalEditPart ep = (GraphicalEditPart) childen.get(i);
						if (i == selectedIndex) {
							if (!ep.getFigure().isVisible()) {
								setPageVisible(ep, true);
								fSelectedPage = (IJavaObjectInstance) ep.getModel();
							}
						} else if (ep.getFigure().isVisible())
							setPageVisible(ep, false);
					}
				}
			}
		}
	}
	/*
	 * This page has been selected or deselected. 
	 * Make it and all it's children visible or invisible.
	 */
	protected void setPageVisible(EditPart page, boolean bool) {
		if (page != null) {
			((GraphicalEditPart) page).getFigure().setVisible(bool);
			IFigure fig = ((GraphicalEditPart) page).getFigure();
			Iterator children = page.getChildren().iterator();
			while (children.hasNext())
				 ((GraphicalEditPart) children.next()).getFigure().setVisible(bool);
			fig.revalidate();
		}
	}
	/*
	 * @see EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		ResourceSet rset = ((RefObject) model).getResourceSet();
		sfTabs = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABBEDPANE_TABS);
		sfComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_COMPONENT);				
	}	
	protected void setListener(EditPartListener listener) {
		if (this.pageListener != null)
			removeEditPartListener(this.pageListener);
		this.pageListener = listener;
		if (this.pageListener != null)
			addEditPartListener(this.pageListener);
	}
	/*
	 * The selected page of the JTabbedPane has changed. Bring this page to the front.
	 */
	protected void selectPage(EditPart page) {
		fSelectedPage = (IJavaObjectInstance)page.getModel(); // save for later checks... see createPageListener()
		fSelectingPage = true; // set this so we can ignore the proxy adapter when it notifies us later in pageSelected()
		getJTabbedPaneProxyAdapter().setSelectedComponent((IJavaObjectInstance)page.getModel());
		fSelectingPage = false;
	}
}