package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import java.util.List;

import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.notify.*;
import com.ibm.etools.emf.notify.impl.AdapterImpl;
import com.ibm.etools.emf.ref.*;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.*;

/**
 * Create the bean proxy. 
 * We override because we use addtab(String title, Icon icon, Component component, String tip, int index) or 
 * insertTab(String title, Icon icon, Component component, String tip, int index)
 * to add components as tabs (pages).
 * 
 * This also keeps a List of those who are interested in the page selection
 * of the JTabbedPane. The listener must implement IJTabbedPanePageListener and 
 * notifies the listeners when a page is selected.
 */
public class JTabbedPaneProxyAdapter extends ComponentProxyAdapter {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature
		sfTabs,
		sfComponent,
		sfTitle,
		sfIcon,
		sfTooltip;

	public JTabbedPaneProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
		
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());
		sfTabs = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABBEDPANE_TABS);
		sfComponent = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_COMPONENT);		
		sfTitle = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_TITLE);
		sfIcon = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_ICON);
		sfTooltip = JavaInstantiation.getSFeature(rset, JBCFConstants.SF_JTABCOMPONENT_TOOLTIP);		
	}
	protected void applied(RefStructuralFeature as, Object newValue, int position) {

		if (as == sfTabs) {
			addComponent((RefObject) newValue, position);
		} else {
			super.applied(as, newValue, position);
		}
	}
	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {

		if (sf == sfTabs) {
			removeComponent((RefObject) oldValue);
		} else {
			super.canceled(sf, oldValue, position);
		}

	}
	
	/*
	 * Return the true component added for the tab at the requested position.
	 * It may not be the one associated with the tab because it may not of been
	 * instantiated, in which case we would of created a dummy one to use.
	 * Return null if not found.
	 */
	protected IBeanProxy getBeanProxyAt(int position) {
		List tabs = (List) ((RefObject) getTarget()).refValue(sfTabs);
		return position < tabs.size() ? getBeanProxyFor((RefObject) tabs.get(position)) : null;
	}
	
	/*
	 * Return the bean proxy component that was used for the specified component.
	 * It may be different if the specified component could not be instantiated.
	 */
	protected IBeanProxy getBeanProxyFor(RefObject tab) {
		JTabComponentAdapter adapter = (JTabComponentAdapter) tab.getAdapter(JTabComponentAdapter.class);
		if (adapter != null)
			return adapter.getComponentProxy();
		
		// The adapter doesn't have a component, so some problem, so we go to the end instead.
		return null;		
	}
	
	/**
	 * We don't really do an add component. If an index is specified, do an insertTab(...),
	 * otherwise do an addTab(...). 
	 */
	protected void addComponent(RefObject jtabComponent, int position) {	
		// The component to actually add is within the JTabComponent.
		IBeanProxyHost componentProxyHost =
			BeanProxyUtilities.getBeanProxyHost(
				(IJavaInstance) jtabComponent.refValue(sfComponent));
		// Ensure the bean is instantiated
		IComponentProxyHost componentAdapter = (IComponentProxyHost) componentProxyHost;
		componentProxyHost.instantiateBeanProxy();
		if (getErrorStatus() == IBeanProxyHost.ERROR_SEVERE)
			return;	// We arent' instantiated, can't add a tab to us, but we did instantiate child.		
	
		IJavaInstance titleAttributeValue = null;
		IJavaInstance iconAttributeValue = null;
		// Get the values of each of the attributes of the JTabComponent.
		if (jtabComponent.refIsSet(sfTitle))
			titleAttributeValue =
				(IJavaInstance) jtabComponent.refValue(sfTitle);
		if (jtabComponent.refIsSet(sfIcon))
			iconAttributeValue =
				(IJavaInstance) jtabComponent.refValue(sfIcon);
				
		IBeanProxy componentBeanProxy = null;
		if (componentProxyHost.getErrorStatus() != IBeanProxyHost.ERROR_SEVERE)
			componentBeanProxy = componentProxyHost.getBeanProxy();
		else {
			// We need to new up a dummy. Just create a JPanel.
			try {
				componentBeanProxy = getBeanProxyDomain().getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy("javax.swing.JPanel").newInstance(); //$NON-NLS-1$
			} catch (ThrowableProxy e) {
				// This really shouldn't of occurred.
				JBCFPlugin.log(e, JBCFMsgLogger.LOG_WARNING);				
			}
		}
		// If the position is not -1, use insertTab to add the component, otherwise use addTab
		IBeanProxy positionBeanProxy = null;
		if (position != -1) {
			positionBeanProxy = getBeanProxyAt(position+1);	// We've already at the specified position in the EMF list, so we want the next one.
		}
		
		if (positionBeanProxy != null) {
			BeanAwtUtilities.invoke_insert_Tab_before(
					getBeanProxy(),
					BeanProxyUtilities.getBeanProxy(titleAttributeValue),
					BeanProxyUtilities.getBeanProxy(iconAttributeValue),
					componentBeanProxy,
					null,	// Tooltip doesn't matter for us here. It's never visible, so we don't bother.
					positionBeanProxy);
		} else {
			BeanAwtUtilities.invoke_add_Tab(
				getBeanProxy(),
				BeanProxyUtilities.getBeanProxy(titleAttributeValue),
				BeanProxyUtilities.getBeanProxy(iconAttributeValue),
				componentBeanProxy,
				null);
		}

		// Add a listener to the JTabComponent for when the special features changes.
		// To be on the safe side, remove any old ones, they shouldn't be there, but...
		JTabComponentAdapter a = (JTabComponentAdapter) jtabComponent.getExistingAdapter(JTabComponentAdapter.class);
		if (a != null)
			jtabComponent.removeAdapter(a);
		a = new JTabComponentAdapter();
		a.setComponentProxy(componentBeanProxy);
		jtabComponent.addAdapter(a);
		a.setTarget(jtabComponent);

		// Now that we've added it, set the parent component.
		componentAdapter.setParentComponentProxyHost((IComponentProxyHost) this);
		// This is required because AWT will not invalidate and relayout the container	
		revalidateBeanProxy();
	}

	/**
	 * Returns the currently selected index for this tabbed pane as an int. It will
	 * be only for ones we've defined. Any others (such as from a superclass) will not
	 * be selectable and -1 will be returned in that case.
	 */
	public int getSelectedIndex() {
		if (getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
			IBeanProxy componentProxy = BeanAwtUtilities.invoke_tab_getSelectedComponent(getBeanProxy());
			if (componentProxy != null) {
				List tabs = (List) ((RefObject) getTarget()).refValue(sfTabs);
				for (int i=0; i <tabs.size(); i++)
					if (componentProxy == getBeanProxyFor((RefObject) tabs.get(i)))
						return i;
			}
		}
		
		return -1;
	}
	
	protected boolean isJTabComponentFeature(Object feature) {
		if (feature == sfIcon
			|| feature == sfTitle
			|| feature == sfTooltip)
			return true;
		return false;
	}
	/**
	 * Sets the icon for the page at the tab with the given component
	 */
	protected void setIconAt(IBeanProxy componentProxy, IJavaObjectInstance icon) {
		BeanAwtUtilities.invoke_tab_setIconAt(getBeanProxy(), componentProxy, BeanProxyUtilities.getBeanProxy(icon));
	}
	/**
     * Sets the selected component for this tabbedpane. This is public and is used by 
     * the edit parts to bring a selected page to the front so it can be viewed and edited.
     * PageModel is the actual component, not the tab.
	 */
	public void setSelectedComponent(IJavaObjectInstance pageModel) {
		if (pageModel == null || getErrorStatus() == IBeanProxyHost.ERROR_SEVERE)
			return;
		JTabComponentAdapter a = (JTabComponentAdapter) ((RefObject) pageModel.refContainer()).getAdapter(JTabComponentAdapter.class);
		BeanAwtUtilities.invoke_tab_setSelectedComponent(getBeanProxy(), a.getComponentProxy());
		revalidateBeanProxy();
	}
	/**
	 * Sets the title text for the page with the given component
	 */
	protected void setTitleAt(IBeanProxy componentProxy, IJavaObjectInstance title) {
		IBeanProxy titleProxy = BeanProxyUtilities.getBeanProxy(title);
		if (titleProxy == null) {
			// Null for title is really invalid. We are using it to mean cancel.
			titleProxy = componentProxy.getProxyFactoryRegistry().getBeanProxyFactory().createBeanProxyWith(""); //$NON-NLS-1$
		}
		BeanAwtUtilities.invoke_tab_setTitleAt(getBeanProxy(), componentProxy, titleProxy);
	}
	
	/**
	 * We need to remove the adapter and the actual component which is part of the JTabComponent
	 */
	protected void removeComponent(RefObject aJTabComponent) {
		if (getErrorStatus() == IBeanProxyHost.ERROR_SEVERE)
			return;	// Never added anything
		clearError(sfTabs, aJTabComponent);			
		IBeanProxy componentBeanProxy = null;	// Component to remove.
		IBeanProxyHost componentBeanProxyHost = BeanProxyUtilities.getBeanProxyHost(
					(IJavaInstance) aJTabComponent.refValue(sfComponent));
		// Remove the old constraint adapter, if one.
		JTabComponentAdapter a = (JTabComponentAdapter) aJTabComponent.getExistingAdapter(JTabComponentAdapter.class);
		if (a != null) {
			componentBeanProxy = a.getComponentProxy();
			aJTabComponent.removeAdapter(a);
			BeanAwtUtilities.invoke_remove_Component(getBeanProxy(), componentBeanProxy);
			if (componentBeanProxyHost.getBeanProxy() != componentBeanProxy)
				componentBeanProxy.getProxyFactoryRegistry().releaseProxy(componentBeanProxy);
		}
		
		releaseTab((InternalProxy) aJTabComponent);
		
		// This is required because AWT will not invalidate and relayout the container
		revalidateBeanProxy();
	}
	

	public void releaseBeanProxy() {	
		// Need to release all of the tabs. This isn't being done in BeanProxyAdapter because
		// the tabs aren't bean proxies directly.

		// NOTE: Using refBasicValue so that we don't go creating a list and a settings adapter
		// if none were set.
		List tabs = (List) ((Internals) target).refBasicValue(sfTabs);
		if (tabs != null) {
			Iterator itr = tabs.iterator();
			while (itr.hasNext()) {
				InternalProxy ip = (InternalProxy) itr.next();
				releaseTab(ip);
			}
		}
		
		super.releaseBeanProxy();
	}

	protected void releaseTab(InternalProxy ip) {
		// Get all of the basic contains and release those that are beans. 
		Iterator settings = ip.refBasicContains().iterator(); // Get only the attrs and composite refs.
		while (settings.hasNext()) {
			Object v = settings.next();
			// Other settings like initializationString exist, only interested in java instances.
			if (v instanceof IJavaInstance) {
				// Also, if it is a proxy (i.e. never resolved) don't release it since we never did anything with it.
				if (!(v instanceof InternalProxy) || !((InternalProxy) v).refIsProxy()) {
					IBeanProxyHost value = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) v);
					if (value != null)
						value.releaseBeanProxy();
				}
			}
		}
		
		// Also need to remove the tab component adapter and release the componentProxy within it.
		JTabComponentAdapter a = (JTabComponentAdapter) ((Notifier) ip).getAdapter(JTabComponentAdapter.class);
		if (a != null) {
			IBeanProxy c = a.getComponentProxy();
			if (c != null && c.isValid())
				c.getProxyFactoryRegistry().releaseProxy(c);	// Even if it is the same as the real component, re-release doesn't hurt. It checks and doesn't barf.
			((Notifier) ip).removeAdapter(a);
		}
	}
	
	/**
	 * An internal adapter added to the JTabComponent to listen for changes to
	 * JTabComponent's features so that we can reapply if necessary.
	 * It also holds the actual component that was added. This is necessary
	 * so that when we add an empty entry, we have unique component in it.
	 */
	private class JTabComponentAdapter extends AdapterImpl {
		
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
		protected IBeanProxy componentProxy;	// The proxy for the component in the tab.

		public JTabComponentAdapter() {
		}
		
		public IBeanProxy getComponentProxy() {
			return componentProxy;
		}
		
		public void setComponentProxy(IBeanProxy componentProxy) {
			this.componentProxy = componentProxy;
		}

		public boolean isAdapterForType(Object type) {
			return type == JTabComponentAdapter.class;
		}

		public void notifyChanged(Notification msg) {
			RefObject sf = msg.getStructuralFeature();
			if (isJTabComponentFeature(sf)) {
				Object newValue = msg.getNewValue();
				int eventType = msg.getEventType();
				switch (eventType) {
					// ADD,REMOVE don't apply because these are single valued features.
					case Notifier.SET :
					case Notifier.TOUCH :
						if (sf == sfIcon)
							setIconAt(getComponentProxy(), (IJavaObjectInstance) newValue);
						else if (sf == sfTitle)
							setTitleAt(getComponentProxy(), (IJavaObjectInstance) newValue);
						break;
					case Notifier.UNSET :
						if (sf == sfIcon)
							setIconAt(getComponentProxy(), null);
						else if (sf == sfTitle)
							setTitleAt(getComponentProxy(), null);
						break;
				}
				revalidateBeanProxy();
			}
		}

	}
}