package com.ibm.etools.jbcf.visual;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.Iterator;
import java.util.List;

import com.ibm.etools.beaninfo.adapters.Utilities;
import com.ibm.etools.emf.ecore.EStructuralFeature;
import com.ibm.etools.emf.ref.RefObject;
import com.ibm.etools.emf.ref.RefStructuralFeature;
import com.ibm.etools.emf.resource.ResourceSet;
import com.ibm.etools.java.JavaClass;
import com.ibm.etools.java.instantiation.base.IJavaInstance;
import com.ibm.etools.java.instantiation.base.IJavaObjectInstance;
import com.ibm.etools.jbcf.*;
import com.ibm.etools.proxy.*;

/**
 * @author pwalker
 *
 * Proxy adapter for JToolBar. Allows adding Actions to the JToolBar.
 */
public class JToolBarProxyAdapter extends ComponentProxyAdapter {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected EStructuralFeature sfItems;
	protected JavaClass classComponent, classAction;
	protected IMethodProxy fAddActionMethodProxy, fRemoveAllMethodProxy;

	/**
	 * Constructor for JToolBarProxyAdapter.
	 * @param domain
	 */
	public JToolBarProxyAdapter(IBeanProxyDomain domain) {
		super(domain);
		ResourceSet rset = JBCFEditDomainHelper.getResourceSet(domain.getEditDomain());
		classComponent = (JavaClass) Utilities.getJavaClass("java.awt.Component", rset); //$NON-NLS-1$
		classAction = (JavaClass) Utilities.getJavaClass("javax.swing.Action", rset); //$NON-NLS-1$
	}

	protected void applied(RefStructuralFeature as, Object newValue, int position) {

		if (as == getSFItems()) {
			// Instantiate just this one. applyItems will only apply those that are instantiated.
			IJavaInstance item = (IJavaInstance) newValue;
			IBeanProxyHost itemBeanProxyHost = BeanProxyUtilities.getBeanProxyHost(item);
			itemBeanProxyHost.instantiateBeanProxy();
			if (itemBeanProxyHost.getErrorStatus() == ERROR_SEVERE)
				processError(getSFItems(), ((ExceptionError) itemBeanProxyHost.getErrors().get(0)).error, item);
			else {						
				removeItems();
				applyItems();
				revalidateBeanProxy();
			}
		} else {
			super.applied(as, newValue, position);
		}
	}

	protected void canceled(RefStructuralFeature sf, Object oldValue, int position) {

		if (sf == getSFItems()) {
			removeComponent((RefObject) oldValue);
		} else {
			super.canceled(sf, oldValue, position);
		}

	}

	/** 
	 * Iterate over the items in the JToolBar and add them.
	 */
	protected void applyItems() {

		if (getErrorStatus() == IBeanProxyHost.ERROR_SEVERE)
			return;

		RefStructuralFeature sfitems = getSFItems();
		List items = (List) ((RefObject) getTarget()).refValue(sfitems);
		Iterator iter = items.iterator();
		while (iter.hasNext()) {
			IJavaObjectInstance item = (IJavaObjectInstance) iter.next();
			if (!isValidFeature(sfitems, item))
				continue;	// It wasn't valid for some reason.
			// There is a hole here in that we may of changed something and is not valid to apply, but it is difficult if we need
			// to not test for validity. The appliedList(...boolean testValidity) usually handles this, but that only applies
			// to the one setting. So for now once it goes bad it stays bad until reload from scratch.
			IBeanProxyHost itemBeanProxyHost = BeanProxyUtilities.getBeanProxyHost(item);
			IBeanProxy itemBeanProxy = itemBeanProxyHost.getBeanProxy();			
			if (itemBeanProxy != null && itemBeanProxyHost.getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
				try {
					// Invoke a method to add the item (a component or Action) to the JToolBar.
					if (classComponent.isInstance(item)) {
						BeanAwtUtilities.invoke_add_Component(getBeanProxy(), itemBeanProxy);
						// Now that we've added it, set the parent component.
						IComponentProxyHost componentAdapter = (IComponentProxyHost) itemBeanProxyHost;
						componentAdapter.setParentComponentProxyHost((IComponentProxyHost) this);
					} else if (classAction.isInstance(item)) {
						IMethodProxy addActionMethodProxy = getAddActionMethodProxy();
						if (addActionMethodProxy != null) {
							addActionMethodProxy.invoke(getBeanProxy(), itemBeanProxy);
						}
					}
				} catch (ThrowableProxy e) {
					processError(getSFItems(), e, item);
				}
			}
		}
	}

	protected IMethodProxy getAddActionMethodProxy() {
		if (fAddActionMethodProxy == null) {
			fAddActionMethodProxy = getBeanProxy().getTypeProxy().getMethodProxy("add", "javax.swing.Action"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return fAddActionMethodProxy;
	}

	/*
	 * Return the "items" structural feature for JToolBar
	 */
	protected EStructuralFeature getSFItems() {
		if (sfItems == null) {
			JavaClass modelType = (JavaClass) ((RefObject) getTarget()).refMetaObject();
			sfItems = (EStructuralFeature) modelType.metaObject("items"); //$NON-NLS-1$
		}
		return sfItems;
	}

	protected IMethodProxy removeAllMethodProxy() {
		if (fRemoveAllMethodProxy == null) {
			fRemoveAllMethodProxy = getBeanProxy().getTypeProxy().getMethodProxy("removeAll"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return fRemoveAllMethodProxy;
	}

	protected void removeComponent(RefObject aComponent) {
		clearError(getSFItems(), aComponent);

		// The component to actually remove is within the ConstraintComponent too.
		IBeanProxyHost aComponentBeanProxyHost = BeanProxyUtilities.getBeanProxyHost((IJavaInstance) aComponent);

		// It is possible the component or ourselves didn't instantiate.  We then can't cancel it.
		if (aComponentBeanProxyHost.getErrorStatus() != IBeanProxyHost.ERROR_SEVERE
			&& getErrorStatus() != IBeanProxyHost.ERROR_SEVERE) {
			if (classComponent.isInstance(aComponent))
				BeanAwtUtilities.invoke_remove_Component(getBeanProxy(), aComponentBeanProxyHost.getBeanProxy());
			else if (classAction.isInstance(aComponent))
				BeanAwtUtilities.invoke_jtoolbar_remove_item_action(
					getBeanProxy(),
					aComponentBeanProxyHost.getBeanProxy());

			aComponentBeanProxyHost.releaseBeanProxy();
			// This is required because AWT will not invalidate and relayout the container
			revalidateBeanProxy();
		}
	}
	/* 
	 * Remove all items
	 */
	protected void removeItems() {
		if (getErrorStatus() == IBeanProxyHost.ERROR_SEVERE)
			return;

		removeAllMethodProxy().invokeCatchThrowableExceptions(getBeanProxy());
	}
	
	/**
	 * @see com.ibm.etools.jbcf.IBeanProxyHost#releaseBeanProxy()
	 */
	public void releaseBeanProxy() {
		super.releaseBeanProxy();
		if (fRemoveAllMethodProxy != null) {
			fRemoveAllMethodProxy.getProxyFactoryRegistry().releaseProxy(fRemoveAllMethodProxy);
			fRemoveAllMethodProxy = null;
		}
		if (fAddActionMethodProxy != null) {
			fAddActionMethodProxy.getProxyFactoryRegistry().releaseProxy(fAddActionMethodProxy);
			fAddActionMethodProxy = null;
		}
		
	}

}
