package com.ibm.etools.jbcf.visual.tab;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.*;

import org.eclipse.swt.widgets.Display;

import com.ibm.etools.cde.CDEHack;
import com.ibm.etools.java.instantiation.base.*;
import com.ibm.etools.jbcf.BeanProxyUtilities;
import com.ibm.etools.jbcf.visual.ComponentGraphicalEditPart;
import com.ibm.etools.proxy.*;

/**
 * Implementation of IOrderManager for a Swing container
 */
public class JTabOrderManager extends TabOrderManager {
	
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
public JTabOrderManager(ITabRootEditPart rootEditPart){
	super(rootEditPart);
}

/**
 * This will spawn off a thread to actually perform the generate.
 * This is necessary because SWT has focus changing problems if
 * we are on the UI thread. For Swing we must change the focus and
 * continue working after the focus has changed. For SWT, the focus
 * change doesn't actually occur until after the message queue is
 * processed, so we won't see the change until later if running in the UI thread.
 */
protected void generateTabOrderLines(final IJavaInstance containerBean) {
	Thread thread = new Thread(new Runnable() {
		public void run() {
			generateTabOrderLinesInThread(containerBean);
//			listenToBeans();
		}
	});
	// Reduce the thread priority by one (but not below min) so that current thread can continue to
	// run. (Normally the current thread is the UI thread, and we want that to run before this thread.
//	thread.setPriority(thread.getPriority() != Thread.MIN_PRIORITY ? thread.getPriority()-1 : Thread.MIN_PRIORITY);
	thread.start();
}
/**
 * Return a vector of component proxies.  This is based on using the TabOrderCalculator class
 * to get the actual tab order of the live components. It is very important that this
 * DOES NOT run in the IDE UI Thread. Focus changing will not work correctly if it is.
 */
private IArrayBeanProxy getComponentProxyOrder(IJavaInstance containerBean) {

	// Get the bean proxy for the frame
	final IBeanProxy jFrameProxy = BeanProxyUtilities.getBeanProxy(containerBean);
	final IMethodProxy requestFocus = jFrameProxy.getTypeProxy().getMethodProxy("requestFocus");

	// Now switch the focus over to the frame. This needs to be done in two steps,
	// 1) In UI Thread, switch the focus, then return
	// 2) Sleep for a tenth second to give it time to loose focus. This is a kludge
	//    but until the VM IDE goes away this is necessary to get the focus away
	//    from the IDE. There is nothing to check or wait upon for deactivation, so
	//    we need to just wait. (There is a listener on the Shell that can tell us
	//    when the shell is deactivated, but there is no way to find out if it
	//    currently is deactivated. So if we already had lost focus before we start
	//    waiting, we wouldn't know it and would wait forever).
	Display.getDefault().asyncExec(new Runnable() {
		public void run() {
			requestFocus.invokeCatchThrowableExceptions(jFrameProxy);
		}
	});

	// For now, kludge in a wait.
	try {
		Thread.currentThread().sleep(100);
	} catch (InterruptedException e) {
	}


	// Get the proxy to the class for TabOrderCalculator and then create a new instance passing in our JFrame bean proxy
	IBeanTypeProxy calcClass = jFrameProxy.getProxyFactoryRegistry().getBeanTypeProxyFactory().getBeanTypeProxy("com.ibm.etools.jbcf.visual.vm.TabOrderCalculator");
	IConstructorProxy constructor = calcClass.getConstructorProxy(new String[] {"javax.swing.JFrame"});
	final IBeanProxy calc = constructor.newInstanceCatchThrowableExceptions(new IBeanProxy[] {jFrameProxy});
	// Now we have the calculator use it to get the component order.  This will be an IArrayBeanProxy
	final IMethodProxy getComponentOrder = calcClass.getMethodProxy("getComponentOrder");
	return (IArrayBeanProxy) getComponentOrder.invokeCatchThrowableExceptions(calc);

}
/**
 * IBean is a JFrame container component.  We need to get its IBeanProxyHost and then use the TabOrderCalculator
 * class to get the actual component tab order from the live JFrame.  ( This is because unlike AWT which is Z-Order based
 * we have no prior knowledge of the tab order which is determined by the FocusManager ).
 * Having got the proxies to the component beans we need the view object for each one
 * To do this we create a hashtable keyed by bean proxy which a value of view object for all view objects that are
 * children ( immediate or nested ) of the frame's view object.  Even though the IBeanProxy objects might not be the
 * same ones the .equals and hash are overridden so that we can use the hashtable to order the view objects by
 * bean proxy component order.
 *
 * This MUST NOT be run from the UI Thread. Focus changing will cause problems if it is.
 */
private void generateTabOrderLinesInThread(IJavaInstance containerBean) {

	// Get a vector of that is a vector of vectors.  Each vector contains a tab order
	// This is because in Swing there can be multiple different roots to each series of tabbable components
	IArrayBeanProxy componentCycles = getComponentProxyOrder(containerBean);
	if (componentCycles == null)
		return; // There are no components to cycle through.
	Object veryFirstComponentEP = null;

	// Get a hashtable whose key is the bean proxy and whose value is the view object
	Hashtable proxyToViewObject = getProxyToViewObject();

	// Iterate over the component orders creating TabOrderLines for each line ( between two components )
	// Also create a view object for each TabOrderLine.  The first and last view objects are special
	for(int i=0;i<componentCycles.getLength();i++){
		Object c = componentCycles.getCatchThrowableException(i);
		IArrayBeanProxy components = (IArrayBeanProxy)c; 
		// Now iterate over the components in the cycle
		ComponentGraphicalEditPart fPreviousComponentInGroup = null;
		// For each new cycle make sure the previous component is not left pointing to a previous cycle
		fPreviousTabOrderLineConnectionEditPart = null;
		for(int j=0;j<components.getLength();j++){
			Object componentProxy = components.getCatchThrowableException(j);
			// Get the view object for the component from the hashtable
			ComponentGraphicalEditPart componentEP = (ComponentGraphicalEditPart) proxyToViewObject.get(componentProxy);
			// If firstComponent is null then this is the very first component.  Remember it
			if (veryFirstComponentEP == null)
				veryFirstComponentEP = componentEP;
			// If this is the first view object then we cannot create a first line ( unlike AWT )
			// because there is no way to define the first tabbable component in Swing
			if (fPreviousComponentInGroup == null) {
				// This is the first line in the group.  Set the index to 0
				fComponentIndex = 1;
			} else {
				// Viewobjects need to be created in the UI thread, so we will invokeAndWait in the UI thread.
				final ComponentGraphicalEditPart prevCompInGroup = fPreviousComponentInGroup;
				final ComponentGraphicalEditPart compEP = componentEP;
				final Object veryFirstEP = veryFirstComponentEP;
//				Display.getDefault().asyncExec(new Runnable() {
//					public void run() {
						// Create a tab order line from the last component to the current one
						// unless the current one is the very first component.  If this is the case this we draw a line
						/// from the current one to the frame and break the loop
						if (compEP == veryFirstEP && prevCompInGroup != null) {
							// Create a line from the last component to the frame itself
							createLastTabOrderLine(prevCompInGroup);
						} else {
							createTabOrderLine(prevCompInGroup, compEP);
						}
//					}
//				});
			}
			// Store a reference to the last component to receive focus
			fPreviousComponentInGroup = componentEP;
		}
	}

	// For swing we should listen to all of the beans, whether they are part of the TabOrder or not
	// This will be all IBeans for JComponents that have a graph view object
	// This is because we need to know when the attribute "components" , "visible" and "constraints" change
	// All of these have an affect on tab order
//	Enumeration enum = proxyToViewObject.elements();
//	while (enum.hasMoreElements()) {
//		ComponentGraphViewObject componentVO = (ComponentGraphViewObject) enum.nextElement();
//		fListenToBeans.addElement(componentVO.getBean());
		// For Swing we add input policies to each of the ComponentGraphViewObjects.  These are responsible for letting us
		// drag lines onto components ( which is how we re-order swing tab order lines )
//		fComponentViewObjects.addElement(componentVO);
//		componentVO.putInputPolicy("tab", new TabOrderLineNodeInputPolicy());
//	}
}
/**
 * Return a hashtable of IBeanProxy to EditPart
 */
protected Hashtable getProxyToViewObject() {

	Hashtable result = new Hashtable();
	writeProxyToEditPartWithChildren( (ComponentGraphicalEditPart)fTabRootEditPart , result );
	return result;
	
}	
/**
 * Get the bean proxy for the component graph view object and add it to the hashtable
 * Iterate over all children view objects and for any that are component view objects write them
 * and their children to the hashtable in an iterative fashion
 */
protected void writeProxyToEditPartWithChildren(ComponentGraphicalEditPart aComponentEP , Hashtable aHashtable){

	IBeanProxy beanProxy = BeanProxyUtilities.getBeanProxy(((ComponentGraphicalEditPart)aComponentEP).getBean());
 	aHashtable.put( beanProxy , aComponentEP );
 	// Iterate over the children view objects
	Iterator iter = aComponentEP.getChildren().iterator();
	while ( iter.hasNext() ) {
		Object childEP = iter.next();
		if ( childEP instanceof ComponentGraphicalEditPart ) {
			writeProxyToEditPartWithChildren( (ComponentGraphicalEditPart)childEP , aHashtable);
		}
	}
}
}

