package com.ibm.etools.jbcf.visual.vm;

import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JSplitPane;

import com.ibm.etools.jbcf.visual.common.Common;
import com.ibm.etools.proxy.common.*;
import com.ibm.etools.proxy.common.ICallback;
import com.ibm.etools.proxy.common.IVMServer;

/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
/**
 * This class manages the splitpane.
 * @author richkulp
 */
public class JSplitPaneManager implements ICallback {
	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	
	protected IVMServer fServer;
	protected int fCallbackID;	
	protected JSplitPane splitpane;
	protected static final int DIVIDER_NOT_SET = Integer.MIN_VALUE;
	protected int setDividerLocation = DIVIDER_NOT_SET;
	protected PropertyChangeListener dividerListener = new PropertyChangeListener() {
		boolean imSetting = false;	// Am I in the process of setting it here in the property change listener.
		/**
		 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
		 */
		public void propertyChange(PropertyChangeEvent evt) {
			boolean reallyIamSetting = false;
			synchronized (dividerListener) {
				reallyIamSetting = imSetting;
			}
			// If I'm (the propertyChange listener) setting it, I don't care to listen for that particular change, because
			// it either accepted or it didn't. If it accepted it, then it would match, and if it didn't accept it, it wouldn't
			// match. But if it didn't accept it, I don't want to go ahead and try to set it back because it would fail again and again.
			// A case of not being accepted is if divider location is outside the max/min divider location. The splitpane will always
			// reset it back.
			if (!reallyIamSetting && ((Integer) evt.getNewValue()).intValue() != setDividerLocation) {
				// It got changed to something else other than us explicitly setting it. This
				// occurs due to resizes, or any relayout. We want to maintain our set size.
				// Queue it off. We can't do it here because we can get into a deadlock
				Toolkit.getDefaultToolkit().getSystemEventQueue().invokeLater(new Runnable() {
					public void run() {
						synchronized (dividerListener) {
							imSetting = true;
						}
						try {
							splitpane.setDividerLocation(setDividerLocation);
							splitpane.revalidate();
							invalidateImage();	// Now tell it to re-get the image. This is because sometimes these changes happen after the image has already been captured.
						} finally {
							synchronized (dividerListener) {
								imSetting = false;
							}
						}
					}
				});
			}
		}
	};
	
	/**
	 * The listener initialize for callback server.
	 */
	public void initializeCallback(IVMServer server, int callbackID){
		fServer = server;
		fCallbackID = callbackID;
	}	
		
	public void setSplitPane(JSplitPane splitpane) {
		if (this.splitpane != null)
			this.splitpane.removePropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY, dividerListener);
			
		this.splitpane = splitpane;
	}
	
	public void setDividerLocation(int dividerLocation) {
		if (setDividerLocation != DIVIDER_NOT_SET && dividerLocation == -1) {
			// We're setting to default, and didn't had it set before, so remove our listener.
			splitpane.removePropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY, dividerListener);
		} else if (setDividerLocation == DIVIDER_NOT_SET && dividerLocation != -1) {
			// We're setting to something explicit, and we weren't set before, so add our listener
			splitpane.addPropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY, dividerListener);
		}
		
		if (dividerLocation == -1)
			setDividerLocation = DIVIDER_NOT_SET;
		else
			setDividerLocation = dividerLocation;
		
		splitpane.setDividerLocation(dividerLocation);
	}
	
	public void resetToPreferredSizes() {
		splitpane.resetToPreferredSizes();
	}
	
	/**
	 * Send back a invalidate image notification so that IDE can ask for a new image.
	 */
	public void invalidateImage() {
		if (fServer != null) {	
			try {
				fServer.doCallback(new ICallbackRunnable() {
					public Object run(ICallbackHandler handler) throws CommandException {
						return handler.callbackWithParms(fCallbackID, Common.JSP_INVALIDATE, null);						
					}
				});
			} catch (CommandException exp) {
			}
		}
	}	
	
}
