package com.ibm.etools.jbcf.wizard;
/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.*;

import com.ibm.etools.jbcf.JBCFPlugin;
import com.ibm.etools.logging.util.Level;

/**
 * Dynamic variable initializer to initialize variables and to copy over
 * files that are needed for a variable if the plugin is on a separate host.
 * Since the plugin could be on a different host, we can refer to directories
 * but can only refer to files. This is because we need to copy them over to
 * the local host for the classpath to find them.
 * 
 * It is instantiated from your plugin using the following format:
 * <pre>
 *     &lt;extension point = "org.eclipse.jdt.core.classpathVariableInitializer"&gt;
 *       &lt;classpathVariableInitializer
 *         variable="<it>your variable name</it>"
 *         &lt;class
 *           plugin="com.ibm.etools.jbcf"
 *           class="com.ibm.etools.jbcf.wizard.ClasspathVariablesInitializer"&gt;
 *             &lt;parameter name="variablepath" value="<it>your classpath variable root path wrt/your plugin</it>"/&gt;
 *             &lt;parameter name="file<it>nn</it>" value="<it>your file in the plugin wrt/variablepath</it>"/&gt; 
 *         &lt;/class&gt;
 *       &lt;classpathVariableInitializer&gt;
 *     &lt;/extension&gt;
 * </pre>
 * 
 * Parameter "variablepath" is only needed if you want the root of your variable to be a subdirectory from the root of your plugin.
 * 
 * Parameters "file<it>nn</it>" are the individual files that need to be referred to and copied if necessary. They
 * will be found relative to "variablepath" and will be stored locally relative to that path too. The order of "nn"
 * is not preserved; it is just used to make sure the file parameters are unique if more than one.
 */

public class ClasspathVariablesInitializer extends ClasspathVariableInitializer implements IExecutableExtension {
	
	/**
	 * @see org.eclipse.jdt.core.ClasspathVariableInitializer#initialize(String)
	 */
	public void initialize(String variable) {
		// First see if we are local plugin and not remote.
		try {
			URL install = Platform.resolve(configElement.getDeclaringExtension().getDeclaringPluginDescriptor().getInstallURL());
			IPath varValue = null;
			String varPath = null;
			if (configData != null)
				varPath = (String) configData.get("variablepath"); //$NON-NLS-1$
			if ("file".equals(install.getProtocol())) { //$NON-NLS-1$
				varValue = new Path(install.getFile());
				if (configData != null)
					if (varPath != null)
						varValue = varValue.append(varPath);	// Attach the extended root
			} else {
				// It is remote, so we need to copy them across into our metadata area.
				varValue = configElement.getDeclaringExtension().getDeclaringPluginDescriptor().getPlugin().getStateLocation();				
				if (varPath != null) {
					install = new URL(install, varPath);	// Actually copy from varPath wrt/install					
					varValue = varValue.append(varPath);	// Attach the extended root
				}
				String toDirectory = varValue.toOSString();
				
				if (configData != null) {
					Iterator itr = configData.entrySet().iterator();
					while (itr.hasNext()) {
						Map.Entry entry = (Map.Entry) itr.next();
						if (((String) entry.getKey()).startsWith("file")) { //$NON-NLS-1$
							String fileToCopy = (String) entry.getValue();
							copyFile(install, toDirectory, fileToCopy);							
						}
					}
				}
			}
			if (varValue != null && !varValue.equals(JavaCore.getClasspathVariable(variable))) {
				// It has changed so save it.
				JavaCore.setClasspathVariable(variable, varValue, new NullProgressMonitor());
			}				
		} catch (MalformedURLException e) {
		} catch (JavaModelException e) {
		} catch (IOException e) {
		} catch (CoreException e) {
		}
	}
	
	protected void copyFile(URL fromURL, String toDirectory, String filename) throws MalformedURLException, IOException {
		InputStream srcis = null;
		FileOutputStream tgtos = null;
		try {
			File tgtFile = new File(toDirectory, filename);
			tgtFile.getParentFile().mkdirs();
			tgtos = new FileOutputStream(tgtFile);

			URL jarurl = new URL(fromURL, filename);			
			srcis = jarurl.openStream();
			byte[] buf = new byte[1000];
			int count = srcis.read(buf);			
			
			while(count!=-1) {
				tgtos.write(buf,0,count);
				count = srcis.read(buf);
			}		
		} catch (FileNotFoundException e) {
			JBCFPlugin.getPlugin().getMsgLogger().log(e, Level.SEVERE);
		} finally {
			if (srcis != null) {
				try {
					srcis.close();
				} catch (IOException e) {
				}
			}
			if (tgtos != null) {
				try {
					tgtos.close();
				} catch (IOException e) {
				}
			}
		}
	}

	public static final String copyright = "(c) Copyright IBM Corporation 2002."; //$NON-NLS-1$
	protected IConfigurationElement configElement;
	protected Map configData;
	/**
	 * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(IConfigurationElement, String, Object)
	 */
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		configElement = config;
		if (data instanceof Map) {
			configData = (Map) data;
		}	
	}

}
